IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'adfCategoryUpdate')
	BEGIN
		PRINT 'Dropping Procedure adfCategoryUpdate'
		DROP  Procedure  adfCategoryUpdate
	END

GO

PRINT 'Creating Procedure adfCategoryUpdate'
GO
/******************************************************************************
**		File: 
**		Name: adfCategoryUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure adfCategoryUpdate
	@DocumentGuid TGuid,
	@ReferenceID TIdentifier,
	@Xml XML
AS
	DECLARE @result INT, @hdoc INT, @class TClass
	SET NOCOUNT ON

	SELECT @class = [Class] FROM [Class] WHERE [Guid] = @DocumentGuid

	EXEC @result = sp_xml_preparedocument @hdoc OUTPUT, @Xml
	IF @result != 0
		RETURN 1
	
	DECLARE categories CURSOR LOCAL FAST_FORWARD READ_ONLY FOR
	SELECT 
		[Name], [State]
	FROM 
		OPENXML(@hdoc, '/List/Item', 2)
			WITH (	[Name] nvarchar(256), 
					[State] int
					)

	OPEN categories
	DECLARE @name TName, @state INT
	
	FETCH NEXT FROM categories INTO @name, @state
	WHILE @@FETCH_STATUS = 0
	BEGIN
		IF @state = 2 -- added
		BEGIN
			IF EXISTS(SELECT * FROM [Category] WHERE 
				[FileAs] = @name AND [DocumentClass] = @class AND 
				(@ReferenceID IS NULL OR (@ReferenceID IS NOT NULL AND [ReferenceID] = @ReferenceID)))	
			BEGIN
				UPDATE 
					[Category] 
				SET 
					[State] = 1, 
					[Modified] = GETDATE(), 
					[ModifiedBy] = dbo.adfGetCurrentUserID() 
				WHERE 
					[FileAs] = @name 
					AND [DocumentClass] = @class 
					AND (@ReferenceID IS NULL OR (@ReferenceID IS NOT NULL AND [ReferenceID] = @ReferenceID))
			END
			ELSE
			BEGIN
				INSERT INTO [Category] ([State], [FileAs], [DocumentClass], [ReferenceID]) VALUES(1, @name, @class, @ReferenceID)
			END
		END
		ELSE IF @state = 3 -- deleted
		BEGIN
			IF EXISTS(SELECT * FROM [Category] WHERE [FileAs] = @name AND [DocumentClass] = @class AND 
				(@ReferenceID IS NULL OR (@ReferenceID IS NOT NULL AND [ReferenceID] = @ReferenceID)))
			BEGIN
				UPDATE 
					[Category] 
				SET 
					[State] = 2, 
					[Modified] = GETDATE(), 
					[ModifiedBy] = dbo.adfGetCurrentUserID() 
				WHERE 
					[FileAs] = @name 
					AND [DocumentClass] = @class 
					AND (@ReferenceID IS NULL OR (@ReferenceID IS NOT NULL AND [ReferenceID] = @ReferenceID))
			END
		END
		IF @@error != 0 GOTO ERROR_HANDLER
		FETCH NEXT FROM categories INTO @name, @state
	END
	
	EXEC sp_xml_removedocument @hdoc
	CLOSE categories
	DEALLOCATE categories
	
	RETURN 0
ERROR_HANDLER:
	EXEC sp_xml_removedocument @hdoc
	CLOSE categories
	DEALLOCATE categories
	
	RETURN 1
GO

GRANT EXEC ON adfCategoryUpdate TO PUBLIC
GO
