IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'adfDocumentContactUpdate')
	BEGIN
		PRINT 'Dropping Procedure adfDocumentContactUpdate'
		DROP  Procedure  adfDocumentContactUpdate
	END

GO

PRINT 'Creating Procedure adfDocumentContactUpdate'
GO
/******************************************************************************
**		File: 
**		Name: adfDocumentContactUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure adfDocumentContactUpdate
	@DocumentID TIdentifier,
	@DocumentClass TClass,
	@Xml XML
AS
	DECLARE @result INT, @doc INT

	SET NOCOUNT ON
	EXEC @result = sp_xml_preparedocument @doc OUTPUT, @Xml
	IF @result != 0
		RETURN 1


	BEGIN TRANSACTION
	
	DELETE [DocumentContact] WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		EXEC sp_xml_removedocument @doc
		RETURN 1
	END

	INSERT [DocumentContact]
		([DocumentID], [DocumentClass], [BusinessCardID], [BusinessCardClass])
	SELECT DISTINCT
		@DocumentID,
		@DocumentClass,
		Contactlist.[ID],
		Contactlist.[Class]
	FROM
		OPENXML(@doc, '/List/Item', 2)
			WITH (	
					[ID] int, 
					[Class] smallint
				 ) AS Contactlist INNER JOIN [Document] ON Contactlist.[ID] = [Document].[ID] AND Contactlist.[Class] = [Document].[Class]

	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		EXEC sp_xml_removedocument @doc
		RETURN 1
	END
			
	
	COMMIT TRANSACTION

	EXEC sp_xml_removedocument @doc
	
	RETURN 0
GO

GRANT EXEC ON adfDocumentContactUpdate TO PUBLIC
GO
