IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfContactFind]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfContactFind'
		DROP PROCEDURE [dbo].[adfContactFind]
	END
GO

PRINT 'Creating procedure adfContactFind'
GO

SET QUOTED_IDENTIFIER ON 
GO

/******************************************************************************
**		File: adfContactFind.sql
**		Name: adfContactFind
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Dmitry
**		Date: 15/08/2006 14:14:59
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		19.01.11	MAN				  .
**		15.12.11	PAM				   
**		27.06.12	MAB				    
**		14.08.12	SOV				     [Employee].[FileAs]
*******************************************************************************/
CREATE PROCEDURE [adfContactFind]
	@Parameters XML
AS
	SET NOCOUNT ON

	--     
	IF dbo.adfHasAccessRightByName('ViewAllContacts') = 0 AND dbo.adfHasAccessRightByName('ManageContacts') = 0
		RETURN 0

	DECLARE @doc xml,
			@Text nvarchar(256), -- NULL -  , 
			@TextField int, -- 0 - FileAs, -- 1 - FileAs, Comments
			@State int, -- NULL - 
			@Flag tinyint, -- NULL -  , 0-7 
			@UserOption int,
			@TimeField int, -- 0 - , 1 - Created, 2 - Modified
			@StartTime datetime,
			@EndTime datetime,
			@EmployeeID TIdentifier,
			@UserID TIdentifier,
			@MaskedText nvarchar(256)

	SET @doc = CAST(@Parameters as xml)
	
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @UserID = dbo.adfGetCurrentUserID()

	SELECT
		@Text = [dbo].[fnsiteprocEscapeValue](T.c.value('Text[1]', 'nvarchar(256)')),
		@TextField = T.c.value('TextField[1]', 'int'),
		@State = T.c.value('State[1]', 'int'),
		@Flag = T.c.value('Flag[1]', 'tinyint'),
		@UserOption = T.c.value('UserOption[1]', 'int'),
		@TimeField = T.c.value('TimeField[1]', 'int'),
		@StartTime = CAST(T.c.value('StartTime[1]', 'varchar(19)') as datetime),
		@EndTime = CAST(T.c.value('EndTime[1]', 'varchar(19)') as datetime)
	FROM
		@doc.nodes('/SearchParameters') T(c)
		
	SET @MaskedText = REPLACE(@Text, ' ', '')
	SET @MaskedText = REPLACE(@MaskedText, '-', '')
	SET @MaskedText = REPLACE(@MaskedText, '(', '')
	SET @MaskedText = REPLACE(@MaskedText, ')', '')
	SET @MaskedText = REPLACE(@MaskedText, '+', '')
	
	SELECT DISTINCT
		[Employee].[ID],
		[Employee].[Class],
		[Employee].[State],
		[Employee].[Flag],
		[Employee].[FileAs],
		[Company].[FileAs] as [Company],
		COALESCE([vwPhone].[#PhoneType#1], [vwPhone].[#PhoneType#2], [vwPhone].[#PhoneType#3], [vwPhone].[#PhoneType#4],
			[vwPhone].[#PhoneType#5], [vwPhone].[#PhoneType#6], [vwPhone].[#PhoneType#7], [vwPhone].[#PhoneType#8],
			[vwPhone].[#PhoneType#9], [vwPhone].[#PhoneType#10], [vwPhone].[#PhoneType#11]) as [Phone],
		[Employee].[Modified]
	FROM
		[Employee] 
		LEFT OUTER JOIN [Company]
			ON ([Employee].[CompanyID] = [Company].[ID])
		LEFT OUTER JOIN [Address]
			ON ([Employee].[ID] = [Address].[BusinessCardID] AND [Address].[BusinessCardClass] = dbo.adfGetClassByName('Adf.Employee'))
		LEFT OUTER JOIN [vwPhone]
			ON ([Employee].[Class]=[vwPhone].[BusinessCardClass] AND [Employee].[ID]=[vwPhone].[BusinessCardID])
		LEFT OUTER JOIN [vwEmail]
			ON ([Employee].[Class] = [vwEmail].[BusinessCardClass] AND [Employee].[ID] = [vwEmail].[BusinessCardID])
	WHERE
		(
			@Text IS NULL
			OR
			(@TextField = 0 AND [Employee].[FileAs] LIKE '%' + @Text + '%')
			OR
			(@TextField = 1 AND ([Employee].[FileAs] LIKE '%' + @Text + '%' OR [Employee].[Comments] LIKE '%' + @Text + '%'))
			OR
			(@TextField = 2 AND ([Employee].[FileAs] LIKE '%' + @Text + '%' OR [Company].[FileAs] LIKE '%' + @Text + '%' OR [Employee].[Comments] LIKE '%' + @Text + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#1]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#2]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#3]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#4]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#5]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#6]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#7]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#8]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#9]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#10]) LIKE '%' + @MaskedText + '%'
				OR [dbo].[fnPjm5GetCorrectPhone]([vwPhone].[#PhoneType#11]) LIKE '%' + @MaskedText + '%'
				OR [vwEmail].[#EmailType#1] LIKE '%' + @Text + '%'
				OR [vwEmail].[#EmailType#2] LIKE '%' + @Text + '%'
				OR [vwEmail].[#EmailType#3] LIKE '%' + @Text + '%'
				OR [vwEmail].[#EmailType#4] LIKE '%' + @Text + '%'
				OR ISNULL([Address].[Street], '') + ' ' + ISNULL([Address].[City], '') + ' ' + ISNULL([Address].[Region], '') + ' ' +
					ISNULL([Address].[PostalCode], '') + ' ' + ISNULL([Address].[Country], '') LIKE '%' + @Text + '%'
				))			
		)
		AND
		(
			@State IS NULL
			OR
			(@State IS NOT NULL AND [Employee].[State] = @State)
		)		
		AND
		(
			@Flag IS NULL
			OR
			(@Flag IS NOT NULL AND [Employee].[Flag] = @Flag)
		)
		AND
		(
			@UserOption IS NULL
			OR
			(@UserOption = 0 AND [Company].[ManagerEmployeeID] = @EmployeeID)
			OR
			(@UserOption = 1 AND ([Employee].[CreatedBy] = @UserID OR [Employee].[ModifiedBy] = @UserID))
			OR
			(@UserOption = 2 AND [Employee].[CreatedBy] = @UserID)
			OR
			(@UserOption = 3 AND [Employee].[ModifiedBy] = @UserID)
		)
		AND
		(
			@TimeField = 0
			OR
			(@TimeField = 1 AND [Employee].[Created] BETWEEN @StartTime AND @EndTime)
			OR
			(@TimeField = 2 AND [Employee].[Modified] BETWEEN @StartTime AND @EndTime)
		)
	ORDER BY
		[Employee].[FileAs]
	RETURN 0
GO

GRANT EXECUTE ON [adfContactFind] TO PUBLIC
GO
  