IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfEmployeeChangeState]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfEmployeeChangeState'
		DROP PROCEDURE [dbo].[adfEmployeeChangeState]
	END
GO

PRINT 'Creating procedure adfEmployeeChangeState'
GO

/******************************************************************************
**		File: adfEmployeeChangeState.sql
**		Name: adfEmployeeChangeState
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Dmitry.Kolchev
**		Date: 10/05/2004 18:09:19
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    26.01.10		syv				   ,  
**									   
**    14.05.10		SYV				  pjm5AllUserReminderDismiss
**    14.05.10		SYV				  FolderNotificationSettings
**	20.01.11		MAN			  
*******************************************************************************/
CREATE PROCEDURE adfEmployeeChangeState
	@ID TIdentifier,
	@ToState TState,
	@Forced TBool
AS
	DECLARE @FromState TState,
			@TransitionTemplateID TIdentifier,
			@RightID TIdentifier,
			@Class TClass,
			@AssignmentID TIdentifier,
			@UserID TIdentifier,
			@Own TBool,
			@rv int,
			@ProjectID TIdentifier
	
	SET NOCOUNT ON
	
	SELECT
		@Class = [Employee].[Class],
		@FromState = [Employee].[State],
		@UserID = [Employee].[UserID],
		@Own = [Company].[Own]
	FROM
		[Employee]
		LEFT OUTER JOIN [Company]
			ON [Company].[ID] = [Employee].[CompanyID] 
	WHERE
		[Employee].[ID] = @ID
		
	IF @Own = 1 AND dbo.adfHasAccessRightByName('ManageEmployees') = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	IF @Own = 0 AND dbo.adfHasAccessRightByName('ManageContacts') = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
		
	IF @FromState = @ToState
	BEGIN
		RAISERROR('#invalidargument', 18, -1)
		RETURN 1
	END
	
	SELECT
		@TransitionTemplateID = [ID],
		@RightID = [RightID]
	FROM
		[TransitionTemplate]
	WHERE
		[TransitionTemplate].[FromState] = @FromState AND [TransitionTemplate].[ToState] = @ToState
	AND
		[TransitionTemplate].[Class] = @Class
		
	IF @TransitionTemplateID IS NULL
	BEGIN
		RAISERROR('#invalidoperation', 18, -1)
		RETURN 1
	END
	
	SET @TransitionTemplateID = dbo.adfCheckTransitionRights(@TransitionTemplateID, @RightID, dbo.adfGetCurrentUserID())
	
	IF @TransitionTemplateID IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	DECLARE assignments CURSOR LOCAL SCROLL FOR
		SELECT 
			[vwAssignment].[ID] 
		FROM 
			[vwAssignment] 
			INNER JOIN [Project]
				ON [Project].[ID] = [vwAssignment].[ProjectID]
		WHERE 
			[vwAssignment].[EmployeeID] = @ID AND 
			[vwAssignment].[State] = 1 AND
			[Project].[State] != 4
	BEGIN TRANSACTION

	IF @FromState = 1 AND @ToState = 2
	BEGIN
		OPEN assignments
		
		FETCH NEXT FROM assignments INTO @AssignmentID
		IF @@FETCH_STATUS = 0 AND @Forced = 0
		BEGIN
			RAISERROR('?EmployeeWillBeExcludedFromAllProjects', 18, -1)
			ROLLBACK TRANSACTION
			RETURN 1
		END
		
		WHILE @@FETCH_STATUS = 0
		BEGIN	
			SELECT
				@ProjectID = [ProjectID]
			FROM
				[vwAssignment]
			WHERE
				[ID] = @AssignmentID
			IF EXISTS (SELECT * 
						FROM
							[TaskAssignment]
							INNER JOIN [Task]
								ON [Task].[ID] = [TaskAssignment].[TaskID]
								AND [Task].[ProjectID] = @ProjectID
							INNER JOIN [TimeEntry]
								ON [TaskAssignment].[AssignmentID] = [TimeEntry].[AssignmentID]
								AND [TaskAssignment].[TaskID] = [TimeEntry].[TaskID]
						WHERE [TimeEntry].[State] IN (1,2,5)
							AND [TaskAssignment].[AssignmentID] = @AssignmentID)
			BEGIN
				RAISERROR('#EmployeeWithNotApprovedTimeDismissal', 18, -1)
				ROLLBACK TRANSACTION
				RETURN 1	
			END
			FETCH NEXT FROM assignments INTO @AssignmentID
		
		END
		
		FETCH FIRST FROM assignments INTO @AssignmentID
		
		
		
		WHILE @@FETCH_STATUS = 0
		BEGIN
			EXEC @rv = pjm5AssignmentChangeState @AssignmentID,	2
			IF @@error != 0 OR @rv != 0
			BEGIN
				CLOSE assignments
				DEALLOCATE assignments
				ROLLBACK TRANSACTION
				RETURN 1
			END
			FETCH NEXT FROM assignments INTO @AssignmentID
		END
		CLOSE assignments
		DEALLOCATE assignments
		
		DELETE FROM [UserReminder] WHERE [UserID] = @UserID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

		DELETE FROM [SubscriptionRule] WHERE [UserID] = @UserID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
		
		DECLARE @ReminderID TIdentifier
		SELECT 
			@ReminderID = [ID]
		FROM 
			[Reminder]
		WHERE 
			[DocumentClass] = @Class
			AND [DocumentID] = @ID 
			
		IF @ReminderID IS NOT NULL
		BEGIN
			EXEC pjm5AllUserReminderDismiss @ReminderID
		END
		
		DELETE FROM [FolderNotificationSettings] WHERE [TrusteeID] = @UserID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

	END
		
	UPDATE [Employee]
	SET
		[Employee].[State] = @ToState,
		[Employee].[Modified] = DEFAULT,
		[Employee].[ModifiedBy] = DEFAULT
	WHERE
		[Employee].[ID] = @ID
	AND
		[Employee].[State] = @FromState
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	IF @UserID IS NOT NULL
	BEGIN
		IF @ToState = 1
			UPDATE [User] SET [Locked]=0 WHERE [ID] = @UserID
		ELSE
			UPDATE [User] SET [Locked]=1 WHERE [ID] = @UserID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END

	UPDATE [BusinessCard]
	SET
		[State] = @ToState,
		[Modified] = DEFAULT,
		[ModifiedBy] = DEFAULT
	WHERE
		[ID] = @ID AND [Class] = @Class
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON adfEmployeeChangeState TO PUBLIC
GO
 