IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CompanyID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [CompanyID] TIdentifier NULL
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'FolderNotificationSettings') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: FolderNotificationSettings                            */
	/*==============================================================*/
	create table FolderNotificationSettings (
	   TrusteeID            TIdentifier          not null,
	   FolderID             TIdentifier          not null,
	   FolderSettings       TBitMask             not null,
	   CardSettings         TBitMask             not null,
	   FileSettings         TBitMask             not null,
	   constraint FolderNotificationSettingsPK primary key  (TrusteeID, FolderID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'CaseOfFolder') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: CaseOfFolder                                          */
	/*==============================================================*/
	create table CaseOfFolder (
	   FolderID             TIdentifier          not null,
	   CaseID               TIdentifier          not null,
	   Allowed              TBool                not null default 0,
	   MandatoryFields      xml                  null,
	   DefaultValues        xml                  null,
	   constraint CaseOfFolderPK primary key  (FolderID, CaseID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentSendType') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentSendType                                      */
	/*==============================================================*/
	create table DocumentSendType (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   Code                 TCode                not null,
	   FileAs               TName                not null,
	   Comments             TNotes               null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Timestamp            timestamp            not null,
	   constraint DocumentSendTypePK primary key  (ID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'UserFavoriteFolder') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: UserFavoriteFolder                                    */
	/*==============================================================*/
	create table UserFavoriteFolder (
	   UserID               TIdentifier          not null,
	   FolderID             TIdentifier          not null,
	   FileAs               TName                not null,
	   AddSubfolders        TInt32               not null default 0,
	   constraint UserFavoriteFolderPK primary key  (UserID, FolderID)
	)
go

-- HAVE TO DELETE [DocumentCard].[DocumentItemID] COLUMN AFTER DATA UPGRADE AND DROP Incoming andOutgoing Cards
IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentItemID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [IncomingCardID] TIdentifier NULL'
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [OutgoingCardID] TIdentifier NULL'

	EXEC sp_executesql N'
	INSERT INTO [Case] 
		([State], 
		[Flag], 
		[Code],
		[FileAs],
		[CompanyID],
		[EmployeeID],
		[StartDate],
		[EndDate],
		[NumberTemplate],
		[CurrentNumber],
		[Increment],
		[AllowInternal],
		[AllowIncoming],
		[AllowOutgoing],
		[Comments])
	SELECT
		1, 1, '''', '' '', [Folder].[CompanyID], NULL, ''20070101'', NULL,
		''{NUMBER}'', 1, 1, 0, 1, 0, NULL
	FROM
		[Folder]
	WHERE
		[Folder].[CompanyID] IS NOT NULL AND
		NOT EXISTS(SELECT * FROM [Case] c WHERE c.[CompanyID] = [Folder].[CompanyID] AND c.[AllowIncoming] = 1)
	
	INSERT INTO [Case] 
		([State], 
		[Flag], 
		[Code],
		[FileAs],
		[CompanyID],
		[EmployeeID],
		[StartDate],
		[EndDate],
		[NumberTemplate],
		[CurrentNumber],
		[Increment],
		[AllowInternal],
		[AllowIncoming],
		[AllowOutgoing],
		[Comments])
	SELECT
		1, 1, '''', '' '', [Folder].[CompanyID], NULL, ''20070101'', NULL,
		''{NUMBER}'', 1, 1, 0, 0, 1, NULL
	FROM
		[Folder]
	WHERE
		[Folder].[CompanyID] IS NOT NULL AND
		NOT EXISTS(SELECT * FROM [Case] c WHERE c.[CompanyID] = [Folder].[CompanyID] AND c.[AllowOutgoing] = 1)

	INSERT INTO [Case]
		([State], 
		[Flag], 
		[Code],
		[FileAs],
		[CompanyID],
		[EmployeeID],
		[StartDate],
		[EndDate],
		[NumberTemplate],
		[CurrentNumber],
		[Increment],
		[AllowInternal],
		[AllowIncoming],
		[AllowOutgoing],
		[Comments])
	SELECT DISTINCT
		1, 1, [DocumentCardClass].[Code], [DocumentCardClass].[FileAs], [DocumentCard].[CompanyID], NULL, ''20070101'', NULL,
		''{NUMBER}'', 1, 1, 1, 0, 0, NULL
	FROM
		[DocumentCard]
		INNER JOIN [DocumentCardClass]
			ON [DocumentCardClass].[ID] = [DocumentCard].[DocumentCardClassID]
	WHERE
		[DocumentCard].[CompanyID] IS NOT NULL AND
		[DocumentCard].[CaseID] IS NULL
	
	ALTER TABLE [DocumentCard] DISABLE TRIGGER ALL
	UPDATE
		[DocumentCard]
	SET
		[CaseID] = [Case].[ID]
	FROM
		[DocumentCard]
		INNER JOIN [DocumentCardClass]
			ON [DocumentCardClass].[ID] = [DocumentCard].[DocumentCardClassID]
		INNER JOIN [Case]
			ON [Case].[CompanyID] = [DocumentCard].[CompanyID] AND
				[Case].[Code] = [DocumentCardClass].[Code] AND
				[Case].[FileAs] = [DocumentCardClass].[FileAs] AND
				[Case].[AllowInternal] = 1
	WHERE
		[DocumentCard].[CaseID] IS NULL
	ALTER TABLE [DocumentCard] ENABLE TRIGGER ALL

	ALTER TABLE [IncomingDocument] DISABLE TRIGGER ALL
	UPDATE
		[IncomingDocument]
	SET
		[CaseID] = [Case].[ID]
	FROM
		[IncomingDocument]
		INNER JOIN [Case]
			ON [Case].[CompanyID] = [IncomingDocument].[CompanyID] AND
				[Case].[AllowIncoming] = 1
	WHERE
		[IncomingDocument].[CaseID] IS NULL
	ALTER TABLE [IncomingDocument] ENABLE TRIGGER ALL

	ALTER TABLE [OutgoingDocument] DISABLE TRIGGER ALL
	UPDATE
		[OutgoingDocument]
	SET
		[CaseID] = [Case].[ID]
	FROM
		[OutgoingDocument]
		INNER JOIN [Case]
			ON [Case].[CompanyID] = [OutgoingDocument].[CompanyID] AND
				[Case].[AllowOutgoing] = 1
	WHERE
		[OutgoingDocument].[CaseID] IS NULL
	ALTER TABLE [OutgoingDocument] ENABLE TRIGGER ALL'

	EXEC sp_executesql N'
	INSERT INTO [DocumentCard]
		([State], 
		[Flag], 
		[FileAs], 
		[DocumentCardClassID],
		[DocumentItemID],
		[Number],
		[Date],
		[NumberIn],
		[DateIn],
		[NumberOut],
		[DateOut],
		[CompanyID],
		[ContactID],
		[CorrespondentCompanyID],
		[CorrespondentContactID],
		[DivisionID],
		[CaseID],
		[CardOwnerID],
		[Package],
		[Copy],
		[AppendixCount],
		[PageCount],
		[StorePeriod], 
		[CustodyPlaceID],
		[Comments],
		[IncomingCardID])
	SELECT
		1, 
		[Flag], 
		[FileAs], 
		[DocumentCardClassID],
		[DocumentItemID],
		[Number],
		[Date],
		[NumberIn],
		[DateIn],
		[NumberOut],
		[DateOut],
		[CompanyID],
		[ContactID],
		[CorrespondentCompanyID],
		[CorrespondentContactID],
		[DivisionID],
		[CaseID],
		[CardOwnerID],
		[Package],
		[Copy],
		[AppendixCount],
		[PageCount],
		[StorePeriod], 
		[CustodyPlaceID],
		[Comments],
		[ID]
	FROM
		[IncomingDocument]
	
	INSERT INTO [DocumentCard]
		([State], 
		[Flag], 
		[FileAs], 
		[DocumentCardClassID],
		[DocumentItemID],
		[Number],
		[Date],
		[NumberIn],
		[DateIn],
		[NumberOut],
		[DateOut],
		[CompanyID],
		[ContactID],
		[CorrespondentCompanyID],
		[CorrespondentContactID],
		[DivisionID],
		[CaseID],
		[CardOwnerID],
		[Package],
		[Copy],
		[AppendixCount],
		[PageCount],
		[StorePeriod], 
		[CustodyPlaceID],
		[Comments],
		[OutgoingCardID])
	SELECT
		1, 
		[Flag], 
		[FileAs], 
		[DocumentCardClassID],
		[DocumentItemID],
		[Number],
		[Date],
		[NumberIn],
		[DateIn],
		[NumberOut],
		[DateOut],
		[CompanyID],
		[ContactID],
		[CorrespondentCompanyID],
		[CorrespondentContactID],
		[DivisionID],
		[CaseID],
		[CardOwnerID],
		[Package],
		[Copy],
		[AppendixCount],
		[PageCount],
		[StorePeriod], 
		[CustodyPlaceID],
		[Comments],
		[ID]
	FROM
		[OutgoingDocument]'
		
	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 2
	FROM
		[DocumentCard]
		INNER JOIN [IncomingDocument]
			ON [IncomingDocument].[ID] = [DocumentCard].[IncomingCardID]
	WHERE
		[IncomingDocument].[State] > 1'

	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 3
	FROM
		[DocumentCard]
		INNER JOIN [IncomingDocument]
			ON [IncomingDocument].[ID] = [DocumentCard].[IncomingCardID]
	WHERE
		[IncomingDocument].[State] > 2'
	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 4
	FROM
		[DocumentCard]
		INNER JOIN [IncomingDocument]
			ON [IncomingDocument].[ID] = [DocumentCard].[IncomingCardID]
	WHERE
		[IncomingDocument].[State] = 4'

	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 2
	FROM
		[DocumentCard]
		INNER JOIN [OutgoingDocument]
			ON [OutgoingDocument].[ID] = [DocumentCard].[OutgoingCardID]
	WHERE
		[OutgoingDocument].[State] > 1'

	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 3
	FROM
		[DocumentCard]
		INNER JOIN [OutgoingDocument]
			ON [OutgoingDocument].[ID] = [DocumentCard].[OutgoingCardID]
	WHERE
		[OutgoingDocument].[State] > 2'
	EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 4
	FROM
		[DocumentCard]
		INNER JOIN [OutgoingDocument]
			ON [OutgoingDocument].[ID] = [DocumentCard].[OutgoingCardID]
	WHERE
		[OutgoingDocument].[State] = 4'

	EXEC sp_executesql N'
		UPDATE
			[DocumentCard]
		SET
			[State] = 3
		FROM
			[DocumentCard]
			LEFT OUTER JOIN [DocumentItem]
				ON [DocumentCard].[DocumentItemID] = [DocumentItem].[ID]
		WHERE
			[DocumentCard].[State] = 4'

	EXEC sp_executesql N'
		UPDATE
			[Attachment]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[Attachment]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [Attachment].[DocumentClass] AND
					[IncomingDocument].[ID] = [Attachment].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]
	'

	EXEC sp_executesql N'
		UPDATE
			[Attachment]
		SET
			[LinkDocumentClass] = [DocumentCard].[Class],
			[LinkDocumentID] = [DocumentCard].[ID]
		FROM
			[Attachment]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [Attachment].[LinkDocumentClass] AND
					[IncomingDocument].[ID] = [Attachment].[LinkDocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]
	'

	EXEC sp_executesql N'
		UPDATE
			[Attachment]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[Attachment]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [Attachment].[DocumentClass] AND
					[OutgoingDocument].[ID] = [Attachment].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]
	'

	EXEC sp_executesql N'
		UPDATE
			[Attachment]
		SET
			[LinkDocumentClass] = [DocumentCard].[Class],
			[LinkDocumentID] = [DocumentCard].[ID]
		FROM
			[Attachment]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [Attachment].[LinkDocumentClass] AND
					[OutgoingDocument].[ID] = [Attachment].[LinkDocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]
	'

	EXEC sp_executesql N'
		UPDATE
			[DocumentCategory]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentCategory]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [DocumentCategory].[DocumentClass] AND
					[IncomingDocument].[ID] = [DocumentCategory].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]'
	
	EXEC sp_executesql N'
		UPDATE
			[DocumentCategory]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentCategory]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [DocumentCategory].[DocumentClass] AND
					[OutgoingDocument].[ID] = [DocumentCategory].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]'
	
	EXEC sp_executesql N'ALTER TABLE [Category] DISABLE TRIGGER ALL'
	EXEC sp_executesql N'
		DECLARE @DocumentCard int, @IncomingDocument int, @OutgoingDocument int

		SELECT @DocumentCard = [Class] FROM [DocumentCard]
		SELECT @IncomingDocument = [Class] FROM [IncomingDocument]
		SELECT @OutgoingDocument = [Class] FROM [OutgoingDocument]

		UPDATE 
			[Category]
		SET
			[DocumentClass] = @DocumentCard
		FROM
			[Category]
		WHERE
			[DocumentClass] = @IncomingDocument OR
			[DocumentClass] = @OutgoingDocument
	'
	EXEC sp_executesql N'ALTER TABLE [Category] ENABLE TRIGGER ALL'

	EXEC sp_executesql N'
		UPDATE
			[DocumentContact]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentContact]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [DocumentContact].[DocumentClass] AND
					[IncomingDocument].[ID] = [DocumentContact].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]	'
			
	EXEC sp_executesql N'
		UPDATE
			[DocumentContact]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentContact]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [DocumentContact].[DocumentClass] AND
					[OutgoingDocument].[ID] = [DocumentContact].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]	'

	EXEC sp_executesql N'
		UPDATE
			[DocumentFlowEntry]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentFlowEntry]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [DocumentFlowEntry].[DocumentClass] AND
					[IncomingDocument].[ID] = [DocumentFlowEntry].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]'
				
	EXEC sp_executesql N'
		UPDATE
			[DocumentFlowEntry]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[DocumentFlowEntry]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [DocumentFlowEntry].[DocumentClass] AND
					[OutgoingDocument].[ID] = [DocumentFlowEntry].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]'

	EXEC sp_executesql N'ALTER TABLE [MailItem] DISABLE TRIGGER ALL'
	EXEC sp_executesql N'
		UPDATE
			[MailItem]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[MailItem]
			INNER JOIN [IncomingDocument]
				ON [IncomingDocument].[Class] = [MailItem].[DocumentClass] AND
					[IncomingDocument].[ID] = [MailItem].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[IncomingCardID] = [IncomingDocument].[ID]
	'
	EXEC sp_executesql N'
		UPDATE
			[MailItem]
		SET
			[DocumentClass] = [DocumentCard].[Class],
			[DocumentID] = [DocumentCard].[ID]
		FROM
			[MailItem]
			INNER JOIN [OutgoingDocument]
				ON [OutgoingDocument].[Class] = [MailItem].[DocumentClass] AND
					[OutgoingDocument].[ID] = [MailItem].[DocumentID]
			INNER JOIN [DocumentCard]
				ON [DocumentCard].[OutgoingCardID] = [OutgoingDocument].[ID]
	'
	EXEC sp_executesql N'ALTER TABLE [MailItem] ENABLE TRIGGER ALL'

	EXEC sp_executesql N'ALTER TABLE [DocumentCard] DROP COLUMN [IncomingCardID]'
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] DROP COLUMN [OutgoingCardID]'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'FolderID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [FolderID] TIdentifier null'

	EXEC sp_executesql N'
	ALTER TABLE [DocumentCard] DISABLE TRIGGER ALL
	UPDATE
		[DocumentCard]
	SET
		[FolderID] = [DocumentItem].[FolderID]
	FROM
		[DocumentCard]
		INNER JOIN [DocumentItem]
			ON [DocumentItem].[ID] = [DocumentCard].[DocumentItemID]

	UPDATE
		[DocumentCard]
	SET
		[FolderID] = 1
	WHERE
		[FolderID] IS NULL
	ALTER TABLE [DocumentCard] ENABLE TRIGGER ALL'

	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ALTER COLUMN [FolderID] TIdentifier not null'
END
GO
-- HAVE TO DELETE [DocumentCard].[DocumentItemID] COLUMN AFTER DATA UPGRADE

IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Date' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ALTER COLUMN [Date] TDate null
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentCardID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [DocumentCardID] TIdentifier null
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentSendTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [DocumentSendTypeID] TIdentifier null
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SentTime' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [SentTime] TDateTime null
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentCardID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentItem')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [DocumentItem] ADD [DocumentCardID] TIdentifier null'
	
	EXEC sp_executesql N'
	ALTER TABLE [DocumentItem] DISABLE TRIGGER ALL
	UPDATE
		[DocumentItem]
	SET
		[DocumentCardID] = [DocumentCard].[ID]
	FROM
		[DocumentItem]
		INNER JOIN [DocumentCard]
			ON [DocumentItem].[ID] = [DocumentCard].[DocumentItemID]
	ALTER TABLE [DocumentItem] ENABLE TRIGGER ALL'
	
	--EXEC sp_executesql N'ALTER TABLE [DocumentCard] DROP COLUMN [DocumentItemID]'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Code' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [Code] TCode null
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CompanyID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [CompanyID] TIdentifier null
GO

-- HAVE TO DELETE [DocumentCard].[DocumentItemID] COLUMN AFTER DATA UPGRADE AND DROP Incoming andOutgoing Cards
IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentItemID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Folder] ADD [SourceFolderID] TIdentifier'
	EXEC sp_executesql N'
	DECLARE @UserID int
	SET @UserID = dbo.adfGetCurrentUserID()

	INSERT INTO [Folder]
		([State], [Flag], [CompanyID], [FolderID], [Code], [FileAs], [VersionControl], [PublicationControl], [Parameters], [SourceFolderID])
	SELECT DISTINCT
		1, 0, [Company].[ID], NULL, NULL, [Company].[FileAs], 1, 0, NULL, 1
	FROM
		[Company]
		INNER JOIN [DocumentCard]
			ON [DocumentCard].[CompanyID] = [Company].[ID]

	INSERT INTO [FolderAccessControl] ([FolderID], [TrusteeID], [AccessRights]) 
	SELECT 
		[ID], @UserID, 0x333F
	FROM
		[Folder]
	WHERE
		[CompanyID] IS NOT NULL

		
	INSERT INTO [FolderAccessControl] ([FolderID], [TrusteeID], [AccessRights]) 
	SELECT 
		[ID], dbo.adfGetTrusteeID(''Everyone''), 0x0022
	FROM
		[Folder]
	WHERE
		[CompanyID] IS NOT NULL	
	DECLARE @Folders TABLE(ID int);
	DECLARE @row_count int
	SET @row_count = 1
	WHILE @row_count > 0
	BEGIN
		DELETE FROM @Folders;
		INSERT INTO [Folder]
			([State], [Flag], [CompanyID], [FolderID], [Code], [FileAs], [VersionControl], [PublicationControl], [Parameters], [SourceFolderID])
		OUTPUT INSERTED.[ID] INTO @Folders
		SELECT
			[Folder].[State], 
			[Folder].[Flag], 
			[CompanyFolder].[CompanyID], 
			[CompanyFolder].[ID], 
			[Folder].[Code], 
			[Folder].[FileAs], 
			[Folder].[VersionControl], 
			[Folder].[PublicationControl], 
			[Folder].[Parameters], 
			[Folder].[ID]
		FROM
			[Folder]
			INNER JOIN [Folder] AS [CompanyFolder]
				ON [Folder].[FolderID] = [CompanyFolder].[SourceFolderID]
		WHERE
			NOT EXISTS (SELECT * FROM [Folder] f WHERE f.[SourceFolderID] = [Folder].[ID])

		SET @row_count = @@ROWCOUNT
		IF @row_count > 0
			INSERT INTO [FolderAccessControl] ([FolderID], [TrusteeID], [AccessRights]) SELECT [ID], @UserID, 0x333F FROM @Folders
	END'
	
	EXEC sp_executesql N'
	ALTER TABLE [DocumentCard] DISABLE TRIGGER ALL
	UPDATE
		[DocumentCard]
	SET
		[FolderID] = [Folder].[ID]
	FROM
		[DocumentCard]
		INNER JOIN [Folder]
			ON ([DocumentCard].[FolderID] = [Folder].[SourceFolderID] AND
				[DocumentCard].[CompanyID] = [Folder].[CompanyID])
	ALTER TABLE [DocumentCard] ENABLE TRIGGER ALL

	ALTER TABLE [DocumentItem] DISABLE TRIGGER ALL
	UPDATE
		[DocumentItem]
	SET
		[FolderID] = [DocumentCard].[FolderID]
	FROM
		[DocumentItem]
		INNER JOIN [DocumentCard]
			ON [DocumentItem].[DocumentCardID] = [DocumentCard].[ID]
	ALTER TABLE [DocumentItem] ENABLE TRIGGER ALL

	UPDATE
		[DocumentItem]
	SET
		[State] = CASE WHEN [DocumentItem].[State] = 1 AND [DocumentCard].[CaseID] IS NOT NULL THEN 2 ELSE [DocumentItem].[State] END
	FROM
		[DocumentItem]
		INNER JOIN [DocumentCard]
			ON [DocumentItem].[DocumentCardID] = [DocumentCard].[ID]'
			
	EXEC sp_executesql N'
	DECLARE @Folders TABLE(ID int);
	DECLARE @row_count int
	SET @row_count = 1
	WHILE @row_count > 0
	BEGIN
		DELETE FROM @Folders;
		DELETE 
			[Folder] 
		OUTPUT DELETED.[ID] INTO @Folders
		WHERE 
			[CompanyID] IS NOT NULL AND [SubfolderCount] = 0 AND
			NOT EXISTS (SELECT * FROM [DocumentCard] WHERE [DocumentCard].[FolderID] = [Folder].[ID])
		SET @row_count = @@ROWCOUNT
		
		IF @row_count > 0
			DELETE [FolderAccessControl]
			FROM 
				[FolderAccessControl]
				INNER JOIN @Folders AS [Folders]
					ON [FolderAccessControl].[FolderID] = [Folders].[ID]
	END'
	
	EXEC sp_executesql N'
	DELETE [FolderAccessControl]
	FROM
		[FolderAccessControl]
		INNER JOIN [Folder]
			ON [Folder].[ID] = [FolderAccessControl].[FolderID]
	WHERE
		[Folder].[FolderID] IS NOT NULL AND
		[Folder].[CompanyID] IS NOT NULL AND
		[Folder].[SourceFolderID] IS NOT NULL
		
	INSERT INTO [FolderAccessControl] ([FolderID], [TrusteeID], [AccessRights])
	SELECT
		[Folder].[ID],
		[SourceFolderAccess].[TrusteeID],
		[SourceFolderAccess].[AccessRights]
	FROM
		[Folder]
		INNER JOIN [FolderAccessControl] AS [SourceFolderAccess]
			ON [SourceFolderAccess].[FolderID] = [Folder].[SourceFolderID]
	WHERE
		[Folder].[FolderID] IS NOT NULL AND
		[Folder].[CompanyID] IS NOT NULL'
			
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] DROP COLUMN [DocumentItemID]'
	EXEC sp_executesql N'ALTER TABLE [Folder] DROP COLUMN [SourceFolderID]'
END

INSERT INTO [CaseOfFolder] ([CaseID], [FolderID])
SELECT 
	[Case].[ID], [Folder].[ID]
FROM
	[Case]
	INNER JOIN [Folder]
		ON [Folder].[CompanyID] = [Case].[CompanyID]
WHERE
	NOT EXISTS(
		SELECT * 
		FROM [CaseOfFolder] c 
		WHERE 
			c.[CaseID] = [Case].[ID] AND
			c.[FolderID] = [Folder].[ID]
	)
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'IncomingDocument') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	UPDATE [IncomingDocument] SET [State] = 3 WHERE [State] = 4		
	UPDATE [IncomingDocument] SET [State] = 1 WHERE [State] = 3
	DELETE [IncomingDocument]
	DROP TABLE [IncomingDocument]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'OutgoingDocument') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	UPDATE [OutgoingDocument] SET [State] = 3 WHERE [State] = 4		
	UPDATE [OutgoingDocument] SET [State] = 1 WHERE [State] = 3
	DELETE [OutgoingDocument]
	DROP TABLE [OutgoingDocument]
END
GO

/*
-- HAVE TO DELETE SECURITY INFO
SELECT * FROM 
	INFORMATION_SCHEMA.COLUMNS 
	INNER JOIN INFORMATION_SCHEMA.TABLES
		ON INFORMATION_SCHEMA.TABLES.TABLE_NAME = INFORMATION_SCHEMA.COLUMNS.TABLE_NAME AND
			INFORMATION_SCHEMA.TABLES.TABLE_TYPE = 'BASE TABLE'
WHERE [COLUMN_NAME] = 'Class' AND [COLUMN_DEFAULT] IS NULL
*/
GO

EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 3
	FROM
		[DocumentCard]
	WHERE
		[DocumentCard].[CaseID] IS NULL AND
		[DocumentCard].[State] = 4'

GO
EXEC sp_executesql N'
	UPDATE
		[DocumentCard]
	SET
		[State] = 1
	FROM
		[DocumentCard]
	WHERE
		[DocumentCard].[CaseID] IS NULL AND
		[DocumentCard].[State] IN (2, 3)'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'AllowFolder' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Case')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Case] ADD [AllowFolder] TBool NOT NULL DEFAULT 0'
	EXEC sp_executesql N'UPDATE [Case] SET [AllowFolder] = 1'
END
GO

UPDATE [DocumentCard] SET [State] = 1 WHERE [State] = 5
GO

IF NOT EXISTS(SELECT * FROM [Folder] WHERE [ID] = -1)
BEGIN
	SET IDENTITY_INSERT [Folder] ON
	INSERT INTO [Folder] ([ID], [State], [Flag], [FileAs], [VersionControl]) VALUES (-1, 1, 0, 'Bin', 0) 
	SET IDENTITY_INSERT [Folder] OFF
END
GO

IF NOT EXISTS(SELECT * FROM [Folder] WHERE [ID] = 0)
BEGIN
	SET IDENTITY_INSERT [Folder] ON
	INSERT INTO [Folder] ([ID], [State], [Flag], [FileAs], [VersionControl]) VALUES (0, 1, 0, 'GarbageCollection', 1) 
	SET IDENTITY_INSERT [Folder] OFF
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentSendBackTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [DocumentSendBackTypeID] TIdentifier NULL'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'HasToBeSentBack' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [HasToBeSentBack] TBool NOT NULL DEFAULT 0'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SentBackTime' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	EXEC sp_executesql N'ALTER TABLE [DocumentCard] ADD [SentBackTime] TDateTime NULL'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'HasToBeSentBack' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Case')
	EXEC sp_executesql N'ALTER TABLE [Case] ADD [HasToBeSentBack] TBool NOT NULL DEFAULT 0'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'WordTemplatesFolderID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Case')
	EXEC sp_executesql N'ALTER TABLE [Case] ADD [WordTemplatesFolderID] TIdentifier NULL'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'WordTemplatesPath' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Case')
	EXEC sp_executesql N'ALTER TABLE [Case] ADD [WordTemplatesPath] TNotes null'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'FileTemplate' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	EXEC sp_executesql N'ALTER TABLE [Folder] ADD [FileTemplate] TName NULL'
GO

