IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentCard') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentCard                                          */
	/*==============================================================*/
	create table DocumentCard (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   FileAs               TName                not null,
	   FolderID             TIdentifier          not null,
	   CompanyID            TIdentifier          null,
	   ContactID            TIdentifier          null,
	   CorrespondentCompanyID TIdentifier          null,
	   CorrespondentContactID TIdentifier          null,
	   CardOwnerID          TIdentifier          null,
	   DocumentCardID       TIdentifier          null,
	   DivisionID           TIdentifier          null,
	   DocumentCardClassID  TIdentifier          null,
	   CustodyPlaceID       TIdentifier          null,
	   CaseID               TIdentifier          null,
	   DocumentSendTypeID   TIdentifier          null,
	   Date                 TDate                null,
	   Number               TName                null,
	   DateIn               TDate                null,
	   NumberIn             TName                null,
	   DateOut              TDate                null,
	   NumberOut            TName                null,
	   AppendixCount        TCount               null,
	   PageCount            TCount               null,
	   Copy                 TBool                null,
	   StorePeriod          TCount               null,
	   Package              TCode                null,
	   SentTime             TDateTime            null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint DocumentCardPK primary key  (ID)
	)
go
IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Date' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard' AND [isnullable]=1)
BEGIN
	EXEC sp_executesql N'
		ALTER TABLE [DocumentCard] DISABLE TRIGGER ALL
		UPDATE
			[DocumentCard]
		SET
			[Date] = [Created]
		WHERE
			[Date] IS NULL
		ALTER TABLE [DocumentCard] ENABLE TRIGGER ALL
		ALTER TABLE [DocumentCard] ALTER COLUMN [Date] TDate NULL
		'
END
GO
IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CompanyCorrespondentID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	EXEC sp_rename 'DocumentCard.CompanyCorrespondentID', 'CorrespondentCompanyID'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CustodyPlaceID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [CustodyPlaceID] TIdentifier NULL
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Package' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [Package] TCode NULL
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'DocumentCardID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [DocumentCardID] TIdentifier NULL
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CaseID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentCard')
	ALTER TABLE [DocumentCard] ADD [CaseID] TIdentifier NULL
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentCardClass') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentCardClass                                     */
	/*==============================================================*/
	create table DocumentCardClass (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   Code                 TCode                not null,
	   FileAs               TName                not null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint DocumentCardClassPK primary key  (ID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Case') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: Case                                          */
	/*==============================================================*/
	create table [Case] (
		Class                TClass               not null,
		ID                   TIdentifier          identity,
		State                TState               not null,
		Flag                 TFlag                not null,
		Code                 TCode                not null,
		FileAs               TName                not null,
		CompanyID            TIdentifier          not null,
		EmployeeID           TIdentifier          null,
		StartDate            TDate                not null,
		EndDate              TDate                null,
		NumberTemplate       TName                null,
		CurrentNumber        TCount               not null default 1,
		Increment            TCount               not null default 1,
		AllowInternal        TBool                not null default 0,
		AllowIncoming        TBool                not null default 0,
		AllowOutgoing        TBool                not null default 0,
		Created              TSysDateTime         not null default GETDATE(),
		CreatedBy            TUserID              not null,
		Modified             TSysDateTime         not null default GETDATE(),
		ModifiedBy           TUserID              not null,
		Comments             TNotes               null,
		Timestamp            timestamp            not null,
		constraint CasePK primary key  (ID)
	)

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentFlowEntry') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Class' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentFlowEntry')
		DROP TABLE [DocumentFlowEntry]
END
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentFlowEntry') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentFlowEntry                                     */
	/*==============================================================*/
	create table DocumentFlowEntry (
	   ID                   TIdentifier          identity,
	   DocumentClass        TClass               not null,
	   DocumentID           TIdentifier          not null,
	   EmployeeID           TIdentifier          not null,
	   ChargeDate           TDate                not null,
	   ReturnDate           TDate                null,
	   Comments             TNotes               null,
	   constraint DocumentFlowEntryPK primary key  (ID)
		)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentItem') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentItem                                          */
	/*==============================================================*/
	create table DocumentItem (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   FileAs               TName                not null,
	   FileName             TName                null,
	   Extension            TName                null,
	   FolderID             TIdentifier          not null,
	   DocumentCardID       TIdentifier          null,
	   LatestVersion        TCount               not null default 1,
	   CheckedOutBy         TUserID              null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Timestamp            timestamp            not null,
	   constraint DocumentItemPK primary key  (ID)
	)
go
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CheckedOutBy' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentItem')
BEGIN
	ALTER TABLE [DocumentItem] ADD [CheckedOutBy] TUserID null
	EXEC sp_executesql N'
		ALTER TABLE [DocumentItem] DISABLE TRIGGER ALL
		UPDATE
			[DocumentItem]
		SET
			[CheckedOutBy] = [ModifiedBy]
		WHERE
			[State] = 3
		ALTER TABLE [DocumentItem] ENABLE TRIGGER ALL
		'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Extension' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentItem')
BEGIN
	ALTER TABLE [DocumentItem] ADD [Extension] TName null
	EXEC sp_executesql N'
		DECLARE @ID TIdentifier, @FileAs TName, @Extension TName, @str TName, @index int
		ALTER TABLE [DocumentItem] DISABLE TRIGGER ALL
		DECLARE items CURSOR FOR SELECT [ID], [FileName] FROM [DocumentItem]
		OPEN items
		FETCH NEXT FROM items INTO @ID, @FileAs
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @str = @FileAs
			SET @index = CHARINDEX(''.'', @str)
			
			WHILE @index > 0
			BEGIN
				SET @str = SUBSTRING(@str, @index, LEN(@str) - @index + 1)
				SET @index = CHARINDEX(''.'', @str, @index + 1)
			END
			IF @str != @FileAs
				SET @Extension = @str
			ELSE
				SET @Extension = ''''
			
			UPDATE
				[DocumentItem]
			SET
				[Extension] = @Extension
			WHERE
				[ID] = @ID
				
			FETCH NEXT FROM items INTO @ID, @FileAs
		END
		CLOSE items
		DEALLOCATE items
		ALTER TABLE [DocumentItem] ENABLE TRIGGER ALL
		'
END
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentItemReadProtocol') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentItemReadProtocol                              */
	/*==============================================================*/
	create table DocumentItemReadProtocol (
	   ID                   TIdentifier          not null,
	   Version              TCount               not null,
	   UserID               TIdentifier          not null,
	   Created              TDate                not null default GETDATE(),
       constraint DocumentItemReadProtocolPK primary key  (ID, UserID, Created)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentType') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentType                                          */
	/*==============================================================*/
	create table DocumentType (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   Code                 TCode                not null,
	   FileAs               TName                not null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint DocumentTypePK primary key  (ID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'DocumentVersion') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: DocumentVersion                                       */
	/*==============================================================*/
	create table DocumentVersion (
	   ID                   TIdentifier          identity,
	   DocumentItemID       TIdentifier          not null,
	   Version              TCount               null,
	   Data                 TBlob                null,
	   Comments				TNotes				 null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TIdentifier          not null,
	   constraint DocumentVersionPK primary key  (ID),
	   constraint DocumentVersionAK unique (DocumentItemID, Version)
	)
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Comments' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentVersion')
	EXEC sp_executesql N'ALTER TABLE [DocumentVersion] ADD [Comments] TNotes null'
GO

GRANT SELECT, INSERT, UPDATE, DELETE ON DocumentVersion TO PUBLIC
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Folder') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: Folder                                                */
	/*==============================================================*/
	create table Folder (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   Code                 TCode                null,
	   FileAs               TName                not null,
	   CompanyID            TIdentifier          null,
	   FolderID             TIdentifier          null,
	   VersionControl       TBool                null,
	   PublicationControl   TBool                not null default 0,
	   SubfolderCount       TCount               not null default 0,
	   ItemCount            TCount               not null default 0,
	   Parameters           xml                  null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Timestamp            timestamp            not null,
	   constraint FolderPK primary key  (ID),
	   constraint FolderAK unique (FolderID, FileAs)
	)
go

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Parameters' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [Parameters] XML NULL
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CompanyID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [CompanyID] TIdentifier NULL
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Code' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Folder')
	ALTER TABLE [Folder] ADD [Code] TCode NULL
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'FolderAccessControl') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: FolderAccessControl                                   */
	/*==============================================================*/
	create table FolderAccessControl (
	   TrusteeID            TIdentifier          not null,
	   FolderID             TIdentifier          not null,
	   AccessRights         TBitMask             not null default 0,
	   constraint FolderAccessControlPK primary key  (TrusteeID, FolderID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'CustodyPlace') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: CustodyPlace                                          */
	/*==============================================================*/
	create table CustodyPlace (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   Code                 TCode                not null,
	   FileAs               TName                not null,
	   Comments             TNotes               null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Timestamp            timestamp            not null,
	   constraint CustodyPlacePK primary key  (ID)
	)
go

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'IncomingDocument') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	create table IncomingDocument (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   FileAs               TName                not null,
	   CompanyID            TIdentifier          null,
	   ContactID            TIdentifier          null,
	   CorrespondentCompanyID TIdentifier          null,
	   CorrespondentContactID TIdentifier          null,
	   CardOwnerID          TIdentifier          null,
	   DocumentItemID       TIdentifier          null,
	   DivisionID           TIdentifier          null,
	   DocumentCardClassID  TIdentifier          null,
	   CustodyPlaceID       TIdentifier          null,
	   CaseID       TIdentifier          null,
	   Date                 TDate                not null,
	   Number               TName                null,
	   DateIn               TDate                null,
	   NumberIn             TName                null,
	   DateOut              TDate                null,
	   NumberOut            TName                null,
	   AppendixCount        TCount               null,
	   PageCount            TCount               null,
	   Copy                 TBool                null,
	   StorePeriod          TCount               null,
	   Package              TCode                null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint IncomingDocumentPK primary key  (ID)
	)
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CaseID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='IncomingDocument')
	ALTER TABLE [IncomingDocument] ADD [CaseID] TIdentifier NULL
GO

IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Date' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='IncomingDocument' AND [isnullable]=1)
BEGIN
	EXEC sp_executesql N'
		ALTER TABLE [IncomingDocument] DISABLE TRIGGER ALL
		UPDATE
			[IncomingDocument]
		SET
			[Date] = [Created]
		WHERE
			[Date] IS NULL
		ALTER TABLE [IncomingDocument] ENABLE TRIGGER ALL
		ALTER TABLE [IncomingDocument] ALTER COLUMN [Date] TDate NULL
		'
END
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'OutgoingDocument') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	create table OutgoingDocument (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   FileAs               TName                not null,
	   CompanyID            TIdentifier          null,
	   ContactID            TIdentifier          null,
	   CorrespondentContactID TIdentifier          null,
	   CorrespondentCompanyID TIdentifier          null,
	   CardOwnerID          TIdentifier          null,
	   DocumentItemID       TIdentifier          null,
	   DivisionID           TIdentifier          null,
	   DocumentCardClassID  TIdentifier          null,
	   CustodyPlaceID       TIdentifier          null,
	   CaseID       TIdentifier          null,
	   Date                 TDate                null,
	   Number               TName                null,
	   DateIn               TDate                null,
	   NumberIn             TName                null,
	   DateOut              TDate                null,
	   NumberOut            TName                null,
	   AppendixCount        TCount               null,
	   PageCount            TCount               null,
	   Copy                 TBool                null,
	   StorePeriod          TCount               null,
	   Package              TCode                null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint OutgoingDocumentPK primary key  (ID)
	)
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'CaseID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='OutgoingDocument')
	ALTER TABLE [OutgoingDocument] ADD [CaseID] TIdentifier NULL
GO

IF EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Date' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='OutgoingDocument' AND [isnullable]=1)
BEGIN
	EXEC sp_executesql N'
		ALTER TABLE [OutgoingDocument] DISABLE TRIGGER ALL
		UPDATE
			[OutgoingDocument]
		SET
			[Date] = [Created]
		WHERE
			[Date] IS NULL
		ALTER TABLE [OutgoingDocument] ENABLE TRIGGER ALL
		ALTER TABLE [OutgoingDocument] ALTER COLUMN [Date] TDate NULL
		'
END
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'SearchFolder') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	/*==============================================================*/
	/* Table: SearchFolder                                          */
	/*==============================================================*/
	create table SearchFolder (
	   ID                   TIdentifier          identity,
	   UserID               TIdentifier          null,
	   FileAs               TName                not null,
	   Criteria             XML                  not null,
	   constraint SearchFolderPK primary key  (ID)
	)
go

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Extension' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='DocumentVersion')
BEGIN
	ALTER TABLE [DocumentVersion] ADD [Extension] TName NULL
	EXEC sp_executesql N'
		UPDATE
			[DocumentVersion]
		SET
			[Extension] = [DocumentItem].[Extension]
		FROM 
			[DocumentVersion]
			INNER JOIN [DocumentItem]
				ON [DocumentItem].[ID] = [DocumentVersion].[DocumentItemID]
		'	
END
GO
