IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnPjm6GetCalculationCost]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnPjm6GetCalculationCost'
		DROP FUNCTION [dbo].[fnPjm6GetCalculationCost]
	END

GO

PRINT 'Creating function fnPjm6GetCalculationCost'
GO
/******************************************************************************
**		File: 
**		Name: fnPjm6GetCalculationCost
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: yuriy.soldatkin
**		Date: 19.04.2011
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		29.04.11	SYV				  view-
*******************************************************************************/
CREATE FUNCTION fnPjm6GetCalculationCost(@ID TIdentifier, @InProjectCurrency TBool)
RETURNS TMoney
AS
BEGIN
	DECLARE @Cost TMoney,
			@Total TMoney,
			@TotalWithVAT TMoney,
			@RatesWithVAT TBool,
			@VATRate TPercent,
			@DocumentClass TClass,
			@value TVariant,
			@GlobalVATRate TPercent
			
	SELECT
		@VATRate = [VATRate],
		@RatesWithVAT = [RatesWithVAT],
		@DocumentClass = [DocumentClass]
	FROM 
		[Calculation] 
	WHERE 
		[ID] = @ID
	
	SELECT @Value=[Value] FROM [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53] WHERE [Name]='VATRate' AND [Group]='CommonBusiness'
	SET	@GlobalVATRate = ISNULL(CAST(@value as decimal(18,4)), 0)

	IF @DocumentClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	BEGIN
		--  
		SELECT
			@Total = SUM([EntryAmount].[Price] * [EntryAmount].[Amount] / (1 + @RatesWithVAT * @VATRate)),
			@TotalWithVAT = SUM([EntryAmount].[Price] * [EntryAmount].[Amount] * (1 + (1 - @RatesWithVAT) * @VATRate))
		FROM
			[TimeEntry]
			INNER JOIN [EntryAmount]
				ON [TimeEntry].[ID] = [EntryAmount].[ID]
			INNER JOIN [CalculationLine]
				ON [TimeEntry].[ID] = [CalculationLine].[DocumentID]
				AND [TimeEntry].[Class] = [CalculationLine].[DocumentClass]
		WHERE
			[CalculationLine].[CalculationID] = @ID
	END
	ELSE IF @DocumentClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	BEGIN
		--
		SELECT
			@Total = SUM(CASE WHEN [ServiceRequest].[VATIsIncluded] = 0
								THEN [ServiceRequest].[Total]
							ELSE [ServiceRequest].[TotalWithVAT] / (1 + @GlobalVATRate)
						END),
			@TotalWithVAT = SUM(CASE @VATRate
				WHEN 0 
					THEN CASE WHEN [ServiceRequest].[VATIsIncluded] = 0
								THEN [ServiceRequest].[Total]
							ELSE [ServiceRequest].[TotalWithVAT] / (1 + @GlobalVATRate)
						END
				ELSE [ServiceRequest].[TotalWithVAT]
			END)
		FROM
			[ServiceRequest]
			INNER JOIN [CalculationLine]
				ON [ServiceRequest].[ID] = [CalculationLine].[DocumentID]
				AND [ServiceRequest].[Class] = [CalculationLine].[DocumentClass]
		WHERE
			[CalculationLine].[CalculationID] = @ID
	END
	ELSE IF @DocumentClass = dbo.adfGetClassByName('Pjm5.Expense')
		OR @DocumentClass = dbo.adfGetClassByName('Pjm5.PhoneCall')
	BEGIN
		IF(@InProjectCurrency = 1)
		BEGIN	
			--   
			SELECT
				@Total = SUM([TotalPC]),
				@TotalWithVAT = SUM([TotalWithVATPC])
			FROM
				[Expense]
				INNER JOIN [CalculationLine]
					ON [Expense].[ID] = [CalculationLine].[DocumentID]
					AND [Expense].[Class] = [CalculationLine].[DocumentClass]
			WHERE
				[CalculationLine].[CalculationID] = @ID
		END
		ELSE
		BEGIN
			--   
			SELECT
				@Total = SUM(CASE
					WHEN [Expense].[Amount] = 0
						THEN 0
					WHEN [Expense].[VATIsIncluded] = 1 
						THEN [Expense].[Amount] * [Expense].[Price] / (1 + [Expense].[VATRate])
					ELSE [Expense].[Amount] * [Expense].[Price]
				END),
				@TotalWithVAT = SUM(CASE 
					WHEN [Expense].[Amount] = 0
						THEN 0
					WHEN @VATRate = [Expense].[VATRate]
						THEN 
							CASE
								WHEN [Expense].[VATIsIncluded] = 1
									THEN [Expense].[Price] * [Expense].[Amount]
								ELSE
									[Expense].[Amount] * [Expense].[Price] + [Expense].[VAT]
							END
					ELSE
						CASE
							WHEN [Expense].[VATIsIncluded] = 1
								THEN ([Expense].[Price] * [Expense].[Amount] / (1 + [Expense].[VATRate])) * (1 + @VATRate)
							ELSE
								[Expense].[Price] * [Expense].[Amount] * (1 + @VATRate)
						END	
				END)
			FROM
				[Expense]
				INNER JOIN [CalculationLine]
					ON [Expense].[ID] = [CalculationLine].[DocumentID]
					AND [Expense].[Class] = [CalculationLine].[DocumentClass]
			WHERE
				[CalculationLine].[CalculationID] = @ID
		END
	END

	SET @Total = ISNULL(@Total, 0)
	SET @TotalWithVAT = ISNULL(@TotalWithVAT, 0)

	SET @Cost = @Total * (1 - @RatesWithVAT) + @TotalWithVAT * @RatesWithVAT
	
	RETURN @Cost
END
GO

GRANT EXEC ON fnPjm6GetCalculationCost TO PUBLIC
GO
