IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].Pjm6GetExperienceFormatted') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function Pjm6GetExperienceFormatted'
		DROP FUNCTION [dbo].Pjm6GetExperienceFormatted
	END

GO

PRINT 'Creating Procedure Pjm6GetExperienceFormatted'
GO
/******************************************************************************
**		File:	Pjm6GetExperienceFormatted.sql
**		Name: 	Pjm6GetExperienceFormatted
**		Desc:	returns formatted time interval
*******************************************************************************
**		Date:		Author:		Description:
**		-----		-------		-------------
**    	10.11.09		PAM		first version
*******************************************************************************/

CREATE FUNCTION Pjm6GetExperienceFormatted(@HireDate datetime,
	@FireDate datetime)
RETURNS varchar(64)
BEGIN
	DECLARE
		@Language TCode, 
		@MonthDif int,
		@YearDif int,
		@YearStr varchar(16),
		@YearsStr1 varchar(16),
		@YearsStr2 varchar(16),
		@MonthStr varchar(16),
		@MonthsStr varchar(16),
		@Result varchar(64)

	IF(@HireDate IS NULL)
		RETURN NULL

	SET @Result = ''
	SET @language = dbo.adfGetLanguage()
	IF(@Language = 'en')
	BEGIN
		SET @YearStr = 'year'
		SET @YearsStr1 = 'years'
		SET @YearsStr2 = 'years'
		SET @MonthStr = 'month'
		SET @MonthsStr = 'months'
	END
	ELSE IF(@Language = 'ru')
	BEGIN
		SET @YearStr = ''
		SET @YearsStr1 = ''
		SET @YearsStr2 = ''
		SET @MonthStr = '.'
		SET @MonthsStr = '.'
	END	

	IF(@FireDate IS NULL)
	BEGIN
		SET @FireDate = GetDate()
	END

	SET @MonthDif = DATEDIFF(month, @HireDate,@FireDate)
	
	IF(@MonthDif > 0)
	BEGIN
		DECLARE @DayStart int,
				@DayEnd int

		SET @DayStart = DATEPART(day, @HireDate)
		SET @DayEnd = DATEPART(day, @FireDate)

		IF(@DayEnd < @DayStart - 1) 
		BEGIN
			SET @MonthDif = @MonthDif - 1
		END 
	END
	SET @YearDif = @MonthDif / 12
	SET @MonthDif = @MonthDif - @YearDif*12

	IF(@YearDif > 0)
	BEGIN
		SET @Result = CAST(@YearDif AS varchar) + ' '
		IF(@YearDif = 1)
			SET @Result = @Result + @YearStr
		ELSE IF( (@YearDif > 10) AND (@YearDif < 20))
			SET @Result = @Result + @YearsStr2
		ELSE
		BEGIN
			DECLARE @Modulo int
			SET @Modulo = @YearDif % 10

			IF (@Modulo in (0, 5, 6, 7, 8, 9))
				SET @Result = @Result + @YearsStr2
			ELSE IF (@Modulo in (2, 3, 4))
				SET @Result = @Result +@YearsStr1
			ELSE IF(@Modulo in (1))
				SET @Result = @Result +@YearStr
		END
	END

	IF(@MonthDif > 0)
	BEGIN
		IF(@Result = '')
			SET @Result = CAST(@MonthDif AS varchar)
		ELSE
			SET @Result = @Result + ' ' + CAST(@MonthDif AS varchar)
		IF(@MonthDif = 1)
			SET @Result = @Result + ' ' + @MonthStr
		ELSE
			SET @Result = @Result + ' ' + @MonthsStr
	END

	IF(@MonthDif <= 0 and @YearDif <= 0)
		SET @Result = '0 ' + @MonthStr
	RETURN @Result
END
GO

GRANT EXEC ON Pjm6GetExperienceFormatted TO PUBLIC
GO
