IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'TaskBookingInfoUpdate')
	BEGIN
		PRINT 'Dropping Procedure TaskBookingInfoUpdate'
		DROP  Procedure  TaskBookingInfoUpdate
	END

GO

PRINT 'Creating Procedure TaskBookingInfoUpdate'
GO
/******************************************************************************
**		File: 
**		Name: TaskBookingInfoUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    06.10.09		syv				  pjm6AssignmentCheckBudgetOverdraft
*******************************************************************************/
CREATE Procedure TaskBookingInfoUpdate
	@Xml XML
AS
	DECLARE @doc INT, @TaskID INT, @ProjectID INT, @State TState, @rv int
	
	SET NOCOUNT ON

	EXEC sp_xml_preparedocument @doc OUTPUT, @Xml
	SELECT @TaskID = [Task].[ID], @ProjectID = [Task].[ProjectID], @State = [Task].[State]
	FROM 
		OPENXML(@doc, '/Task', 2) WITH ([ID] int) AS [XmlTable]
		INNER JOIN [Task]
			ON [Task].[ID] = [XmlTable].[ID]
	
/*	IF @State NOT IN (7, 8) AND 
		dbo.fnPjm5IsProjectManager(@ProjectID, NULL) = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END*/
	
	BEGIN TRANSACTION
	
	IF EXISTS (
		SELECT * FROM [Project] WHERE [ID] = @ProjectID AND [MsProjectID] IS NULL)
	BEGIN
		DELETE [BusyEntry] WHERE [TaskID] = @TaskID
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			EXEC sp_xml_removedocument @doc
			RETURN 1
		END
		
		/*INSERT INTO [BusyEntry] 
			([ProjectID], [TaskID], [AssignmentID], [EmployeeID], [Date], [Planned], [Committed])
		SELECT
			[XmlTable].[ProjectID],
			[XmlTable].[TaskID],
			[XmlTable].[AssignmentID],
			[Assignment].[EmployeeID],
			[XmlTable].[Date],
			[XmlTable].[WorkMinutes],
			(CASE [Committed] WHEN 'false' THEN 0 ELSE 1 END)
		FROM
			OPENXML(@doc, '/Task/TaskAssignments/TaskAssignment/WorkList/WorkEntry', 2)
				WITH (
					[ProjectID] INT '../../../../ProjectID',
					[TaskID] INT '../../../../ID',
					[AssignmentID] INT '../../AssignmentID',
					[Date] varchar(19),
					[WorkMinutes] int,
					[Committed] varchar(5)
					) as [XmlTable] LEFT OUTER JOIN [vwAssignment] as [Assignment]
						ON ([XmlTable].[AssignmentID] = [Assignment].[ID])*/				
		;
		WITH [ActualTime] ([AssignmentID], [EmployeeID], [Date], [ActualWork], [ActualOverTimeWork]) AS
		(
			SELECT 
				[TimeEntry].[AssignmentID] as [AssignmentID],
				[TimeEntry].[EmployeeID],
				CONVERT(varchar(8), [TimeEntry].[StartTime], 112) as [Date],
				(CASE [TimeEntry].[Overtime] WHEN 0 THEN SUM([TimeEntry].[Duration]) ELSE 0 END) as [ActualWork],
				(CASE [TimeEntry].[Overtime] WHEN 1 THEN SUM([TimeEntry].[Duration]) ELSE 0 END) as [ActualOvertimeWork]
			FROM 
				[TimeEntry]
			WHERE
				[TimeEntry].[TaskID] = @TaskID AND
				[TimeEntry].[State] IN (2, 3, 6)
			GROUP BY
				[TimeEntry].[AssignmentID], 
				[TimeEntry].[EmployeeID],
				[TimeEntry].[Overtime],
				CONVERT(varchar(8), [TimeEntry].[StartTime], 112)
		)
		INSERT INTO [BusyEntry] ([AssignmentID], [ProjectID], [TaskID], [Date], [EmployeeID], [Planned], [Committed], [ActualWork], [ActualOverTimeWork])
		SELECT
			ISNULL([XmlTable].[AssignmentID], [ActualTime].[AssignmentID]),
			@ProjectID,
			@TaskID,
			CAST(ISNULL([XmlTable].[Date], [ActualTime].[Date])  AS datetime),
			ISNULL([AssignmentEntry].[EmployeeID], [ActualTime].[EmployeeID]),
			ISNULL([XmlTable].[WorkMinutes], 0),
			--ISNULL([XmlTable].[Canceled], 0),
			ISNULL((CASE [XmlTable].[Committed] WHEN 'false' THEN 0 ELSE 1 END), 0),
			ISNULL([ActualTime].[ActualWork], 0),
			ISNULL([ActualTime].[ActualOvertimeWork], 0)
		FROM
			OPENXML(@doc, '/Task/TaskAssignments/TaskAssignment/WorkList/WorkEntry', 2)
				WITH (
					[ProjectID] INT '../../../../ProjectID',
					[TaskID] INT '../../../../ID',
					[AssignmentID] INT '../../AssignmentID',
					[Date] varchar(19),
					[WorkMinutes] int,
					[Committed] varchar(5)
					) as [XmlTable] 
			INNER JOIN [AssignmentEntry]
				ON [AssignmentEntry].[AssignmentID] = [XmlTable].[AssignmentID]
			FULL OUTER JOIN [ActualTime]
				ON [ActualTime].[AssignmentID] = [XmlTable].[AssignmentID]
				AND CAST([ActualTime].[Date] As DateTime) = CAST([XmlTable].[Date] As DateTime)
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			EXEC sp_xml_removedocument @doc
			RETURN 1
		END
	END	
	
	UPDATE 
		[TaskAssignment]
	SET
		[Work] = CAST([XmlTable].[Work] * 60 as INT),
		[Units] = [XmlTable].[Units],
		[RemainingWork] = [XmlTable].[RemainingWork],
		[RemainingOvertimeWork] = [XmlTable].[RemainingOvertimeWork],
		[State] = [XmlTable].[State]
	FROM
		[TaskAssignment]
		INNER JOIN OPENXML(@doc, '/Task/TaskAssignments/TaskAssignment', 2)
			WITH (
				[TaskID] int '../../ID',
				[AssignmentID] int,
				[Work] decimal(18, 4),
				[Units] decimal(18, 4),
				[RemainingWork] decimal(18,4),
				[RemainingOvertimeWork] decimal(18,4),
				[State] int
				) AS [XmlTable] 
			ON ([XmlTable].[TaskID] = [TaskAssignment].[TaskID] AND 
				[XmlTable].[AssignmentID] = [TaskAssignment].[AssignmentID])
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		EXEC sp_xml_removedocument @doc
		RETURN 1
	END
	
	DECLARE @DueDate TDateTime, @TaskClass TClass
	SET @TaskClass = dbo.adfGetClassByName('Pjm5.Task')

	DECLARE @Budget TMoney
	SELECT
		@Budget = SUM([XmlTable].[Work] * [Assignment].[StandardRate])
	FROM
		OPENXML(@doc, '/Task/TaskAssignments/TaskAssignment', 2)
			WITH (
				[TaskID] int '../../ID',
				[AssignmentID] int,
				[Work] decimal(18, 4)
				) AS [XmlTable]
		INNER JOIN [Assignment]
			ON [XmlTable].[AssignmentID] = [Assignment].[ID]
	
	UPDATE 
		[Task]
	SET
		[StartDate] = [XmlTable].[StartDate],
		[DueDate] = [XmlTable].[DueDate],
		@DueDate = [XmlTable].[DueDate],
		[Duration] = [XmlTable].[Duration],
		[ManHour] = CAST([XmlTable].[Work] * 60 as INT),
		[Budget] = @Budget
	FROM
		OPENXML(@doc, '/Task', 2)
			WITH (
				[ID] int,
				[StartDate] varchar(19),
				[DueDate] varchar(19),
				[Duration] decimal(18,4),
				[Work] decimal(18,4)
				) as [XmlTable] 
		INNER JOIN [Task]
			ON ([XmlTable].[ID] = [Task].[ID])

	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		EXEC sp_xml_removedocument @doc
		RETURN 1
	END
	
	IF EXISTS (
		SELECT * FROM [Reminder]
		WHERE [DocumentClass] = @TaskClass AND [DocumentID] = @TaskID AND
			[Subtype] = 'TD'
	)
	BEGIN
		UPDATE [Reminder] SET [EventTime] = @DueDate
		WHERE [DocumentClass] = @TaskClass AND [DocumentID] = @TaskID AND
			[Subtype] = 'TD'
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			EXEC sp_xml_removedocument @doc
			RETURN 1
		END
	END
	
	exec @rv = pjm6TaskPercentUpdate @TaskID
	IF @rv != 0
	BEGIN
		ROLLBACK TRANSACTION
		EXEC sp_xml_removedocument @doc
		RETURN 1
	END
		
	COMMIT TRANSACTION
	EXEC sp_xml_removedocument @doc
	
	--DECLARE @AssignmentID TIdentifier
	--DECLARE items CURSOR LOCAL FAST_FORWARD FOR SELECT [AssignmentID] FROM [TaskAssignment] WHERE [TaskID] = @TaskID
	--OPEN items
	--FETCH NEXT FROM items INTO @AssignmentID
	--WHILE @@FETCH_STATUS = 0
	--BEGIN
	--	EXEC pjm6AssignmentCheckBudgetOverdraft @TaskID, @AssignmentID, NULL
	--	FETCH NEXT FROM items INTO @AssignmentID
	--END
	--CLOSE items
	--DEALLOCATE items
	
	RETURN 0
GO

GRANT EXEC ON TaskBookingInfoUpdate TO PUBLIC
GO
