IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trBudgetLineExpenseUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trBudgetLineExpenseUpdate'
		DROP  Trigger pjm5trBudgetLineExpenseUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trBudgetLineExpenseUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: Anton.Paramonov
**		Date: 17/11/2010 15:20:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trBudgetLineExpenseUpdate'
GO
CREATE Trigger pjm5trBudgetLineExpenseUpdate 
ON [BudgetLineExpense] FOR INSERT, UPDATE, DELETE
AS
BEGIN
	IF @@ROWCOUNT = 0
		RETURN
	
	IF  NOT EXISTS(
		SELECT *
		FROM
			[inserted]
			FULL JOIN [deleted]
				ON [deleted].[ID] = [inserted].[ID] 
		WHERE
			(
			[deleted].[PlannedAmount] != [inserted].[PlannedAmount] OR
			[deleted].[TotalPCPlannedExpense] != [inserted].[TotalPCPlannedExpense] OR
			[deleted].[TotalWithVATPCPlannedExpense] != [inserted].[TotalWithVATPCPlannedExpense]) OR
			([deleted].[ID] IS NULL AND [inserted].[ID] IS NOT NULL AND ([inserted].[State] = 1 OR [inserted].[State] IS NULL)) OR
			([inserted].[ID] IS NULL AND [deleted].[ID] IS NOT NULL AND [deleted].[State] = 1) 
			)
		RETURN

	DECLARE @changed TABLE ([ID] int)
	INSERT INTO @changed ([ID])
	SELECT
		ISNULL([inserted].[BudgetID], [deleted].[BudgetID])
	FROM
		[inserted]
		FULL OUTER JOIN [deleted]
			ON [inserted].[ID] = [deleted].[ID]

	DECLARE 
		@ID int, 
		@ApplyVAT bit, 
		@VATIsIncluded bit,
		@TotalPCPlannedExpense money, 
		@TotalWithVATPCPlannedExpense money

	DECLARE @totals TABLE (
		[ID] int, 
		[TotalPCPlannedExpense] money, 
		[TotalWithVATPCPlannedExpense] money)
	DECLARE lines CURSOR FAST_FORWARD FOR SELECT [ID] FROM @changed

	OPEN lines
	FETCH NEXT FROM lines INTO @ID
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SELECT
			@TotalPCPlannedExpense = ISNULL(SUM(ROUND([BudgetLineExpense].[TotalPCPlannedExpense], 2)), 0),
			@TotalWithVATPCPlannedExpense = ISNULL(SUM(ROUND([BudgetLineExpense].[TotalWithVATPCPlannedExpense], 2)), 0)
		FROM
			[Budget]
			INNER JOIN [BudgetLineExpense]
				ON [BudgetLineExpense].[BudgetID] = [Budget].[ID]
		WHERE
			[Budget].[ID] = @ID

		INSERT INTO @totals ([ID], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense])
		VALUES (@ID, @TotalPCPlannedExpense, @TotalWithVATPCPlannedExpense)
		FETCH NEXT FROM lines INTO @ID
	END
	CLOSE lines
	DEALLOCATE lines

	UPDATE
		[Budget]
	SET
		[WarningFlagID] = NULL,
		[TotalPCPlannedExpense] = ROUND(ISNULL([Total].[TotalPCPlannedExpense], 0), 2),
		[TotalWithVATPCPlannedExpense] = ROUND(ISNULL([Total].[TotalWithVATPCPlannedExpense], 0), 2),
		[ExpenseBudgetBottom] = (CASE WHEN [Budget].[ExpenseBudgetBottomEqualsPlan] = 1 THEN ROUND(ISNULL(dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [Total].[TotalPCPlannedExpense], [Total].[TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([Total].[TotalPCPlannedExpense], [VATRate], 0, 1)) , 0), 2) ELSE [ExpenseBudgetBottom] END),
		[ExpenseBudgetHigh] = (CASE WHEN [Budget].[ExpenseBudgetHighEqualsPlan] = 1 THEN ROUND(ISNULL(dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [Total].[TotalPCPlannedExpense], [Total].[TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([Total].[TotalPCPlannedExpense], [VATRate], 0, 1)) , 0), 2) ELSE [ExpenseBudgetHigh] END)
	FROM
		[Budget]
		INNER JOIN @totals AS [Total]
			ON [Budget].[ID] = [Total].[ID]
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
END
GO