IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trBudgetLineWorkUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm6trBudgetLineWorkUpdate'
		DROP  Trigger pjm6trBudgetLineWorkUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm6trBudgetLineWorkUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: Anton.Paramonov
**		Date: 01/11/2010 14:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm6trBudgetLineWorkUpdate'
GO
CREATE Trigger pjm6trBudgetLineWorkUpdate 
ON [BudgetLineWork] FOR INSERT, UPDATE, DELETE
AS
BEGIN
	IF @@ROWCOUNT = 0
		RETURN
	
	IF  NOT EXISTS(
		SELECT *
		FROM
			[inserted]
			FULL JOIN [deleted]
				ON [deleted].[ID] = [inserted].[ID] 
		WHERE
			(
			[deleted].[PlannedManHour] != [inserted].[PlannedManHour] OR
			[deleted].[TotalPCPlannedManHour] != [inserted].[TotalPCPlannedManHour] OR
			[deleted].[TotalWithVATPCPlannedManHour] != [inserted].[TotalWithVATPCPlannedManHour]) OR
			([deleted].[ID] IS NULL AND [inserted].[ID] IS NOT NULL AND ([inserted].[State] = 1 OR [inserted].[State] IS NULL)) OR
			([inserted].[ID] IS NULL AND [deleted].[ID] IS NOT NULL AND [deleted].[State] = 1) 
			)
		RETURN

	DECLARE @changed TABLE ([ID] int)
	INSERT INTO @changed ([ID])
	SELECT
		ISNULL([inserted].[BudgetID], [deleted].[BudgetID])
	FROM
		[inserted]
		FULL OUTER JOIN [deleted]
			ON [inserted].[ID] = [deleted].[ID]
		INNER JOIN [Budget]
			ON 	[Budget].[ID] = ISNULL([inserted].[BudgetID], [deleted].[BudgetID])

	DECLARE 
		@ID int,
		@PlannedManHour decimal(18, 4), 
		@TotalPCPlannedManHour money, 
		@TotalWithVATPCPlannedManHour money
	
	DECLARE @totals TABLE (
		[ID] int, 
		[PlannedManHour] decimal(18, 4),
		[TotalPCPlannedManHour] money, 
		[TotalWithVATPCPlannedManHour] money)
	DECLARE lines CURSOR FAST_FORWARD FOR SELECT [ID] FROM @changed

	OPEN lines
	FETCH NEXT FROM lines INTO @ID
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SELECT
			@PlannedManHour = ISNULL(SUM([BudgetLineWork].[PlannedManHour]), 0),
			@TotalPCPlannedManHour = ISNULL(SUM(ROUND([BudgetLineWork].[TotalPCPlannedManHour] , 2)), 0),
			@TotalWithVATPCPlannedManHour = ISNULL(SUM(ROUND([BudgetLineWork].[TotalWithVATPCPlannedManHour], 2)), 0)
		FROM
			[Budget]
			INNER JOIN [BudgetLineWork]
				ON [BudgetLineWork].[BudgetID] = [Budget].[ID]
		WHERE
			[Budget].[ID] = @ID

		INSERT INTO @totals ([ID], [PlannedManHour], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
		VALUES (@ID, @PlannedManHour, @TotalPCPlannedManHour, @TotalWithVATPCPlannedManHour)
		FETCH NEXT FROM lines INTO @ID
	END
	CLOSE lines
	DEALLOCATE lines

	UPDATE
		[Budget]
	SET
		[WarningFlagID] = NULL,
		[PlannedManHour] = ISNULL([Total].[PlannedManHour], 0),
		[TotalPCPlannedManHour] = ROUND(ISNULL([Total].[TotalPCPlannedManHour], 0), 2),
		[TotalWithVATPCPlannedManHour] = ROUND(ISNULL([Total].[TotalWithVATPCPlannedManHour], 0), 2),
		[WorkBudgetBottom] = (CASE WHEN [Budget].[WorkBudgetBottomEqualsPlan] = 1 THEN ROUND(ISNULL(dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [Total].[TotalPCPlannedManHour], [Total].[TotalWithVATPCPlannedManHour]), 0), 2) ELSE [WorkBudgetBottom] END),
		[WorkBudgetHigh] = (CASE WHEN [Budget].[WorkBudgetHighEqualsPlan] = 1 THEN ROUND(ISNULL(dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [Total].[TotalPCPlannedManHour], [Total].[TotalWithVATPCPlannedManHour]), 0), 2) ELSE [WorkBudgetHigh] END)
	FROM
		[Budget]
		INNER JOIN @totals AS [Total]
			ON [Budget].[ID] = [Total].[ID]
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
END
GO