IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5CalculationAddPhoneCallLines]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5CalculationAddPhoneCallLines'
		DROP PROCEDURE [dbo].[pjm5CalculationAddPhoneCallLines]
	END
GO

PRINT 'Creating procedure pjm5CalculationAddPhoneCallLines'
GO

/******************************************************************************
**		File: pjm5CalculationAddPhoneCallLines.sql
**		Name: pjm5CalculationAddPhoneCallLines
**		Desc:        
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Anton.Paramonov
**		Date: 05.07.2012 15:06:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**
*******************************************************************************/
CREATE PROCEDURE pjm5CalculationAddPhoneCallLines
	@CalculationID TIdentifier,
	@Xml xml
AS
SET NOCOUNT ON
	DECLARE
			@ID TIdentifier,
			@DocumentClass TClass,
			@RatesWithVAT TBool,
			@ProjectID TIdentifier,
			@State	TState,
			@Scale TCount,
			@VATRate TPercent,
			@EmployeeID TIdentifier,
			@doc int

	DECLARE @lines TABLE (
		[DocumentID] int,
		[Description] nvarchar(max)
		)
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @DocumentClass = dbo.adfGetClassByName('Pjm5.Expense')

	SELECT 
		@ID = [Calculation].[ID],
		@RatesWithVAT = [RatesWithVAT],
		@ProjectID = [Calculation].[ProjectID],
		@Scale = [Calculation].[Scale],
		@VATRate = [Calculation].[VATRate],
		@State = [Calculation].[State]
	FROM
		[Calculation]
	WHERE
		[ID] = @CalculationID

	IF dbo.adfHasAccessRightByName('ManageAllCalculations') = 0 AND
		dbo.fnPjm5IsProjectManager(@ProjectID, @EmployeeID) = 0 AND
		dbo.fnPjm5IsProjectMember(@ProjectID, @EmployeeID) = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
			
	IF ISNULL(@State, 0) = 3
	BEGIN
		RAISERROR('#TransitionTemplateCouldNotBeFound', 18, -1)
		RETURN 1
	END

	EXEC sp_xml_preparedocument @doc OUTPUT, @Xml
	INSERT INTO @lines
	SELECT
		l.[DocumentID],
		l.[Description]
	FROM
		OPENXML(@doc, '/pjm5CalculationGet/CalculationLine', 2)
			WITH (
				[DocumentID] int,
				[Description] nvarchar(max)
				) as l
	EXEC sp_xml_removedocument @doc

	BEGIN TRANSACTION

	INSERT INTO [CalculationLine] ([CalculationID], [DocumentClass], [DocumentID], [Description], [Amount], [Price], [VATIsIncluded], [VAT])
	SELECT
		@ID, 
		@DocumentClass, 
		l.[DocumentID], 
		l.[Description],
		[Expense].[Amount], 
		CASE 
			WHEN [Expense].[VATIsIncluded] = 0 OR [Expense].[VATRate] = @VATRate
				THEN [Expense].[Price]
			WHEN [Expense].[Amount] != 0
				THEN ([Expense].[Price] - [Expense].[VAT] / [Expense].[Amount]) * (1 + @VATRate)
			ELSE ([Expense].[Price] / (1 + [Expense].[VATRate])) * (1 + @VATRate)
		END,
		[Expense].[VATIsIncluded], 
		CASE 
			WHEN [Expense].[VATRate] = @VATRate
				THEN [Expense].[VAT]
			ELSE
				CASE
					WHEN [Expense].[VATIsIncluded] = 1 AND [Expense].[Amount] != 0
						THEN ([Expense].[Price] - [Expense].[VAT] / [Expense].[Amount]) * @VATRate
					WHEN [Expense].[VATIsIncluded] = 1 AND [Expense].[Amount] = 0
						THEN ([Expense].[Price] / (1 + [Expense].[VATRate])) * @VATRate
					ELSE [Expense].[Price] * @VATRate
				END
		END
	FROM 
		@lines l
		INNER JOIN [Expense]
			ON  l.[DocumentID] = [Expense].[ID] AND @DocumentClass = [Expense].[Class] 
	WHERE 
		[Expense].[Billable] = 1 AND
		NOT EXISTS (
			SELECT * 
			FROM [CalculationLine] 
			WHERE [CalculationLine].[CalculationID] = @ID AND 
				[DocumentClass] = @DocumentClass AND [DocumentID] = l.[DocumentID])
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
			
	COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON pjm5CalculationAddPhoneCallLines TO PUBLIC
GO