IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5CaseUpdate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5CaseUpdate'
		DROP PROCEDURE [dbo].[pjm5CaseUpdate]
	END
GO

PRINT 'Creating procedure pjm5CaseUpdate'
GO

SET QUOTED_IDENTIFIER ON
GO
/******************************************************************************
**		File: pjm5CaseUpdate.sql
**		Name: pjm5CaseUpdate
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAV
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE pjm5CaseUpdate
	@Xml XML
AS
	SET NOCOUNT ON
	DECLARE
		@ID TIdentifier,
		@doc int
	
	DECLARE 
		@FileAs TName,
		@Code TCode,
		@CompanyID TIdentifier,
		@PrevCompanyID TIdentifier,
		@EmployeeID TIdentifier,
		@StartDate TDate,
		@EndDate TDate,
		@NumberTemplate TName,
		@CurrentNumber TCount,
		@Increment TCount,
		@AllowInternal TBool,
		@AllowIncoming TBool,
		@AllowOutgoing TBool,
		@AllowFolder TBool,
		@HasToBeSentBack TBool,
		@Comments TNotes,
		@DocumentCount int,
		@WordTemplatesFolderID TIdentifier,
		@WordTemplatesPath TNotes
		
	EXEC sp_xml_preparedocument @doc OUTPUT, @Xml
	SELECT
		@ID = [XmlTable].[ID],
		@FileAs = [XmlTable].[FileAs],
		@Code = [XmlTable].[Code],
		@CompanyID = [XmlTable].[CompanyID],
		@EmployeeID = [XmlTable].[EmployeeID],
		@StartDate = CAST([XmlTable].[StartDate] as datetime),
		@EndDate = CAST([XmlTable].[EndDate] as datetime),
		@NumberTemplate  = [XmlTable].[NumberTemplate],
		@CurrentNumber = [XmlTable].[CurrentNumber],
		@Increment = [XmlTable].[Increment],
		@AllowInternal = CASE WHEN [XmlTable].[AllowInternal] = 'false' THEN 0 ELSE 1 END,
		@AllowIncoming = CASE WHEN [XmlTable].[AllowIncoming] = 'false' THEN 0 ELSE 1 END,
		@AllowOutgoing = CASE WHEN [XmlTable].[AllowOutgoing] = 'false' THEN 0 ELSE 1 END,
		@AllowFolder = CASE WHEN [XmlTable].[AllowFolder] = 'false' THEN 0 ELSE 1 END,
		@HasToBeSentBack  = CASE WHEN [XmlTable].[HasToBeSentBack] = 'false' THEN 0 ELSE 1 END,
		@Comments = [XmlTable].[Comments],
		@PrevCompanyID = [Case].[CompanyID],
		@WordTemplatesFolderID = [XmlTable].[WordTemplatesFolderID],
		@WordTemplatesPath = [XmlTable].[WordTemplatesPath]
	FROM 
		OPENXML(@doc, '/pjm5CaseGet/Case', 2)
			WITH (
				[ID] int,
				[FileAs] nvarchar(512),
				[Code] nvarchar(32),
				[CompanyID] int,
				[EmployeeID] int,
				[StartDate] varchar(19),
				[EndDate] varchar(19),
				[NumberTemplate] nvarchar(512),
				[CurrentNumber] int,
				[Increment] int,
				[AllowInternal] varchar(5),
				[AllowIncoming] varchar(5),
				[AllowOutgoing] varchar(5),
				[AllowFolder] varchar(5),
				[HasToBeSentBack] varchar(5),
				[Comments] nvarchar(max),
				[WordTemplatesFolderID] int,
				[WordTemplatesPath] nvarchar(max)
				) AS [XmlTable]
		INNER JOIN [Case]
			ON [Case].[ID] = [XmlTable].[ID]
	EXEC sp_xml_removedocument @doc
	
	SET @DocumentCount = 0
	IF EXISTS(SELECT * FROM [DocumentCard] WHERE [CaseID] = @ID)
		SET @DocumentCount = 1
	
	BEGIN TRANSACTION
	UPDATE 
		[Case]
	SET
		[Code] = @Code,
		[FileAs] = @FileAs,
		[CompanyID] = @CompanyID,
		[EmployeeID] = @EmployeeID,
		[StartDate] = @StartDate,
		[EndDate] = @EndDate,
		[NumberTemplate] = @NumberTemplate,
		[CurrentNumber] = @CurrentNumber,
		[Increment] = @Increment,
		[AllowInternal] = CASE @DocumentCount WHEN 1 THEN [AllowInternal] ELSE @AllowInternal END,
		[AllowIncoming] = CASE @DocumentCount WHEN 1 THEN [AllowIncoming] ELSE @AllowIncoming END,
		[AllowOutgoing] = CASE @DocumentCount WHEN 1 THEN [AllowOutgoing] ELSE @AllowOutgoing END,
		[AllowFolder] = @AllowFolder,
		[HasToBeSentBack] = @HasToBeSentBack,
		[Comments] = @Comments,
		[WordTemplatesFolderID] = @WordTemplatesFolderID,
		[WordTemplatesPath] = @WordTemplatesPath
	WHERE
		[ID] = @ID
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	IF @AllowFolder = 0
	BEGIN
		UPDATE
			[CaseOfFolder]
		SET
			[Allowed] = 0
		WHERE
			[CaseID] = @ID AND
			[Allowed] = 1
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	IF @PrevCompanyID != @CompanyID
	BEGIN
		UPDATE
			[CaseOfFolder]
		SET
			[FolderID] = [Folder].[ID],
			[Allowed] = CASE WHEN ISNULL(T.C.value('.', 'int'), 0) = 0 THEN 1 ELSE [CaseOfFolder].[Allowed] END
		FROM
			[CaseOfFolder]
			INNER JOIN [Case]
				ON [Case].[ID] = [CaseOfFolder].[CaseID]
			INNER JOIN [Folder]
				ON [Folder].[CompanyID] = [Case].[CompanyID]
			OUTER APPLY [Folder].[Parameters].nodes('/Parameters/AllowClassFlag') T(C)
		WHERE
			[CaseOfFolder].[CaseID] = @ID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	COMMIT TRANSACTION
	EXEC pjm5CaseGet @ID

	RETURN 0
GO

GRANT EXECUTE ON pjm5CaseUpdate TO PUBLIC
GO
