IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trExpenseUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trExpenseUpdate'
		DROP  Trigger pjm5trExpenseUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trExpenseUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trExpenseUpdate'
GO
CREATE TRIGGER pjm5trExpenseUpdate 
ON [Expense] FOR UPDATE
AS
BEGIN
    IF @@rowcount = 0
       RETURN
       
    SET NOCOUNT ON
    IF EXISTS (
		SELECT *
		FROM [inserted]
			INNER JOIN [CalculationLine]
				ON ([inserted].[Class] = [CalculationLine].[DocumentClass] AND
					[inserted].[ID] = [CalculationLine].[DocumentID])
			INNER JOIN [Calculation]
				ON [Calculation].[ID] = [CalculationLine].[CalculationID]
		WHERE
			[Calculation].[ProjectID] != [inserted].[ProjectID]
	)
	BEGIN
		RAISERROR('#cannotchangeentryproject', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
    DECLARE @CurrentEmployeeID TIdentifier
    SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
   
    IF EXISTS (
		SELECT *
		FROM [inserted]
			INNER JOIN [deleted]
				ON [inserted].[ID] = [deleted].[ID]
		WHERE
			[deleted].[State] = 3 AND [inserted].[State] = 3 AND -- approved entry
			(
				dbo.fnPjm5IsProjectManager([inserted].[ProjectID], @CurrentEmployeeID) != 1 AND
				dbo.fnPjm5IsProjectApprover([inserted].[ProjectID], @CurrentEmployeeID) != 1
			)
	)
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
    
    IF EXISTS (SELECT * FROM [inserted] WHERE [State] IN (3, 6) AND [CurrencyRate] IS NULL)
	BEGIN
		RAISERROR('#CannotApproveBillableExpenseWithoutCurrencyRate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

    IF EXISTS (
		SELECT * FROM [inserted] 
		WHERE 
			(
				[CurrencyRate] < 0 OR
				[Amount] < 0 OR
				[Price] < 0
			)
		)
	BEGIN
		RAISERROR('#CannotApproveBillableExpenseWithoutPriceOrAmountOrCurrencyRate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@ExpenseClass TClass,
		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass,
		@FactureClass TClass
	SET @ExpenseClass = dbo.adfGetClassByName('Pjm5.Expense')
	SET @CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET @StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET @InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET @FactureClass = dbo.adfGetClassByName('Pjm5.Facture')
	
	IF EXISTS(
		SELECT * 
		FROM 
			inserted 
			INNER JOIN deleted
				ON deleted.[ID] = inserted.[ID]
			INNER JOIN [Attachment]
				ON ([Attachment].[DocumentClass] = @ExpenseClass AND
					[Attachment].[DocumentID] = inserted.[ID] AND 
					[Attachment].[AttachmentTypeID] = 3 AND
					[Attachment].[LinkDocumentClass] IN (@CalculationClass, @StatementClass, @InvoiceClass, @FactureClass))
		WHERE
			deleted.[State] IN (3, 6) AND
			inserted.[State] NOT IN (3, 6)
		)
	BEGIN
		RAISERROR('#cannotchangeentrystate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	IF EXISTS(
		SELECT * 
		FROM 
			inserted 
			INNER JOIN deleted
				ON deleted.[ID] = inserted.[ID]
			INNER JOIN [Attachment]
				ON ([Attachment].[DocumentClass] = @ExpenseClass AND
					[Attachment].[DocumentID] = inserted.[ID] AND 
					[Attachment].[AttachmentTypeID] = 3 AND
					[Attachment].[LinkDocumentClass] IN (@CalculationClass, @StatementClass, @InvoiceClass, @FactureClass))
		WHERE
			inserted.[State] = deleted.[State] AND
			deleted.[State] IN (3, 6) AND
			inserted.[FromCurrencyCode] != deleted.[FromCurrencyCode]
			
		)
	BEGIN
		RAISERROR('#cannotchangeentrycurrency', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	IF EXISTS (
		SELECT *
		FROM [inserted]
		INNER JOIN [deleted]
				ON [deleted].[ID] = [inserted].[ID]
		WHERE [inserted].[TaskID] IS NOT NULL AND
			dbo.fnPjm5IsTaskAssigned([inserted].[TaskID], [inserted].[EmployeeID]) = 0
	)
	BEGIN
		RAISERROR('#taskassignmentnotfound', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
    IF EXISTS (
		SELECT *
		FROM [inserted]
		WHERE 
			[inserted].[VATIsIncluded] = 1 AND 
			[inserted].[Price] * [inserted].[Amount] < [inserted].[VAT]
	)
	BEGIN
		RAISERROR('#wrongvatvalue', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

END
GO

 