IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trInvoiceInsert')
	BEGIN
		PRINT 'Dropping Trigger pjm5trInvoiceInsert'
		DROP  Trigger pjm5trInvoiceInsert
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trInvoiceInsert
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trInvoiceInsert'
GO
CREATE Trigger pjm5trInvoiceInsert 
ON [Invoice] FOR INSERT
AS
BEGIN
	IF EXISTS(
		SELECT * 
		FROM 
			[inserted]
			INNER JOIN [Project]
				ON [Project].[ID] = [inserted].[ProjectID]
		WHERE 
			[Project].[CustomerCompanyID] IS NULL
	)
	BEGIN
		RAISERROR('#CustomerCannotBeNull', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trInvoiceUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trInvoiceUpdate'
		DROP  Trigger pjm5trInvoiceUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trInvoiceUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trInvoiceUpdate'
GO
CREATE Trigger pjm5trInvoiceUpdate 
ON [Invoice] FOR UPDATE
AS
BEGIN
	IF EXISTS (SELECT * FROM [inserted] WHERE ([CurrencyRate] IS NULL OR [CurrencyRate] <= 0) AND [State] IN (2, 5))
	BEGIN
		RAISERROR('#CurrencyRateCannotBeNull', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	IF EXISTS (SELECT * FROM [inserted] WHERE ([BaseCurrencyRate] IS NULL OR [BaseCurrencyRate] <= 0) AND [State] IN (2, 5))
	BEGIN
		RAISERROR('#BaseCurrencyRateCannotBeNull', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	IF EXISTS (SELECT * FROM [inserted] WHERE ISNULL([inserted].[Number], '') = '' AND [inserted].[State] NOT IN (1, 3, 6))
	BEGIN
		RAISERROR('#InvalidDocumentNumber', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@calculationClass TClass
	SET @calculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [InvoiceLine]
				ON inserted.[ID] = [InvoiceLine].[InvoiceID]
			INNER JOIN [Calculation]
				ON [InvoiceLine].[DocumentClass] = @calculationClass AND 
					[Calculation].[ID] = [InvoiceLine].[DocumentID] 		
		WHERE
			[Calculation].[State] NOT IN (2, 3)
		)
	BEGIN
		RAISERROR('#CannotApproveInvoiceNotApprovedCalculation', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@serviceRequestClass TClass,
		@serviceAssignmentClass TClass
	SET @serviceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	SET @serviceAssignmentClass = dbo.adfGetClassByName('Pjm5.ServiceAssignment')
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [InvoiceLine]
				ON [InvoiceLine].[InvoiceID] = inserted.[ID]
			INNER JOIN [Document]
				ON [Document].[Class] = [InvoiceLine].[DocumentClass] AND 
					[Document].[ID] = [InvoiceLine].[DocumentID]
			WHERE
				[InvoiceLine].[DocumentClass] != @calculationClass AND
				(
				[InvoiceLine].[DocumentClass] = @serviceRequestClass AND [Document].[State] NOT IN (4, 5) OR
				[InvoiceLine].[DocumentClass] NOT IN (@serviceRequestClass, @serviceAssignmentClass) AND [Document].[State] NOT IN (3, 6)
				)
			)
	BEGIN
		RAISERROR('#CannotApproveInvoiceNotApprovedRecords', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [InvoiceLine]
				ON [InvoiceLine].[InvoiceID] = inserted.[ID]
			LEFT OUTER JOIN [TimeEntry]
				ON [TimeEntry].[Class] = [InvoiceLine].[DocumentClass] AND 
					[TimeEntry].[ID] = [InvoiceLine].[DocumentID]
			LEFT OUTER JOIN [Expense]
				ON [Expense].[Class] = [InvoiceLine].[DocumentClass] AND 
					[Expense].[ID] = [InvoiceLine].[DocumentID]
			LEFT OUTER JOIN [ServiceRequest]
				ON [ServiceRequest].[Class] = [InvoiceLine].[DocumentClass] AND 
					[ServiceRequest].[ID] = [InvoiceLine].[DocumentID]
		WHERE
			(ISNULL([TimeEntry].[ID], 0) + 
				ISNULL([Expense].[ID], 0) +
				ISNULL([ServiceRequest].[ID], 0)) != 0
			AND		
			(ISNULL(CAST([TimeEntry].[Billable] AS int), 0) + 
				ISNULL(CAST([Expense].[Billable] AS int), 0) +
				ISNULL(CAST([ServiceRequest].[Billable] AS int), 0)) = 0
		)
	BEGIN
		RAISERROR('#CannotApproveNotBillableRecords', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	--- REJECT 
	UPDATE
		[Calculation]
	SET
		[Calculation].[State] = 2,
		[Calculation].[InvoiceApproved] = 0,
		[Calculation].[Modified] = DEFAULT,
		[Calculation].[ModifiedBy] = DEFAULT
	FROM
		[Calculation]
		INNER JOIN [InvoiceLine]
			ON [Calculation].[Class] = [InvoiceLine].[DocumentClass] AND 
				[Calculation].[ID] = [InvoiceLine].[DocumentID]
		INNER JOIN inserted
			ON [InvoiceLine].[InvoiceID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
	WHERE
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [Calculation].[Class] AND 
				[DocumentID] = [Calculation].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [StatementLine] 
				INNER JOIN [Statement]
					ON [Statement].[ID] = [StatementLine].[StatementID]
			WHERE 
				[Statement].[State] IN (1, 2, 3) AND
				[DocumentClass] = [Calculation].[Class] AND 
				[DocumentID] = [Calculation].[ID]
			)
		
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitStatementCalculation', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE
		[TimeEntry]
	SET
		[TimeEntry].[State] = 3,
		[TimeEntry].[Modified] = DEFAULT,
		[TimeEntry].[ModifiedBy] = DEFAULT
	FROM
		[TimeEntry]
		INNER JOIN [InvoiceLine]
			ON [TimeEntry].[Class] = [InvoiceLine].[DocumentClass] AND 
				[TimeEntry].[ID] = [InvoiceLine].[DocumentID]
		INNER JOIN inserted
			ON [InvoiceLine].[InvoiceID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
	WHERE
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [TimeEntry].[Class] AND 
				[DocumentID] = [TimeEntry].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [StatementLine] 
				INNER JOIN [Statement]
					ON [Statement].[ID] = [StatementLine].[StatementID]
			WHERE 
				[Statement].[State] IN (2, 3) AND
				[DocumentClass] = [TimeEntry].[Class] AND 
				[DocumentID] = [TimeEntry].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceTimeEntry', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	UPDATE
		[Expense]
	SET
		[Expense].[State] = 3,
		[Expense].[Modified] = DEFAULT,
		[Expense].[ModifiedBy] = DEFAULT
	FROM
		[Expense]
		INNER JOIN [InvoiceLine]
			ON [Expense].[Class] = [InvoiceLine].[DocumentClass] AND 
				[Expense].[ID] = [InvoiceLine].[DocumentID]
		INNER JOIN inserted
			ON [InvoiceLine].[InvoiceID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
	WHERE
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [Expense].[Class] AND 
				[DocumentID] = [Expense].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [StatementLine] 
				INNER JOIN [Statement]
					ON [Statement].[ID] = [StatementLine].[StatementID]
			WHERE 
				[Statement].[State] IN (2, 3) AND
				[DocumentClass] = [Expense].[Class] AND 
				[DocumentID] = [Expense].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceExpense', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE
		[ServiceRequest]
	SET
		[ServiceRequest].[State] = 4,
		[ServiceRequest].[Modified] = DEFAULT,
		[ServiceRequest].[ModifiedBy] = DEFAULT
	FROM
		[ServiceRequest]
		INNER JOIN [InvoiceLine]
			ON [ServiceRequest].[Class] = [InvoiceLine].[DocumentClass] AND 
				[ServiceRequest].[ID] = [InvoiceLine].[DocumentID]
		INNER JOIN inserted
			ON [InvoiceLine].[InvoiceID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
	WHERE
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [ServiceRequest].[Class] AND 
				[DocumentID] = [ServiceRequest].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [StatementLine] 
				INNER JOIN [Statement]
					ON [Statement].[ID] = [StatementLine].[StatementID]
			WHERE 
				[Statement].[State] IN (2, 3) AND
				[DocumentClass] = [ServiceRequest].[Class] AND 
				[DocumentID] = [ServiceRequest].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceServiceRequest', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE @timeClass TClass
	SET @timeClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	UPDATE
		[EntryAmount]
	SET
		[Amount] = ROUND([TimeEntry].[RoundedDuration] / 60.0, 2),
		[Price] = [TimeEntry].[Rate]
	FROM 
		[EntryAmount]
		INNER JOIN [TimeEntry]
			ON [TimeEntry].[ID] = [EntryAmount].[ID]
		INNER JOIN [InvoiceLine]
			ON [InvoiceLine].[DocumentClass] = @timeClass AND
				[InvoiceLine].[DocumentID] = [EntryAmount].[ID]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [InvoiceLine].[InvoiceID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
		LEFT OUTER JOIN [StatementLine]
			ON [StatementLine].[DocumentClass] = @timeClass AND
				[StatementLine].[DocumentID] = [EntryAmount].[ID]
	WHERE
		[StatementLine].[ID] IS NULL
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END			

	--- REJECT 
	DELETE [Attachment]
	FROM
		[Attachment]
		INNER JOIN [inserted]
			ON ([inserted].[Class] = [Attachment].[LinkDocumentClass] AND 
				[inserted].[ID] = [Attachment].[LinkDocumentID])
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 3 AND
		[deleted].[State] != 3 AND
		[Attachment].[AttachmentTypeID] = 3
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRejectInvoice', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DELETE [AdvanceInvoice]
	FROM
		[AdvanceInvoice]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [AdvanceInvoice].[InvoiceID]
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 3 AND
		[deleted].[State] != 3 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

	DELETE [AdvanceInvoice]
	FROM
		[AdvanceInvoice]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [AdvanceInvoice].[AdvanceInvoiceID]
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 3 AND
		[deleted].[State] != 3 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

	DELETE [StatementAdvanceInvoice]
	FROM
		[StatementAdvanceInvoice]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [StatementAdvanceInvoice].[AdvanceInvoiceID]
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 3 AND
		[deleted].[State] != 3 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	
	--   
	DELETE [PaymentDetail]
	FROM
		[PaymentDetail]
		INNER JOIN [inserted]
			ON ([inserted].[ID] = [PaymentDetail].[InvoiceID])
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 3 AND
		[deleted].[State] != 3
	
	--    
	UPDATE
		[UserReminder]
	SET
		[State] = 0
	FROM
		[Reminder]
		INNER JOIN [inserted]
			ON ([inserted].[ID] = [Reminder].[DocumentID] AND
			[inserted].[Class] = [Reminder].[DocumentClass])
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
		INNER JOIN [UserReminder]
			ON [UserReminder].[ReminderID] = [Reminder].[ID]
	WHERE
		([Reminder].[Subtype] = 'VA' OR [Reminder].[Subtype] = 'VP') AND
		[inserted].[State] = 5 AND
		[deleted].[State] != 5 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trInvoiceDelete')
	BEGIN
		PRINT 'Dropping Trigger pjm6trInvoiceDelete'
		DROP  Trigger pjm6trInvoiceDelete
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm6trInvoiceDelete
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: KOV
**		Date: 28.06.10
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm6trInvoiceDelete'
GO
CREATE Trigger pjm6trInvoiceDelete 
ON [Invoice] FOR DELETE
AS
BEGIN
	IF EXISTS(SELECT * FROM [deleted] INNER JOIN [SuperTimer] 
				ON [SuperTimer].[DocumentClass] = [deleted].[Class] AND [SuperTimer].[DocumentID] = [deleted].[ID] AND 
				([SuperTimer].[State] in (1,2) OR ([SuperTimer].[State] = 3 AND [SuperTimer].[Persistent] = 1)))
	BEGIN
		RAISERROR('#CannotDeleteDocumentWithActiveTimer', 18, -1)
		ROLLBACK TRANSACTION
	END
END

GO