IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'lmContract')
	BEGIN
		PRINT 'Dropping Procedure lmContract'
		DROP  Procedure  lmContract
	END

GO

PRINT 'Creating Procedure lmContract'
GO
/******************************************************************************
**		File: 
**		Name: lmContract
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure lmContract
AS
	SET NOCOUNT ON

	DECLARE @EmployeeID TIdentifier
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()

	SELECT
		[ID],
		[ContractNumber] as [Code],
		[ContractSubject] as [Name],
		[State],
		[SupplierCompanyID],
		[CustomerCompanyID],
		[CurrencyCode],
		[RatesWithVAT]
	FROM
		[Project]
	WHERE
		([Project].[State] IN (1, 2, 3, 4)) AND
		([Project].[ProjectTypeID] = 1) AND
		([ID] IN (SELECT [ProjectID] FROM [vwAssignment] WHERE [EmployeeID] = @EmployeeID))
		
	RETURN 0
GO

GRANT EXEC ON lmContract TO PUBLIC
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'lmContractForDocument')
	BEGIN
		PRINT 'Dropping Procedure lmContractForDocument'
		DROP  Procedure  lmContractForDocument
	END

GO

PRINT 'Creating Procedure lmContractForDocument'
GO
/******************************************************************************
**		File: 
**		Name: lmContractForDocument
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		21.04.11	SPV				     
**									       
*******************************************************************************/
CREATE Procedure lmContractForDocument
AS
	SET NOCOUNT ON

	DECLARE
		@EmployeeID TIdentifier,
		@ViewAllFinance TBool,
		@ManageAllBudgets	TBool,
		@ManageBudgets		TBool,
		@BudgetSupervisior	TBool
		
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance'))

	DECLARE
		@ViewAllProjects TBool,
		@Assistant TBool,
		@ExpenseSecretary TIdentifier
		
	SET @ViewAllProjects = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllProjects')) 
	SET @Assistant = dbo.adfHasAccessRight(dbo.adfGetRightID('Assistant'))
	SET @ExpenseSecretary = dbo.adfHasAccessRightByName('ManageExpenseWithoutAssignment')	

	SET @ManageAllBudgets = dbo.adfHasAccessRight(dbo.adfGetRightID('ManageAllBudgets'))
	SET @ManageBudgets = dbo.adfHasAccessRight(dbo.adfGetRightID('ManageBudgets'))
	SET @BudgetSupervisior = dbo.adfHasAccessRight(dbo.adfGetRightID('CanBeBudgetSupervisior'))

	SELECT
		[ID],
		[ContractNumber] as [Code],
		ISNULL([ContractSubject], [FIleAs]) as [Name],
		[State],
		[SupplierCompanyID],
		[CustomerCompanyID],
		[CurrencyCode],
		[RatesWithVAT],
		[Project].[ProjectTypeID]
	FROM
		[Project]
	WHERE
		[State] NOT IN (5, 6) AND
		(
			@ViewAllFinance= 1 OR 
			@ExpenseSecretary = 1 OR
			@ViewAllProjects = 1 AND @Assistant = 1 OR 
			([Project].[ID] IN (SELECT [ID] FROM dbo.fnPjm6GetAvailableProjectsForProjectMemberAndManager())) OR
			@ManageAllBudgets = 1 OR
			([Project].[ID] IN (SELECT [ID] FROM dbo.fnPjm6GetAvailableProjectsForProjectMemberAndManager()) AND @ManageBudgets = 1) OR
			@BudgetSupervisior = 1
		)
	
	RETURN 0
GO

GRANT EXEC ON lmContractForDocument TO PUBLIC
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'lmContractForGrid')
	BEGIN
		PRINT 'Dropping Procedure lmContractForGrid'
		DROP  Procedure  lmContractForGrid
	END

GO

PRINT 'Creating Procedure lmContractForGrid'
GO
/******************************************************************************
**		File: 
**		Name: lmContractForGrid
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure lmContractForGrid
AS
	SET NOCOUNT ON

	SELECT
		[ID],
		[ContractNumber] as [Code],
		ISNULL([ContractSubject], [FIleAs]) as [Name]
	FROM
		[Project]
	WHERE
		[State] NOT IN (5, 6)
	
	RETURN 0
GO

GRANT EXEC ON lmContractForGrid TO PUBLIC
GO