IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'NamedListUpdate')
	BEGIN
		PRINT 'Dropping Procedure NamedListUpdate'
		DROP  Procedure  NamedListUpdate
	END

GO

PRINT 'Creating Procedure NamedListUpdate'
GO
/******************************************************************************
**		File: 
**		Name: NamedListUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure NamedListUpdate
	@ProjectID TIdentifier,
	@Xml XML
AS
	DECLARE @doc INT,
			@result INT			
			
	SET NOCOUNT ON
	
	EXEC @result = sp_xml_preparedocument @doc OUTPUT, @Xml
	
	IF @result != 0 OR @@error != 0
		RETURN 1
	
	BEGIN TRANSACTION
	
	DECLARE NamedListCursor CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
		SELECT
			[ID],
			[Name],
			(CASE [SortOrderAscending] WHEN 'false' THEN 0 ELSE 1 END) as [SortOrderAscending]
		FROM
			OPENXML(@doc, '/NamedListDataSet/NamedList', 2)
				WITH (
					[ID] INT,
					[Name] nvarchar(256),
					[SortOrderAscending] varchar(5)
					)
	
	DELETE [NamedList] WHERE [ProjectID] = @ProjectID

	IF @@ERROR != 0 GOTO ErrorHandler

	DECLARE @NamedListID TIdentifier, @ID TIdentifier, @Name TName, @SortOrderAscending TBool
	
	OPEN NamedListCursor
	
	FETCH NEXT FROM NamedListCursor INTO @ID, @Name, @SortOrderAscending
	WHILE @@FETCH_STATUS = 0
	BEGIN
		INSERT INTO [NamedList] ([ProjectID], [Name], [SortOrderAscending]) VALUES(@ProjectID, @Name, @SortOrderAscending)
		SET @NamedListID = SCOPE_IDENTITY()

		INSERT INTO [NamedListBinding]
		SELECT
			@NamedListID,
			[UDFIndex],
			[UDFCaption]
		FROM
			OPENXML(@doc, '/NamedListDataSet/NamedListBinding', 2)
				WITH (
					NamedListID INT,
					UDFIndex INT,
					UDFCaption nvarchar(256)
					)
		WHERE
			[NamedListID] = @ID
		IF @@error != 0 GOTO ErrorHandler 
			
		INSERT INTO [NamedListEntry] ([NamedListID], [Code])
		SELECT
			@NamedListID,
			[Code]
		FROM
			OPENXML(@doc, '/NamedListDataSet/NamedListEntry', 2)
				WITH (
					NamedListID INT,
					Code varchar(32)
				)
		WHERE
			[NamedListID] = @ID
		IF @@error != 0 GOTO ErrorHandler
	
		FETCH NEXT FROM NamedListCursor INTO @ID, @Name, @SortOrderAscending
	END			
	
	COMMIT TRANSACTION

	CLOSE NamedListCursor
	DEALLOCATE NamedListCursor

	EXEC sp_xml_removedocument @doc
	RETURN 0
	
ErrorHandler:
	CLOSE NamedListCursor
	DEALLOCATE NamedListCursor

	ROLLBACK TRANSACTION
	EXEC sp_xml_removedocument @doc
	RETURN 1
GO

GRANT EXEC ON NamedListUpdate TO PUBLIC
GO
