IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5DocumentUserReminderCreate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5DocumentUserReminderCreate'
		DROP PROCEDURE [dbo].[pjm5DocumentUserReminderCreate]
	END
GO

PRINT 'Creating procedure pjm5DocumentUserReminderCreate'
GO

/******************************************************************************
**		File: pjm5DocumentUserReminderCreate.sql
**		Name: pjm5DocumentUserReminderCreate
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	---				--------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5DocumentUserReminderCreate]
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@ReminderEventTime TDateTime
AS
	SET NOCOUNT ON
	
	IF @DocumentClass IS NULL OR @DocumentID IS NULL
	BEGIN
		RETURN 1
	END
	
	DECLARE
		@DatabaseView TSysname,
		@ReminderSubtype TCode2,
		@ReminderID TIdentifier,
		@ReminderInitialState TState,
		@ReminderInitialInterval TTimeSpan,
		@sql nvarchar(3000),
		@sqlexec int,
		@rv int
		
	CREATE TABLE #view
	([DocumentClass] smallint, [DocumentID] int, [UserID] int, [EventTime] datetime)
			
	DECLARE reminders CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
	SELECT
		[ReminderType].[Subtype],
		[ReminderType].[View]
	FROM
		[ReminderType]
	WHERE
		[ReminderType].[Class] = @DocumentClass
		
	OPEN reminders

	FETCH NEXT FROM reminders INTO @ReminderSubtype, @DatabaseView
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @sql = N'SELECT [DocumentClass], [DocumentID], [RecipientUserID], [EventTime] FROM ' +
		@DatabaseView + ' WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass'
		
		DELETE FROM #view
		INSERT INTO #view ([DocumentClass], [DocumentID], [UserID], [EventTime])
		exec @sqlexec = sp_executesql @sql,
			N'@DocumentID int, @DocumentClass smallint', @DocumentID, @DocumentClass
		IF @sqlexec != 0
		BEGIN
			GOTO ERROR_HANDLER
		END
		
		SELECT TOP 1 @ReminderEventTime = [DbView].[EventTime]
		FROM #view AS [DbView]
		
		EXEC @rv = pjm5ReminderCreateInternal @ReminderID OUTPUT,
			@DocumentClass, @DocumentID, @ReminderSubtype, @ReminderEventTime
			
		IF @rv != 0 OR @ReminderID IS NULL
		BEGIN
			GOTO ERROR_HANDLER
		END
		
		DECLARE @UserID TIdentifier
		DECLARE users CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
		SELECT [DbView].[UserID]
		FROM #view AS [DbView]
		WHERE [DbView].[UserID] IS NOT NULL
		
		OPEN users

		FETCH NEXT FROM users INTO @UserID
		WHILE @@FETCH_STATUS = 0
		BEGIN
			IF NOT EXISTS (
				SELECT * FROM [UserReminder]
				WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
			)
			BEGIN
				IF @ReminderEventTime IS NOT NULL
				BEGIN
					SELECT 
						@ReminderInitialState = ISNULL(dbo.fnPjm5GetUserInitialReminderState(@DocumentClass, @ReminderSubtype, @UserID), 0),
						@ReminderInitialInterval = ISNULL(dbo.fnPjm5GetReminderInterval(@DocumentClass, @ReminderSubtype, @UserID), 15)
					
					INSERT INTO [UserReminder] ([ReminderID], [UserID], [State],
						[OriginalInterval], [NextInterval], [UserDefined])
					VALUES (@ReminderID, @UserID, @ReminderInitialState,
						@ReminderInitialInterval, @ReminderInitialInterval, 0)
				END
			END
			ELSE
			BEGIN
				UPDATE [UserReminder] SET
					[State] =	CASE
									WHEN @ReminderEventTime IS NOT NULL THEN [State]
									ELSE 0
								END
				WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
			END
			FETCH NEXT FROM users INTO @UserID
		END
		IF @@error != 0
		BEGIN
			CLOSE users
			DEALLOCATE users
			GOTO ERROR_HANDLER
		END
		
		CLOSE users
		DEALLOCATE users
		
		FETCH NEXT FROM reminders INTO @ReminderSubtype, @DatabaseView
	END

	CLOSE reminders
	DEALLOCATE reminders
	RETURN 0

ERROR_HANDLER:
	CLOSE reminders
	DEALLOCATE reminders
	RETURN 1
		
	RETURN 0
GO

GRANT EXECUTE ON [pjm5DocumentUserReminderCreate] TO PUBLIC
GO