IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5ReminderCreate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5ReminderCreate'
		DROP PROCEDURE [dbo].[pjm5ReminderCreate]
	END
GO

PRINT 'Creating procedure pjm5ReminderCreate'
GO

/******************************************************************************
**		File: pjm5ReminderCreate.sql
**		Name: pjm5ReminderCreate
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5ReminderCreate]
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@ReminderSubtype TCode2,
	@ReminderEventTime TDateTime,
	@UserID TIdentifier,
	@ReminderState TState,
	@ReminderInterval TTimeSpan
AS
	SET NOCOUNT ON
	
	IF @DocumentClass IS NULL OR @DocumentID IS NULL OR @UserID IS NULL
	BEGIN
		RETURN 1
	END
	
	IF @ReminderInterval IS NULL
	BEGIN
		SET @ReminderInterval = dbo.fnPjm5GetReminderInterval(@DocumentClass, @ReminderSubtype, @UserID)
	END
	
	DECLARE @ReminderID TIdentifier, @rv int
		
	EXEC @rv = pjm5ReminderCreateInternal @ReminderID OUTPUT,
		@DocumentClass, @DocumentID, @ReminderSubtype, @ReminderEventTime
		
	IF @rv != 0 OR @ReminderID IS NULL
	BEGIN
		RETURN 1
	END
	
	IF NOT EXISTS (
		SELECT * FROM [UserReminder]
		WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
	)
	BEGIN
		IF @ReminderEventTime IS NOT NULL
		BEGIN
			INSERT INTO [UserReminder] ([ReminderID], [UserID], [State],
				[OriginalInterval], [NextInterval], [UserDefined])
			VALUES (@ReminderID, @UserID, @ReminderState, @ReminderInterval,
				@ReminderInterval, 0)
			IF @@error != 0
			BEGIN
				RETURN 1
			END
		END
	END
	ELSE
	BEGIN
		UPDATE [UserReminder] SET
			[State] =	CASE
							WHEN @ReminderEventTime IS NOT NULL THEN @ReminderState
							ELSE 0
						END,
			[OriginalInterval] = @ReminderInterval,
			[NextInterval] = @ReminderInterval
		WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
		IF @@error != 0
		BEGIN
			RETURN 1
		END
	END
	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5ReminderCreate] TO PUBLIC
GO