IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5ReminderGet]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5ReminderGet'
		DROP PROCEDURE [dbo].[pjm5ReminderGet]
	END
GO

PRINT 'Creating procedure pjm5ReminderGet'
GO

/******************************************************************************
**		File: pjm5ReminderGet.sql
**		Name: pjm5ReminderGet
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5ReminderGet]
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@UserID TIdentifier
AS
	SET NOCOUNT ON
	
	IF @UserID IS NULL
	BEGIN
		SET @UserID = dbo.adfGetCurrentUserID()
	END
	
	IF @UserID IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	SELECT
		[Reminder].[ID],
		[ReminderType].[Name],
		[Reminder].[EventTime],
		[UserReminder].[UserID],
		CASE
			WHEN [UserReminder].[UserID] IS NULL THEN
				ISNULL([ReminderSetting].[InitialState], [ReminderType].[DefaultState])
			ELSE [UserReminder].[State]
		END AS [State],
		CASE
			WHEN [UserReminder].[UserID] IS NULL THEN
				ISNULL([ReminderSetting].[InitialInterval], [ReminderType].[DefaultInterval])
			ELSE [UserReminder].[OriginalInterval]
		END AS [Interval]
	FROM
		[Reminder]
		INNER JOIN [ReminderType]
			ON ([ReminderType].[Class] = [Reminder].[DocumentClass] AND
			[ReminderType].[Subtype] = [Reminder].[Subtype])
		LEFT OUTER JOIN [UserReminder]
			ON ([Reminder].[ID] = [UserReminder].[ReminderID] AND
			[UserReminder].[UserID] = @UserID)
		LEFT OUTER JOIN [ReminderSetting]
			ON ([ReminderType].[Class] = [ReminderSetting].[Class] AND
				[ReminderType].[Subtype] = [ReminderSetting].[Subtype] AND
				[ReminderSetting].[UserID] = @UserID)
	WHERE
		[Reminder].[DocumentClass] = @DocumentClass AND
		[Reminder].[DocumentID] = @DocumentID
	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5ReminderGet] TO PUBLIC
GO
  