IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm6DocumentSupervisorReminderCreate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm6DocumentSupervisorReminderCreate'
		DROP PROCEDURE [dbo].[pjm6DocumentSupervisorReminderCreate]
	END
GO

PRINT 'Creating procedure pjm6DocumentSupervisorReminderCreate'
GO

/******************************************************************************
**		File: pjm6DocumentSupervisorReminderCreate.sql
**		Name: pjm6DocumentSupervisorReminderCreate
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm6DocumentSupervisorReminderCreate]
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@ReminderEventTime TDateTime
AS
	SET NOCOUNT ON
	
	IF @DocumentClass IS NULL OR @DocumentID IS NULL
	BEGIN
		RETURN 1
	END
	
	DECLARE
		@DatabaseView				TSysname,
		@ReminderSubtype			TCode2,
		@ReminderID					TIdentifier,
		@ReminderInitialState		TState,
		@ReminderInitialInterval	TTimeSpan,
		@sql						nvarchar(3000),
		@sqlexec					int,
		@rv							int,
		@SupervisorEmployeeID		TIdentifier,
		@UserID						TIdentifier

	--setting UserID
		SELECT
			@UserID = [Employee].[UserID]
		FROM
			[Taskrequest]
		INNER JOIN [Employee] ON
			[TaskRequest].[SupervisorEmployeeID] = [Employee].[ID]
		WHERE
			[Taskrequest].[Class] = @DocumentClass AND [Taskrequest].[ID] = @DocumentID
	
	IF (@SupervisorEmployeeID IS NOT NULL) AND (@UserID IS NOT NULL)
		BEGIN
			--setting VIEW
			SELECT
				@ReminderSubtype = [ReminderType].[Subtype],
				@DatabaseView = [ReminderType].[View]
			FROM [ReminderType]
			WHERE [ReminderType].[Class] = @DocumentClass

			SET @sql = N'SELECT [DocumentClass], [DocumentID], [RecipientUserID], [EventTime] FROM ' +
				@DatabaseView + ' WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass'
				--selecting users in assignment
				CREATE TABLE #view
					([DocumentClass] smallint, [DocumentID] int, [UserID] int, [EventTime] datetime)
				DELETE FROM #view
				INSERT INTO #view ([DocumentClass], [DocumentID], [UserID], [EventTime])
				exec @sqlexec = sp_executesql @sql,
					N'@DocumentID int, @DocumentClass smallint', @DocumentID, @DocumentClass
				IF @sqlexec != 0
				BEGIN
					RETURN 1
				END
			
			---getting last time
			SELECT TOP 1 @ReminderEventTime = [DbView].[EventTime]
			FROM #view AS [DbView]

			EXEC @rv = pjm5ReminderCreateInternal @ReminderID OUTPUT,
				@DocumentClass, @DocumentID, @ReminderSubtype, @ReminderEventTime
				
			IF @rv != 0 OR @ReminderID IS NULL
			BEGIN
				RETURN 1
			END

			--adding supervisor to [UserReminder] table
			--or updateting existing one
			IF NOT EXISTS (
				SELECT * FROM [UserReminder]
				WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
			)
			BEGIN
				IF @ReminderEventTime IS NOT NULL
				BEGIN
					SELECT 
						@ReminderInitialState = ISNULL(dbo.fnPjm5GetUserInitialReminderState(@DocumentClass, @ReminderSubtype, @UserID), 0),
						@ReminderInitialInterval = ISNULL(dbo.fnPjm5GetReminderInterval(@DocumentClass, @ReminderSubtype, @UserID), 15)
					
					INSERT INTO [UserReminder] ([ReminderID], [UserID], [State],
						[OriginalInterval], [NextInterval], [UserDefined])
					VALUES (@ReminderID, @UserID, @ReminderInitialState,
						@ReminderInitialInterval, @ReminderInitialInterval, 0)
				END
			END
			ELSE
			BEGIN
				UPDATE [UserReminder] SET
					[State] =	CASE
									WHEN @ReminderEventTime IS NOT NULL THEN [State]
									ELSE 0
								END
				WHERE [ReminderID] = @ReminderID AND [UserID] = @UserID
			END
			IF @@error != 0
			BEGIN
				RETURN 1
			END

			RETURN 0
		END

	RETURN 0
GO

GRANT EXECUTE ON [pjm6DocumentSupervisorReminderCreate] TO PUBLIC
GO