IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfPaymentCreate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfPaymentCreate'
		DROP PROCEDURE [dbo].[adfPaymentCreate]
	END
GO

PRINT 'Creating procedure adfPaymentCreate'
GO

/******************************************************************************
**		File: adfPaymentCreate.sql
**		Name: adfPaymentCreate
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: vladislav.dremin
**		Date: 01/21/2005 17:11:09
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [adfPaymentCreate]
	@Xml XML
AS
	DECLARE @State TState,
			@FileAs TName,
			@Date TDate,
			@Number TCode,
			@IncomingDate TDate,
			@IncomingNumber TCode,
			@PaymentTypeID TIdentifier,
			@PayeeCompanyID TIdentifier,
			@PayeeBankAccountID TIdentifier,
			@PayerCompanyID TIdentifier,
			@Value TMoney,
			@BaseCurrencyRate TMoney,
			@CurrencyCode TCode3,
			@VAT TMoney,
			@Comments TNotes,
			@ID TIdentifier,
			@Flag TFlag,
			@Reason TName,
			@Payer TName,
			@doc int,
			@val TVariant
	
	SET NOCOUNT ON
	
	SET @State = dbo.adfwfGetInitialState('93c2d727-32db-4dda-9bf5-a52f35ccd952')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	EXEC sp_xml_preparedocument @doc OUTPUT, @Xml
	SELECT 
		@Date = CONVERT(datetime, [XmlTable].[Date]),
		@IncomingDate = CONVERT(datetime, [XmlTable].[IncomingDate]),
		@FileAs = [XmlTable].[FileAs],
		@Number = [XmlTable].[Number],
		@IncomingNumber = [XmlTable].[IncomingNumber],
		@PaymentTypeID = [XmlTable].[PaymentTypeID],
		@CurrencyCode = [XmlTable].[CurrencyCode],
		@PayeeCompanyID = [XmlTable].[SupplierCompanyID],
		@PayeeBankAccountID = [XmlTable].[PayeeBankAccountID],
		@PayerCompanyID = [XmlTable].[CustomerCompanyID],
		@Value = [XmlTable].[Value],
		@VAT = [XmlTable].[VAT],
		@Comments = [XmlTable].[Comments],
		@Flag = [XmlTable].[Flag],
		@BaseCurrencyRate = [XmlTable].[BaseCurrencyRate],
		@Reason = [XmlTable].[Reason],
		@Payer = [XmlTable].[Payer]
	FROM
		OPENXML(@doc, '/adfPaymentGet/Payment', 2)
			WITH (
				[FileAs] nvarchar(512),
				[Date] nvarchar(19),
				[IncomingDate] nvarchar(19),
				[Number] varchar(32),
				[IncomingNumber] varchar(32),
				[PaymentTypeID] int,
				[CurrencyCode] char(3),
				[SupplierCompanyID] int,
				[PayeeBankAccountID] int,
				[CustomerCompanyID] int,
				[Reason] nvarchar(256),
				[Payer] nvarchar(256),
				[Value] money,
				[VAT] money,
				[BaseCurrencyRate] money,
				[Comments] nvarchar(max),
				[Flag] int
				) AS [XmlTable]
		--INNER JOIN [BankAccount]
		--	ON [BankAccount].[ID] = [XmlTable].[PayeeBankAccountID]
	EXEC sp_xml_removedocument @doc
	SET @Date = CAST(CONVERT(varchar(8), @Date, 112) AS DATETIME)
	SET @IncomingDate = CAST(CONVERT(varchar(8), @IncomingDate, 112) AS DATETIME)
	
	IF @PaymentTypeID = 1 --  
	BEGIN
		SET @Payer = NULL
	END
	ELSE IF @PaymentTypeID = 2 --  
	BEGIN
		SET @PayeeBankAccountID = NULL
	END
	ELSE -- Invalid payment type
	BEGIN
		RAISERROR('#internalerror', 16, -1)
		RETURN 1
	END	
	
	IF @Flag IS NULL
	BEGIN
		SET @Flag = 0
	END

	DECLARE @BaseCurrencyCode TCode3
	EXEC adfParameterGet @Name='BaseCurrency', @Group='CommonBusiness', @Value = @val OUTPUT
	SET @BaseCurrencyCode = CAST(@val as varchar(3))
	
	IF @BaseCurrencyCode = @CurrencyCode
	BEGIN
		SET @BaseCurrencyRate = 1.0
	END
		
	IF @BaseCurrencyRate IS NULL
	BEGIN
		IF(@BaseCurrencyCode = dbo.fnPjm5GetNationalCurrency())
			SELECT 
				@BaseCurrencyRate = [Rate]
			FROM
				[CurrencyRate]
			WHERE
				[FromCurrencyCode] = @CurrencyCode AND
				[ToCurrencyCode] = @BaseCurrencyCode AND
				(@Date BETWEEN [ValidFrom] AND [ValidTo])
		ELSE
			SELECT 
				@BaseCurrencyRate = [Rate]
			FROM
				[CurrencyRate]
			WHERE
				[FromCurrencyCode] = @BaseCurrencyCode AND
				[ToCurrencyCode] = @CurrencyCode AND
				(@Date BETWEEN [ValidFrom] AND [ValidTo])
	END

	BEGIN TRANSACTION
	INSERT INTO [Payment]
		([Flag], [State], 
		[FileAs], [Date], [IncomingDate], [Number], [IncomingNumber],
		[PaymentTypeID], [CurrencyCode],
		[PayeeCompanyID], [PayeeBankAccountID], [PayerCompanyID],
		[Value], [VAT], [BaseCurrencyRate],
		[Comments], [Reason], [Payer])
	VALUES
		(@Flag, @State,
		@FileAs, @Date, @IncomingDate, @Number, @IncomingNumber,
		@PaymentTypeID, @CurrencyCode,
		@PayeeCompanyID, @PayeeBankAccountID, @PayerCompanyID, 
		@Value, @VAT, @BaseCurrencyRate,
		@Comments, @Reason, @Payer)
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	SET @ID = SCOPE_IDENTITY()
	COMMIT TRANSACTION

	EXEC adfPaymentGet @ID

	RETURN 0
GO

GRANT EXECUTE ON adfPaymentCreate TO PUBLIC
GO
 