IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PjmSyncExportTaskFunction]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[PjmSyncExportTaskFunction]
GO

CREATE FUNCTION PjmSyncExportTaskFunction(
	@DeviceID varchar(40),
	@DaysBeforeThis int,
	@TimeStamp bigint,
	@CreateDeleted BIT)

RETURNS @Result TABLE
(
	[ID] TIdentifier,
	[Class] TClass,
	[ItemID] TCode,

	[Body]TNotes,
	[Categories] TNotes,
	[Complete] varchar(5),
	[DateCompleted] TDate,
	[DueDate] TDate,
	[Importance] TIdentifier,
	[ReminderTime] TDate,
	[ReminderSet] varchar(5),
	[StartDate] TDate,
	[Subject] varchar(512),
	[TimeStamp] bigint
)

AS
BEGIN
	DECLARE @EmployeeID Tidentifier

	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()

DECLARE @t TABLE
(
	[ID] TIdentifier,
	[Class] TClass,
	[ItemID] TCode,

	[Body]TNotes,
	[Categories] TNotes,
	[Complete] varchar(5),
	[DateCompleted] TDate,
	[DueDate] TDate,
	[Importance] TIdentifier,
	[ReminderTime] TDate,
	[ReminderSet] varchar(5),
	[StartDate] TDate,
	[Subject] varchar(512),
	[TimeStamp] bigint
)

INSERT INTO @t

SELECT 
	[Task].[ID],
	[Task].[Class],
	[ExternalLink].[ItemID],

	REPLACE(REPLACE([Task].[Comments], CHAR(13) + CHAR(10), CHAR(10)), CHAR(10), CHAR(13) + CHAR(10)) AS [Body],
	ISNULL([dbo].[fnPjm5CategoriesConcat] ([Task].[Class], [Task].[ID])+', ','')+ 'PjmSync' AS [Categories],
	CASE [Task].[PercentComplete] WHEN 1 THEN 'true' ELSE 'false' END AS [Complete],
	[Task].[EndDate] AS [DateCompleted],
	[Task].[DueDate] AS [DueDate],
	[Task].[PriorityID] AS [Importance],
	DATEADD(minute,-[UserReminder].[OriginalInterval],[Task].[DueDate]) AS [ReminderTime],
	CASE [UserReminder].[State] WHEN 1 THEN 'true' ELSE 'false' END AS [ReminderSet],
	[Task].[StartDate] AS [StartDate],
	[Task].[FileAs]+ISNULL(' ('+ [Project].[Number]+')', '') AS [Subject],
	CAST([Task].[TimeStamp] as bigint) AS [TimeStamp]
FROM 
	[Task]
	INNER JOIN [Project]
		ON [Task].[ProjectID] = [Project].[ID]
	INNER JOIN [TaskAssignment]
		ON [TaskAssignment].[TaskID] = [Task].[ID]
	INNER JOIN [vwAssignment]
		ON [vwAssignment].[ID] = [TaskAssignment].[AssignmentID]AND
		[vwAssignment].[EmployeeID] = @EmployeeID
	LEFT OUTER JOIN [Reminder]
		ON [Reminder].[DocumentClass] = [Task].[Class] AND
		[Reminder].[DocumentID] = [Task].[ID]
	LEFT OUTER JOIN [UserReminder]
		ON [Reminder].[ID] = [UserReminder].[ReminderID] AND
		[UserReminder].[UserID] = dbo.adfGetCurrentUserID()
	LEFT OUTER JOIN [ExternalLink]
		ON [ExternalLink].[DeviceID] = @DeviceID AND
			[ExternalLink].[DocumentID] = [Task].[ID] AND
			 [ExternalLink].[DocumentClass] = [Task].[Class]
	
WHERE [Task].[DueDate] >= DATEADD(day,-@DaysBeforeThis,GETDATE()) AND
		[Task].[State] IN (2,3,5,8)
		

UNION

SELECT 
	[TaskRequest].[ID],
	[TaskRequest].[Class],
	[ExternalLink].[ItemID],

	REPLACE(REPLACE([TaskRequest].[Comments], CHAR(13) + CHAR(10), CHAR(10)), CHAR(10), CHAR(13) + CHAR(10)) AS [Body],
	ISNULL([dbo].[fnPjm5CategoriesConcat] ([TaskRequest].[Class], [TaskRequest].[ID])+', ','')+ 'PjmSync' AS [Categories],
	CASE [TaskRequest].[PercentComplete] WHEN 1 THEN 'true' ELSE 'false' END AS [Complete],
	CASE [TaskRequest].[PercentComplete] WHEN 1 THEN [TaskRequest].[Modified] ELSE NULL END AS [DateCompleted],
	CASE WHEN [TaskRequest].[DueTime] IS NULL AND [TaskRequest].[StartTime] IS NOT NULL 
		THEN [TaskRequest].[StartTime] 
		ELSE [TaskRequest].[DueTime] 
	END  AS [DueDate],
	[TaskRequest].[PriorityID] AS [Importance],
	DATEADD(minute,-[UserReminder].[OriginalInterval],[TaskRequest].[DueTime]) AS [ReminderTime],
	CASE [UserReminder].[State] WHEN 1 THEN 'true' ELSE 'false' END AS [ReminderSet],
	[TaskRequest].[StartTime] AS [StartDate],
	[TaskRequest].[FileAs] + ISNULL(' ('+ [Project].[Number]+')', '') AS [Subject],
	CAST([TaskRequest].[TimeStamp] as bigint) AS [TimeStamp]
FROM 
	[TaskRequest]
	LEFT OUTER JOIN [Project]
		ON [TaskRequest].[ProjectID] = [Project].[ID]
	INNER JOIN [TaskRequestAssignment]
		ON [TaskRequestAssignment].[TaskRequestID] = [TaskRequest].[ID]
	INNER JOIN [Employee]
		ON [Employee].[ID] = [TaskRequestAssignment].[EmployeeID] AND
			[Employee].[ID] = @EmployeeID
	LEFT OUTER JOIN [Reminder]
		ON [Reminder].[DocumentClass] = [TaskRequest].[Class] AND
		[Reminder].[DocumentID] = [TaskRequest].[ID]
	LEFT OUTER JOIN [UserReminder]
		ON [Reminder].[ID] = [UserReminder].[ReminderID] AND
		[UserReminder].[UserID] = dbo.adfGetCurrentUserID()
	LEFT OUTER JOIN [ExternalLink]
		ON [ExternalLink].[DeviceID] = @DeviceID AND
			[ExternalLink].[DocumentID] = [TaskRequest].[ID] AND
			 [ExternalLink].[DocumentClass] = [TaskRequest].[Class]
	
WHERE ([TaskRequest].[DueTime] >= DATEADD(day,-@DaysBeforeThis,GETDATE()) OR [TaskRequest].[DueTime] IS NULL)
		AND [TaskRequest].[State] != 4
	
INSERT INTO @Result
	([ID],
	[Class],
	[ItemID],

	[Body],
	[Categories],
	[Complete],
	[DateCompleted],
	[DueDate],
	[Importance],
	[ReminderTime],
	[ReminderSet],
	[StartDate],
	[Subject],
	[TimeStamp])
SELECT
	[ID],
	[Class],
	[ItemID],

	[Body],
	[Categories],
	[Complete],
	[DateCompleted],
	[DueDate],
	[Importance],
	[ReminderTime],
	[ReminderSet],
	[StartDate],
	[Subject],
	[TimeStamp]
FROM @t
WHERE [TimeStamp] > @TimeStamp OR [ItemID] IS NULL OR @CreateDeleted=1

UNION

SELECT 
	-[ExternalLink].[DocumentID],
	[ExternalLink].[DocumentClass],
	[ExternalLink].[ItemID],

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL

FROM [ExternalLink]
	LEFT OUTER JOIN @t AS [Table]
		ON [Table].[Class]=[ExternalLink].[DocumentClass] AND
		[Table].[ID]=[ExternalLink].[DocumentID]
WHERE [Table].[ItemID] IS NULL AND 
		[ExternalLink].[DeviceID] = @DeviceID AND
	[ExternalLink].[DocumentClass] IN (dbo.adfGetClassByName('Pjm5.Task'),dbo.adfGetClassByName('Pjm5.TaskRequest'))

UNION

SELECT 
		-[ExternalLink].[DocumentID],
		[ExternalLink].[DocumentClass],
		[ExternalLink].[ItemID],
		
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
FROM 
		[ExternalLink]
			INNER JOIN [TaskRequest]		
			ON [TaskRequest].[Class] = [ExternalLink].[DocumentClass] AND
				[TaskRequest].[ID] = [ExternalLink].[DocumentID]	
	WHERE [TaskRequest].[DueTime] < DATEADD(day,-@DaysBeforeThis,GETDATE()) AND
			@DeviceID = [ExternalLink].[DeviceID]

	UNION

	SELECT 
		-[ExternalLink].[DocumentID],
		[ExternalLink].[DocumentClass],
		[ExternalLink].[ItemID],
		
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	FROM 
		[ExternalLink]
		INNER JOIN [Task]
			ON [Task].[Class] = [ExternalLink].[DocumentClass] AND
				[Task].[ID] = [ExternalLink].[DocumentID]
	WHERE [Task].[DueDate] < DATEADD(day,-@DaysBeforeThis,GETDATE()) AND
			@DeviceID = [ExternalLink].[DeviceID] 
	RETURN
END 
GO

