IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5ProjectBrowseByCategory]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5ProjectBrowseByCategory'
		DROP PROCEDURE [dbo].[pjm5ProjectBrowseByCategory]
	END
GO

PRINT 'Creating procedure pjm5ProjectBrowseByCategory'
GO

/******************************************************************************
**		File: ProjectBrowseByCategory.sql
**		Name: pjm5ProjectBrowseByCategory
**		Desc:     / 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**      31.03.09	ENB				 ,    
**									:     .
**									    
**
*******************************************************************************/
CREATE PROCEDURE [pjm5ProjectBrowseByCategory]
	@StateSelector INT,
	@CustomerStateSelector INT,
	@DateSelector	INT,
	@PeriodStart	datetime,
	@PeriodEnd		datetime,
	@EmployeeID		TIdentifier
AS

	SET NOCOUNT ON

--         
	IF dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllProjects')) = 0 BEGIN
		RAISERROR('#NotEnoughPermissions', 18, -1)
		RETURN 1
	END

	DECLARE 
		@date datetime
		
	DECLARE @none nvarchar(6)
	IF dbo.adfGetLanguage() = 'ru'
		SET @none = '()'
	ELSE
		SET @none = '(none)'
		
	SET @date = GETDATE()
	IF @DateSelector = 1
		SELECT
			@PeriodStart = CAST(CAST(YEAR(@date) AS varchar(4)) + '0101' AS datetime),
			@PeriodEnd = DATEADD(year, 1, @PeriodStart)
	IF @DateSelector = 2
		SELECT
			@PeriodStart = CAST(CAST(YEAR(@date)-1 AS varchar(4)) + '0101' AS datetime),
			@PeriodEnd = DATEADD(year, 1, @PeriodStart)
	IF @DateSelector = 3
		SELECT
			@PeriodStart = CAST(CONVERT(varchar(8), @PeriodStart, 112) AS datetime),
			@PeriodEnd = DATEADD(day, 1, CAST(CONVERT(varchar(8), @PeriodEnd, 112) AS datetime))
	
	SELECT
		[Project].[Class],
		[Project].[ID],
		[Project].[State],
		[Project].[Flag],
		[Project].[Date],
		[Project].[Number],
		[Project].[FileAs],
		[Project].[ProjectTypeID],
		[Project].[SupplierCompanyID],
		[Project].[ProjectManagerEmployeeID],
		[Project].[CustomerCompanyID],
		[Project].[CustomerFinanceManagerID],
		[Project].[ResponsiblePersonEmployeeID],
		[Project].[PriorityID],
		[ProjectDetail].[DueDate],
		[Project].[EndDate],
		[Project].[CurrencyCode],
		[Project].[CostCenterID],
		[ProjectDetail].[PlannedManHour] / 60.0 AS [ManHour],
		CASE
			WHEN [Projects].[ViewFinance] = 1 
			THEN [Project].[Budget]
			ELSE -1
		END AS [Budget],
		[ProjectDetail].[PlannedManHour],
		CASE
			WHEN [Projects].[ViewFinance] = 1
			THEN [ProjectDetail].[PlannedBudget]
			ELSE -1
		END AS [PlannedBudget],
		[Project].[Billable],
		[Project].[RoundingInterval],
		[Project].[RoundingMode],
		[Project].[ContractNumber],
		[Project].[ContractDate],
		[Project].[ContractEndDate],
		[Project].[ContractSubject],
		[Project].[ServiceLevelID],
		[Project].[ApplyVAT],
		[Project].[InvoiceExpirationPeriod],
		[Project].[InvoiceExpirationPeriodType],
		[Project].[Comments],
		[Project].[Created],
		[Project].[CreatedBy],
		[Project].[Modified],
		[Project].[ModifiedBy],
		[Project].[ExchangeSystemName],
		[Project].[Timestamp],
		[ProjectDetail].[PercentComplete],
		(CAST([ProjectDetail].[PercentComplete] * 100 as int) / 25) * 25 as PC,
		ISNULL([Category].[FileAs], @none) as [Category],
		[Project].[PaymentCurrencyPoint],
		[Project].[IsConsignor],
		[BusinessCard].[FileAs] as 'Consignor',
		[Project].[ConsignorContract]
	FROM
		[Project]
		INNER JOIN pjm6GetAvailableProjectsForBrowse (
			@EmployeeID,		-- ID    
			@StateSelector		--  
		) AS [Projects]
			ON [Project].[ID] = [Projects].[ID]
		INNER JOIN [ProjectDetail]
			ON [ProjectDetail].[ID] = [Project].[ID]
		LEFT OUTER JOIN [DocumentCategory]
			ON ([Project].[Class] = [DocumentCategory].[DocumentClass] AND [Project].[ID] = [DocumentCategory].[DocumentID])
		LEFT OUTER JOIN [Category]
			ON ([DocumentCategory].[DocumentClass] = [Category].[DocumentClass] AND [DocumentCategory].[CategoryID] = [Category].[ID])
		LEFT OUTER JOIN [BusinessCard]
			ON [BusinessCard].[Class] = [Project].[ConsignorClass] AND [BusinessCard].[ID] = [Project].[ConsignorID]

	WHERE
		( 
			(@CustomerStateSelector = 0) OR 
			(@CustomerStateSelector = 1 AND [Project].[ProjectTypeID] = 1) OR
			(@CustomerStateSelector = 2 AND [Project].[ProjectTypeID] = 2) OR
			([Project].[ProjectTypeID] NOT IN (1, 2) AND @CustomerStateSelector = 3)
		) AND
		(
			@DateSelector = 0 OR
			([Project].[Date] < @PeriodEnd AND ISNULL([Project].[EndDate], @date) > @PeriodStart)
		) 
		
	RETURN 0
GO

GRANT EXECUTE ON [pjm5ProjectBrowseByCategory] TO PUBLIC
GO

