IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ProjectBrowseMemberActive')
	BEGIN
		PRINT 'Dropping Procedure ProjectBrowseMemberActive'
		DROP  Procedure  ProjectBrowseMemberActive
	END

GO

PRINT 'Creating Procedure ProjectBrowseMemberActive'
GO
/******************************************************************************
**		File: 
**		Name: ProjectBrowseMemberActive
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure ProjectBrowseMemberActive
AS
	SET NOCOUNT ON
	
	DECLARE
		@PeriodStart TDate,
		@PeriodEnd TDate
	SELECT
		@PeriodStart = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
		@PeriodEnd = DATEADD(day, 1, @PeriodStart)
	
	DECLARE 
		@ViewAllFinance TBool,
		@CurrentEmployeeID TIdentifier,
		@HasManageProject TBool
		
	SET @HasManageProject = dbo.adfHasAccessRight(dbo.adfGetRightID('ManageProject'))
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance')) |
		dbo.adfHasAccessRight(dbo.adfGetRightID('ViewFinance'))
	
	SELECT
		[Project].[Class],
		[Project].[ID],
		[Project].[State],
		[Project].[Flag],
		[Project].[Date],
		[Project].[Number],
		[Project].[FileAs],
		[Project].[ProjectTypeID],
		[Project].[SupplierCompanyID],
		[Project].[ProjectManagerEmployeeID],
		[Project].[CustomerCompanyID],
		[Project].[CustomerFinanceManagerID],
		[Project].[ResponsiblePersonEmployeeID],
		[Project].[PriorityID],
		[ProjectDetail].[DueDate],
		[Project].[EndDate],
		[Project].[CurrencyCode],
		[Project].[CostCenterID],
		[ProjectDetail].[PlannedManHour] / 60.0 AS [ManHour],
		CASE
			WHEN @ViewAllFinance = 1 OR 
				(@HasManageProject = 1 AND [Project].[ProjectManagerEmployeeID] = @CurrentEmployeeID)
				THEN [Project].[Budget]
			ELSE -1
		END AS [Budget],
		[ProjectDetail].[PlannedManHour],
		CASE
			WHEN @ViewAllFinance = 1 OR 
				(@HasManageProject = 1 AND [Project].[ProjectManagerEmployeeID] = @CurrentEmployeeID)
				THEN [ProjectDetail].[PlannedBudget]
			ELSE -1
		END AS [PlannedBudget],
		[Project].[Billable],
		[Project].[RoundingInterval],
		[Project].[RoundingMode],
		[Project].[ContractNumber],
		[Project].[ContractDate],
		[Project].[ContractEndDate],
		[Project].[ContractSubject],
		[Project].[ServiceLevelID],
		[Project].[ApplyVAT],
		[Project].[InvoiceExpirationPeriod],
		[Project].[InvoiceExpirationPeriodType],
		[Project].[Comments],
		[Project].[Created],
		[Project].[CreatedBy],
		[Project].[Modified],
		[Project].[ModifiedBy],
		[Project].[Timestamp],
		[ProjectDetail].[PercentComplete],
		(CAST([ProjectDetail].[PercentComplete] * 100 as int) / 25) * 25 as PC,
		[Project].[PaymentCurrencyPoint],
		[PaymentPeriod].[InvoiceNextPaymentDate],
		[PaymentPeriod].[StatementNextPaymentDate],
		[PaymentPeriod].[CalculationNextPaymentDate],
		[Project].[IsConsignor],
		[BusinessCard].[FileAs] as 'Consignor',
		[Project].[ConsignorContract],
		[SubProjects].[ProjectID] AS [ParentProjectID]
	FROM
		[Project]
		INNER JOIN [Assignment]
			ON [Assignment].[ProjectID] = [Project].[ID]
			AND [Assignment].[State] = 1
		INNER JOIN [AssignmentEntry]
			ON [AssignmentEntry].[AssignmentID] = [Assignment].[ID]
			AND [AssignmentEntry].[EmployeeID] = @CurrentEmployeeID 
		INNER JOIN [PaymentPeriod]
			ON [PaymentPeriod].[ProjectID] = [Project].[ID]
		INNER JOIN [ProjectDetail]
			ON [ProjectDetail].[ID] = [Project].[ID]
		LEFT OUTER JOIN [Company] AS [CustomerCompany]
			ON [CustomerCompany].[ID] = [Project].[CustomerCompanyID]
		LEFT OUTER JOIN [BusinessCard]
			ON [BusinessCard].[Class] = [Project].[ConsignorClass] AND [BusinessCard].[ID] = [Project].[ConsignorID]
		LEFT OUTER JOIN fnpjm6GetSubprojectsForBrowse() AS [SubProjects]
			ON [SubProjects].[SubprojectID] = [Project].[ID]
	WHERE
		[Project].[State]= 2 OR 
		[Project].[State] = 4 AND 
		[Project].[EndDate] >= @PeriodStart AND [Project].[EndDate] < @PeriodEnd
		
	RETURN 0
GO

GRANT EXEC ON ProjectBrowseMemberActive TO PUBLIC
GO 