IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm6ProjectBrowseMemberExpire')
	BEGIN
		PRINT 'Dropping Procedure pjm6ProjectBrowseMemberExpire'
		DROP  Procedure  pjm6ProjectBrowseMemberExpire
	END

GO

PRINT 'Creating Procedure pjm6ProjectBrowseMemberExpire'
GO
/******************************************************************************
**		File: pjm6ProjectBrowseMemberExpire.sql
**		Name: pjm6ProjectBrowseMemberExpire
**		Desc:     
""			  /C  /   
**
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**      01.04.09	ENB				 
**
**    
*******************************************************************************/
CREATE Procedure pjm6ProjectBrowseMemberExpire

AS
	SET NOCOUNT ON
	
	DECLARE 
		@date				datetime

	SET @date = GETDATE()
	
	SELECT
		[Project].[Class],
		[Project].[ID],
		[Project].[State],
		[Project].[Flag],
		[Project].[Date],
		[Project].[Number],
		[Project].[FileAs],
		[Project].[ProjectTypeID],
		[Project].[SupplierCompanyID],
		[Project].[ProjectManagerEmployeeID],
		[Project].[CustomerCompanyID],
		[Project].[CustomerFinanceManagerID],
		[Project].[ResponsiblePersonEmployeeID],
		[Project].[PriorityID],
		[ProjectDetail].[DueDate],
		[Project].[EndDate],
		[Project].[CurrencyCode],
		[Project].[CostCenterID],
		[ProjectDetail].[PlannedManHour] / 60.0 AS [ManHour],
		CASE
			WHEN [Projects].[ViewFinance] = 1
			THEN [Project].[Budget]
			ELSE -1
		END AS [Budget],
		[ProjectDetail].[PlannedManHour],
		CASE
			WHEN [Projects].[ViewFinance] = 1
			THEN [ProjectDetail].[PlannedBudget]
			ELSE -1
		END AS [PlannedBudget],
		dbo.fnPjm6GetProjectManHour([Project].[ID]) / 60.0 AS [RealManHour],
		dbo.fnPjm6GetProjectBudget([Project].[ID]) AS [RealBudget],
		[Project].[Billable],
		[Project].[RoundingInterval],
		[Project].[RoundingMode],
		[Project].[ContractNumber],
		[Project].[ContractDate],
		[Project].[ContractEndDate],
		[Project].[ContractSubject],
		[Project].[ServiceLevelID],
		[Project].[ApplyVAT],
		[Project].[InvoiceExpirationPeriod],
		[Project].[InvoiceExpirationPeriodType],
		[Project].[Comments],
		[Project].[Created],
		[Project].[CreatedBy],
		[Project].[Modified],
		[Project].[ModifiedBy],
		[Project].[Timestamp],
		(
			SELECT 
				ISNULL(SUM(dbo.fnPjm5GetDeviationByTask([Task].[ID],@date)), 0)
			FROM 
				[Task]
			WHERE 
				[Task].[State] IN (1,2) AND [Task].[ProjectID] = [Project].[ID]
		) AS [ExpireDays],
		[ProjectDetail].[PercentComplete],
		(CAST([ProjectDetail].[PercentComplete] * 100 as int) / 25) * 25 as PC,
		[Project].[PaymentCurrencyPoint],
		[Project].[ExchangeSystemName],
		[Project].[IsConsignor],
		[BusinessCard].[FileAs] as 'Consignor',
		[Project].[ConsignorContract],
		[Projects].[ParentProjectID] AS [ParentProjectID]
	FROM
		[Project]
		INNER JOIN fnPjm6GetAvailableProjectsForProjectMember (
		) AS [Projects]
			ON [Project].[ID] = [Projects].[ID]
		INNER JOIN [ProjectDetail]
			ON [ProjectDetail].[ID] = [Project].[ID]
		LEFT OUTER JOIN [BusinessCard]
			ON [BusinessCard].[Class] = [Project].[ConsignorClass] AND [BusinessCard].[ID] = [Project].[ConsignorID]
	WHERE
		CAST(dbo.fnPjm5GetExpireDaysProject([Project].[ID], @date) as int) > 0
		
	RETURN 0
GO

GRANT EXEC ON pjm6ProjectBrowseMemberExpire TO PUBLIC
GO

