IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportRevenueByCostCenter')
	BEGIN
		PRINT 'Dropping Procedure ReportRevenueByCostCenter'
		DROP  Procedure  ReportRevenueByCostCenter
	END

GO

PRINT 'Creating Procedure ReportRevenueByCostCenter'
GO

CREATE Procedure ReportRevenueByCostCenter
		@CompanyID TIdentifier,
		@DateSelector int,
		@QuartalSelector int,
		@PeriodStart TDate,
		@PeriodEnd TDate,
		@InvoiceSelector int,
		@ServiceTypeSelector int
		
AS
	SET NOCOUNT ON
	
	IF @CompanyID IS NULL
	BEGIN
		RAISERROR('#CompanyIsNotSelected', 16, -1)
		RETURN 1
	END
	
	DECLARE
		@UserID TIdentifier,
		@UserName TName,
		@Sum float,
		@InvoiceState TName,
		@Value TVariant,
		@ServiceType TName
		
	EXEC adfParameterGet 'BaseCurrency', 'CommonBusiness', @Value OUTPUT
		
	IF @DateSelector = 0
		BEGIN
			SET @PeriodStart = CAST(YEAR(GETDATE()) AS char(4))+'0101'
			SET @PeriodEnd = CAST((YEAR(GETDATE()) + 1) AS char(4))+'0101'
		END
	ELSE IF @DateSelector = 1
		BEGIN
			SET @PeriodStart = CAST((YEAR(GETDATE()) - 1) AS char(4))+'0101'
			SET @PeriodEnd = CAST(YEAR(GETDATE()) AS char(4))+'0101'
		END 
		ELSE SET @PeriodEnd = DATEADD(day, 1, @PeriodEnd)
	IF @QuartalSelector = 2 SET @PeriodStart = DATEADD(month, 3, @PeriodStart)
	IF @QuartalSelector = 3 SET @PeriodStart = DATEADD(month, 6, @PeriodStart)
    IF @QuartalSelector = 4 SET @PeriodStart = DATEADD(month, 9, @PeriodStart)
	IF @QuartalSelector != 0 SET @PeriodEnd = DATEADD(month, 3, @PeriodStart)
	SET @PeriodEnd = DATEADD(day, -1, @PeriodEnd)
	
	IF @ServiceTypeSelector = 0 SET @ServiceType = '    '
	IF @ServiceTypeSelector = 1 SET @ServiceType = ''
	IF @ServiceTypeSelector = 2 SET @ServiceType = '  '
	
	IF @InvoiceSelector = 0
		SET @InvoiceState = ' '
	ELSE 
		SET @InvoiceState = ' '
		
		SET @UserID = dbo.adfGetCurrentUserID()
	SELECT
		@UserName = [Trustee].[DisplayName]
	FROM
		[Trustee]
	WHERE
		[Trustee].[ID] = @UserID	
		
	DECLARE @CostCenter TABLE([CostCenterID] TIdentifier, [CostCenterName] TName NULL, [Value] TMoney NULL)
	INSERT INTO @CostCenter([CostCenterID], [CostCenterName], [Value]) 
	SELECT 
		[Project].[CostCenterID],
		ISNULL([CostCenter].[FileAs],'( )'),
		SUM ([InvoiceLine].[TotalWithVATBC])
	FROM
		[Project]
		LEFT OUTER JOIN [CostCenter]
			ON [CostCenter].[ID] = [Project].[CostCenterID]
		INNER JOIN [Invoice]
			ON [Project].[ID] = [Invoice].[ProjectID]
		INNER JOIN [InvoiceLine]
			ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
		LEFT OUTER JOIN [ServiceType]
			ON [ServiceType].[ID] = [InvoiceLine].[ServiceTypeID]
		LEFT OUTER JOIN [AdvanceInvoice]
			ON [AdvanceInvoice].[AdvanceInvoiceID] = [Invoice].[ID]
		LEFT OUTER JOIN [Invoice] AS [BaseInvoice]
			ON [AdvanceInvoice].[InvoiceID] = [BaseInvoice].[ID]
	WHERE
		[Invoice].[SupplierCompanyID] = @CompanyID AND
		([BaseInvoice].[State] NOT IN (2,5) OR [BaseInvoice].[State] IS NULL) AND
		[Invoice].[Date] BETWEEN @PeriodStart AND @PeriodEnd 
		AND
		(
		 @InvoiceSelector = 0 AND [Invoice].[State] = 5 OR
		 @InvoiceSelector = 1 AND [Invoice].[State] IN (2,5)
		)
		AND
		(
			@ServiceTypeSelector = 0 OR
			@ServiceTypeSelector = 1 AND [ServiceType].[RatesDepended] = 1 OR
			@ServiceTypeSelector = 2 AND [ServiceType].[RatesDepended] = 0
		)
	GROUP BY
		[Project].[CostCenterID],
		[CostCenter].[FileAs]

	SELECT @Sum = SUM([Value])FROM @CostCenter
	DECLARE @Result TABLE([SortID] int, [CostCenterID] TIdentifier, [CostCenterName] TName NULL, [Value] TMoney NULL)
	INSERT INTO @Result([SortID], [CostCenterID], [CostCenterName], [Value])
	SELECT TOP(6)
		1,
		[CostCenterID], 
		[CostCenterName], 
		[Value]
	FROM
		 @CostCenter
	ORDER BY [Value] DESC 

	DELETE FROM @CostCenter
	FROM
		@CostCenter [C]
		INNER JOIN @Result [R]
			ON ([C].[CostCenterID] = [R].[CostCenterID]OR
				[C].[CostCenterID] IS NULL AND [R].[CostCenterID] IS NULL)

	IF EXISTS(SELECT * FROM @CostCenter)
	BEGIN
	INSERT INTO @Result([SortID],[CostCenterID], [CostCenterName], [Value])
	SELECT
		2,
		NULL,
		'()',
		SUM([Value])
	FROM 
		@CostCenter
	END
			
	SELECT 
		CASE WHEN @QuartalSelector != 0 THEN (CAST(@QuartalSelector AS Char(1)) + '  ') ELSE '' END + CAST(YEAR(@PeriodStart) AS Char(4)) + ' .' AS [Year],
		@UserName AS [UserName],
		@InvoiceState AS [InvoiceState],
		CAST(@Value as nvarchar(32)) AS [CurrencyCode],
		@ServiceType as [ServiceType]

	SELECT 
		[CostCenterName], 
		[Value],
		[Value]/@Sum AS [Percent]
	FROM 
		@Result
	ORDER BY 
		[SortID], 
		[Value] DESC  
	
RETURN 0

GO

GRANT EXEC ON ReportRevenueByCostCenter TO PUBLIC

GO
