IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportBillOfProject')
	BEGIN
		PRINT 'Dropping Procedure ReportBillOfProject'
		DROP  Procedure  ReportBillOfProject
	END

GO

PRINT 'Creating Procedure ReportBillOfProject'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/
CREATE Procedure ReportBillOfProject
	@Date				TDatetime,
	@DateSelector		int,
	@ProjectSelector	int,
	@ActiveProjectID	TIdentifier,
	@NotActiveProjectID TIdentifier
	
AS
	SET NOCOUNT ON
	
	IF @ProjectSelector = 0 AND @ActiveProjectID IS NULL
	BEGIN
		RAISERROR('#ProjectIsNotSelected', 16, -1)
		RETURN 1
	END
	
	IF @ProjectSelector = 1 AND @NotActiveProjectID IS NULL
	BEGIN
		RAISERROR('#ProjectIsNotSelected', 16, -1)
		RETURN 1
	END
	
	DECLARE 
		@ProjectID TIdentifier,
		@TodayDate TDatetime,
		@value1 TVariant,
		@value2 TVariant,
		@EmployeeExpenceCode varchar(256),
		@EmployeeExpenceName varchar(256)
		
	SELECT @TodayDate = GETDATE()
	
	IF @DateSelector = 2 AND @Date > @TodayDate
	BEGIN
		RAISERROR('#CannotGetFutureReport', 16, -1)
		RETURN 1
	END	

	SET @ProjectID = CASE @ProjectSelector
			WHEN 0 THEN @ActiveProjectID
			WHEN 1 THEN @NotActiveProjectID
			END	
	
	SET @Date = CASE @DateSelector
			WHEN 0 THEN	CAST(CONVERT(varchar(8), @TodayDate, 112) AS DATETIME)
			WHEN 1 THEN	DATEADD( Day, -1, CAST(CONVERT(varchar(8), @TodayDate, 112) AS DATETIME))
			ELSE CAST(CONVERT(VARCHAR(8), @Date, 112) AS DATETIME)
			END
	
	EXEC adfParameterGet 'EmployeeExpenceCode', 'Pjm5', @value1 OUTPUT  
	SET @EmployeeExpenceCode = CAST(@value1 AS varchar(256))
	EXEC adfParameterGet 'EmployeeExpenceName', 'Pjm5', @value2 OUTPUT  
	SET @EmployeeExpenceName = CAST(@value2 AS varchar(256))			
		
	EXECUTE ReportBillOfProjectHeader @ProjectID, @Date
	
	SELECT
		[ExpenseCode].[Code] AS [ExpenseGroupCode],
		[ExpenseCode].[FileAs] AS [ExpenseGroupName],
		[Expense].[FileAs] AS [ExpenseName],
		[Expense].[Amount],
		CASE WHEN [Project].[RatesWithVAT] = 0
				THEN [Expense].[Billable] * [Expense].[TotalPC]
			  ELSE
				[Expense].[Billable] * [Expense].[TotalWithVATPC]
		END AS [Sum]	 		
	FROM
		[ExpenseCode]
		INNER JOIN	[Expense]
			ON [Expense].[ExpenseCodeID] = [ExpenseCode].[ID]
		INNER JOIN [Project]
			ON [Expense].[ProjectID] = [Project].[ID]	
	WHERE
		[ExpenseCode].[State] = 1 AND		
		[Expense].[ProjectID] = @ProjectID AND
		CAST(CONVERT(VARCHAR(8), [Expense].[Date], 112) AS DATETIME) <= @Date AND
		[Expense].[State] IN (2,3,6)
	UNION ALL
	SELECT
		@EmployeeExpenceCode,
		@EmployeeExpenceName,
		[Employee].[FileAs],
		ISNULL(SUM([TimeEntry].[BillableDuration] * [TimeEntry].[Billable] / 60.00),0),
		(ISNULL([TimeEntry].[Rate],0) * ISNULL(SUM([TimeEntry].[BillableDuration] * [TimeEntry].[Billable] / 60.00),0))				
	FROM
		[Employee] INNER JOIN [vwTimeEntry] as [TimeEntry]
			ON ([TimeEntry].[EmployeeID] = [Employee].[ID])
	WHERE
		[TimeEntry].[ProjectID] = @ProjectID AND
		CAST(CONVERT(VARCHAR(8), [TimeEntry].[StartTime], 112) AS DATETIME) <= @Date AND
		[TimeEntry].[State] IN (2,3,6)
	GROUP BY
		[Employee].[FileAs],
		[TimeEntry].[Rate]
	UNION  ALL
	SELECT
		' ',
		NULL,
		[Expense].[FileAs],
		[Expense].[Amount],
		CASE WHEN [Project].[RatesWithVAT] = 0
				THEN [Expense].[Billable] * [Expense].[TotalPC]
			  ELSE
				[Expense].[Billable] * [Expense].[TotalWithVATPC]				
		END		
	FROM
		[Expense]
		INNER JOIN [Project]
			ON [Expense].[ProjectID] = [Project].[ID]
	WHERE
		[Expense].[ProjectID] = @ProjectID AND
		[Expense].[ExpenseCodeID] IS NULL AND
		CAST(CONVERT(VARCHAR(8), [Expense].[Date], 112) AS DATETIME) <= @Date AND
		[Expense].[State] IN (2,3,6)
	UNION ALL
	SELECT
		NULL,
		NULL,
		NULL,
		0,
		0	
	

RETURN 0
GO

GRANT EXEC ON ReportBillOfProject TO PUBLIC

GO
