IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportControlOfTimeLoss')
	BEGIN
		PRINT 'Dropping Procedure ReportControlOfTimeLoss'
		DROP  Procedure  ReportControlOfTimeLoss
	END

GO

PRINT 'Creating Procedure ReportControlOfTimeLoss'
GO

/******************************************************************************
**		File: ReportControlOfTimeLoss.sql
**		Name: ReportControlOfTimeLoss
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAN
**		Date: 06/08/2010 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		06.08.2010	MAN				First Version
**		23.09.2010	MAN				   
*******************************************************************************/
CREATE Procedure ReportControlOfTimeLoss
	@CompanyID TIdentifier,
	@DivisionID TIdentifier,
	@TypeSelector int,
	@PeriodStart TDateTime,
	@PeriodEnd TDateTime,
	@EmployeeState TIdentifier
AS
	SET NOCOUNT ON

DECLARE
	@CompanyFileAS TName,
	@DivisionFileAs TName,
	@PeriodName TName,
	@Today TDateTime,
	@UserId TIdentifier,
	@UserName TName


	
--  
IF  dbo.adfHasAccessRightByName('ViewControlOfTimeLossReport') = 0 
BEGIN
      RAISERROR('#NotEnoughPermissions', 18, -1)
      RETURN 1
END

--      
DECLARE @TableTimeDraft TABLE 
(
	ID int,
	AmountDraft int,
	DurationDraft int
)
--      
DECLARE @TableTimeModificationRequired TABLE 
(
	ID int,
	AmountModificationRequired int,
	DurationModificationRequired int
)

--      
DECLARE @TableTimePublished TABLE 
(
	ID int, 
	DurationPublished int, 
	AmountPublished int
)

SET @EmployeeState = @EmployeeState+1
--1  
--2  

SET @Today = GetDate()
IF @PeriodEnd<@Today
	SET @Today=@PeriodEnd

SET @UserID = dbo.adfGetCurrentEmployeeID()
SELECT 
   @UserName=[Employee].[FileAs]
FROM
	[Employee]
WHERE
	[Employee].[ID] = @UserID
	

SELECT
	@CompanyFileAs = [Company].[FileAs]
FROM
	[Company]
WHERE
	[Company].[ID] = @CompanyID	

SELECT
	@DivisionFileAs = [Division].[FileAs]
FROM
	[Division]
WHERE
	[Division].[ID] = @DivisionID	

--       ""	
INSERT INTO @TableTimeDraft
(
	ID,
	AmountDraft,
	DurationDraft
	
)
SELECT 
	[TimeEntry].[EmployeeID] as [ID],
	COUNT(*) AS [AmountDraft],
	SUM([TimeEntry].[Duration]) as [DurationDraft]
FROM 
	[TimeEntry] 
WHERE  
	[TimeEntry].[State] = 1 
	AND  
	(
		EXISTS
		(
			SELECT 	
				[Employee].[Id]
			FROM 
				[Employee]
			WHERE
				[Employee].[Id] = [TimeEntry].[EmployeeID]
				AND
				(
					(
						@CompanyID IS NULL 
						AND
						[Employee].[CompanyID] IN
						(
							SELECT
								[Company].[ID]
							FROM
								[Company]
							WHERE
								[Company].[Own] = 1	
						)
					)	
					OR
					(
						[Employee].[CompanyID] = @CompanyID
						AND
						@CompanyID IS NOT NULL
					)
				)				
		)
	)
	AND
	(
		@DivisionID IS NULL
		OR
		[TimeEntry].[EmployeeID] IN
		(
			SELECT 	
				[Employee].[Id]
			FROM 
				[Employee]
			WHERE
				[Employee].[DivisionID] = @DivisionID
		)
	)
	AND
	(
		DATEADD(mi,[TimeEntry].[Duration],StartTime) BETWEEN @PeriodStart AND @Today
	)
GROUP BY
	[TimeEntry].[EmployeeID]

INSERT INTO @TableTimeModificationRequired
(
	ID,
	AmountModificationRequired,
	DurationModificationRequired
	
)
SELECT 
	[TimeEntry].[EmployeeID] as [ID],
	COUNT(*) AS [AmountModificationRequired],
	SUM([TimeEntry].[Duration]) as [DurationModificationRequired]
FROM 
	[TimeEntry] 
WHERE  
	[TimeEntry].[State] = 5 
	AND  
	(
		[TimeEntry].[ProjectID] in 
		(
			SELECT 
				[Project].[ID] 
			FROM 
				[Project] 
			WHERE
			(
				@CompanyID IS NOT NULL 
				AND
				[Project].[SupplierCompanyId] = @CompanyID 
			)
			OR
			(
				@CompanyID IS NULL
				AND 
				[Project].[SupplierCompanyId] IN
						(
							SELECT
								[Company].[ID]
							FROM
								[Company]
							WHERE
								[Company].[Own] = 1
						)
				)			
		)	
	)
	AND
	(
		@DivisionID IS NULL
		OR
		[TimeEntry].[EmployeeID] IN
		(
			SELECT 	
				[Employee].[Id]
			FROM 
				[Employee]
			WHERE
				[Employee].[DivisionID] = @DivisionID
				
		)
	)
	AND
	(
		DATEADD(mi,[TimeEntry].[Duration],StartTime) BETWEEN @PeriodStart AND @Today
	) 
GROUP BY 
	[TimeEntry].[EmployeeID]

INSERT INTO @TableTimePublished
(
	ID,
	DurationPublished,
	AmountPublished
)
SELECT 
	[TimeEntry].[ProjectID] as [ID],
	SUM([TimeEntry].[Duration]) as [DurationPublished],
	Count(*) as [AmountPublished]
FROM 
	[TimeEntry]
WHERE
	[TimeEntry].[State] = 2
	AND
	[TimeEntry].[ProjectID] in 
	(
		SELECT 
			[Project].[ID] 
		FROM 
			[Project] 
		WHERE
		(
			@CompanyID IS NOT NULL 
			AND
			[Project].[SupplierCompanyId] = @CompanyID 
		)
		OR
		(
			@CompanyID IS NULL
			AND 
			[Project].[SupplierCompanyId] IN
					(
						SELECT
							[Company].[ID]
						FROM
							[Company]
						WHERE
							[Company].[Own] = 1
					)
			)			
	)
	AND
	(
		@DivisionID is NULL
		OR
		[TimeEntry].[EmployeeID] IN
		(
			SELECT 	
				[Employee].[Id]
			FROM 
				[Employee]
			WHERE
				[Employee].[DivisionID] = @DivisionID
		)
	)
	AND
	(
		DATEADD(mi,[TimeEntry].[Duration],StartTime) BETWEEN @PeriodStart AND @Today
	)
GROUP BY 
	[TimeEntry].[ProjectID]

--  
IF @TypeSelector = 1
	EXEC Pjm6GetTimeIntervalFormatted @PeriodStart, @PeriodEnd, 'ru', @Result = @PeriodName  OUTPUT
ELSE
	SET @PeriodName = CONVERT(varchar, @PeriodEnd,4)

SELECT 
	@CompanyFileAS as [CompanyFileAS], 
	@DivisionFileAs as [DivisionFileAS],
	@TypeSelector as [TypeSelector],
	@PeriodName as [PeriodName],
	@UserName as [UserName]

SELECT 
	[Employee].[FileAs] as [EmployeeFileAs],
	[TimeDetail].[AmountDraft] as [AmountDraft],
	[TimeDetail].[DurationDraft]/60.0 as [DurationDraft],
	[TimeDetail].[AmountModificationRequired] as [AmountModificationRequired],
	[TimeDetail].[DurationModificationRequired]/60.0 as [DurationModificationRequired]
FROM
	[Employee]
	INNER JOIN 
	(
		SELECT
			[@TableTimeDraft].[ID],
			[@TableTimeDraft].[AmountDraft],
			[@TableTimeDraft].[DurationDraft],
			[@TableTimeModificationRequired].[ID] as [TableTimeModificationRequiredId],
			[@TableTimeModificationRequired].[AmountModificationRequired],
			[@TableTimeModificationRequired].[DurationModificationRequired]
			
		FROM	
			@TableTimeDraft 
			FULL OUTER JOIN @TableTimeModificationRequired 
				ON [@TableTimeModificationRequired].[ID] = [@TableTimeDraft].[ID]
	) as [TimeDetail]
		ON [Employee].[ID] = [TimeDetail].[ID]
		OR
		[TimeDetail].[TableTimeModificationRequiredId] = [Employee].[ID]
	WHERE
		[Employee].[State]=@EmployeeState
		OR
		@EmployeeState=0

SELECT 
	[Employee].[FileAs] as [ProjectManagerFileAs],
	[Project].[Number] as [ProjectNumber],
	[@TableTimePublished].[DurationPublished]/60.0 as [DurationPublished], 
	[@TableTimePublished].[AmountPublished] as [AmountPublished]
FROM 
	[Project]
	INNER JOIN [Employee]
	ON [Employee].[ID] = [Project].[ProjectManagerEmployeeID]
	INNER JOIN @TableTimePublished
	ON [@TableTimePublished].[ID] = [Project].[ID]
	WHERE
	(
		[Employee].[State]=@EmployeeState
		OR
		@EmployeeState=0
	)

SELECT 
	[Employee].[FileAs] as [OriginatorFileAs],
	[MeetingRequest].[StartTime] as [MeetingRequestStartTime],
	(
		SELECT 
			COUNT(*) 
		FROM 
			[Attendee] 
		WHERE 
			[Attendee].[MeetingRequestID]=[MeetingRequest].[ID]
	)+1 as [AmountMeetingRequest], -- +1  
	[MeetingRequest].[Duration] * 
	(
		(
			(
			SELECT 
				COUNT(*) 
			FROM 
				[Attendee] 
			WHERE 
				[Attendee].[MeetingRequestID]=[MeetingRequest].[ID]
			)
		+1)
	)/60.0 as [DurationMeetingRequest]
FROM 
	[MeetingRequest],[Employee]
WHERE 
	(
		ProjectID in 
		(
			SELECT 
				[Project].[ID] 
			FROM 
				[Project] 
			WHERE 
			(
				@CompanyID IS NOT NULL 
				AND
				[Project].[SupplierCompanyId] = @CompanyID 
			)
			OR
			(
				@CompanyID IS NULL
				AND 
				[Project].[SupplierCompanyId] IN
						(
							SELECT
								[Company].[ID]
							FROM
								[Company]
							WHERE
								[Company].[Own] = 1
						)
				)			
		) 
	)
	AND
	(
		@DivisionID is NULL
		OR
		[Employee].[DivisionID] = @DivisionID
		
	)
	AND 
				(
				[Employee].[State]=@EmployeeState
				OR
				@EmployeeState=0
				)
	AND
	[Employee].[ID] = [MeetingRequest].[OriginatorEmployeeID] 
	AND 
	[MeetingRequest].[State] = 2
	AND
	(
		DATEADD(mi,[MeetingRequest].[Duration],StartTime) BETWEEN @PeriodStart AND @Today
	)

RETURN 0
GO
GRANT EXEC ON ReportControlOfTimeLoss TO PUBLIC
GO
