IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportERCL')
	BEGIN
		PRINT 'Dropping Procedure ReportERCL'
		DROP  Procedure  ReportERCL
	END

GO

PRINT 'Creating Procedure ReportERCL'
GO
--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/

CREATE Procedure ReportERCL
	@EmployeeID int,
	@DateSelector int,
	@PeriodStart datetime,
	@PeriodEnd datetime
AS
	SET NOCOUNT ON
	
	SET DATEFIRST 1
	DECLARE 
		@date			datetime,
		@start			datetime,
		@finish			datetime,
		@result			INT		
		
	SELECT 
		@date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
		@PeriodStart = CASE @DateSelector
			WHEN 0 THEN DATEADD(day, 1-DAY(@date), @date)
			WHEN 1 THEN DATEADD(month, -1, DATEADD(day, 1-DAY(@date), @date))
			ELSE CAST(CONVERT(varchar(8), @PeriodStart, 112) AS datetime)
		END,
		@PeriodEnd = CASE @DateSelector
		WHEN 0 THEN @date
		ELSE DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))
		END,
		@start = CAST(CONVERT(varchar(8), DATEADD(day, 1 - DATEPART(weekday, @PeriodStart), @PeriodStart), 112) AS datetime),
		@finish = CAST(CONVERT(varchar(8), DATEADD(day, 7 - DATEPART(weekday, DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))), DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))), 112) AS datetime)

	SELECT 
		@EmployeeID = [ID]
	FROM 
		[Employee] 
	WHERE 
		([Employee].[ID] = @EmployeeID) OR
		(@EmployeeID IS NULL AND [UserID] = dbo.adfGetCurrentUserID())
			
	DECLARE @period TABLE([Date] datetime)

	WHILE @start <= @finish
	BEGIN
		INSERT INTO @period VALUES(@start)
		SET @start = DATEADD(day, 1, @start)
	END	

	DECLARE @calendar TABLE (
		[Year] int,
		[Week] int,
		[Day] int,
		[Weekday] int,
		[Date] datetime,
		[NonWorkingTime] int,
		[Duration] int,
		[Billable] int,
		[Time] int
		)
	INSERT INTO @calendar
		(
		[Year],
		[Week],
		[Day],
		[Weekday],
		[Date],
		[NonWorkingTime],
		[Duration],
		[Billable],
		[Time]
		)
	SELECT
		YEAR(tp.[Date]),
		DATEPART(week, tp.[Date]),
		CASE
			WHEN tp.[Date] BETWEEN @PeriodStart AND DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))
				THEN DAY(tp.[Date])
			ELSE NULL
		END,
		DATEPART(weekday, tp.[Date]),
		tp.[Date],
		dbo.fnPjm5NonWorkingTime(tp.[Date]),
		SUM(CASE WHEN CAST(CONVERT(varchar(8), je.[StartTime], 112) AS datetime) <= @PeriodEnd
				THEN je.[RoundedDuration]
			ELSE NULL END),	
		SUM(CASE WHEN je.[State]in(3,6) AND CAST(CONVERT(varchar(8), je.[StartTime], 112) AS datetime) <= @PeriodEnd
				THEN je.[RoundedDuration]
			ELSE 0 END),
		SUM(CASE WHEN je.[State]in(3,6) AND CAST(CONVERT(varchar(8), je.[StartTime], 112) AS datetime) <= @PeriodEnd
				THEN (je.[BillableDuration] * je.[Billable])
			ELSE 0 END)	
	FROM
		@period tp 
		LEFT OUTER JOIN [vwTimeEntry] je
			ON (je.[EmployeeID] = @EmployeeID AND tp.[Date] = CAST(CONVERT(varchar(8), je.[StartTime], 112) AS datetime) AND je.[State]in(2,3,6))		
	WHERE 
		tp.[Date] >= @PeriodStart AND tp.[Date] < DATEADD(month, 1, @PeriodStart)		
	GROUP BY
		CASE
			WHEN tp.[Date] BETWEEN @PeriodStart AND DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))
				THEN DAY(tp.[Date])
			ELSE NULL
		END,
		YEAR(tp.[Date]),
		DATEPART(week, tp.[Date]),
		DATEPART(weekday, tp.[Date]),
		tp.[Date]
	ORDER BY
		tp.[Date]

	EXECUTE ReportERCLHeader @EmployeeID, @PeriodStart, @PeriodEnd
			
	SELECT
		@EmployeeID AS [DimensionID],
		c.[Year],
		c.[Week],
		MAX(CASE [Weekday] WHEN 1 THEN c.[Day] ELSE NULL END) AS [d1],
		MAX(CASE [Weekday] WHEN 2 THEN c.[Day] ELSE NULL END) AS [d2],
		MAX(CASE [Weekday] WHEN 3 THEN c.[Day] ELSE NULL END) AS [d3],
		MAX(CASE [Weekday] WHEN 4 THEN c.[Day] ELSE NULL END) AS [d4],
		MAX(CASE [Weekday] WHEN 5 THEN c.[Day] ELSE NULL END) AS [d5],
		MAX(CASE [Weekday] WHEN 6 THEN c.[Day] ELSE NULL END) AS [d6],
		MAX(CASE [Weekday] WHEN 7 THEN c.[Day] ELSE NULL END) AS [d7],
		
		SUM(CASE [Weekday] WHEN 1 THEN c.[NonWorkingTime] ELSE NULL END) AS [t1],
		SUM(CASE [Weekday] WHEN 2 THEN c.[NonWorkingTime] ELSE NULL END) AS [t2],
		SUM(CASE [Weekday] WHEN 3 THEN c.[NonWorkingTime] ELSE NULL END) AS [t3],
		SUM(CASE [Weekday] WHEN 4 THEN c.[NonWorkingTime] ELSE NULL END) AS [t4],
		SUM(CASE [Weekday] WHEN 5 THEN c.[NonWorkingTime] ELSE NULL END) AS [t5],
		SUM(CASE [Weekday] WHEN 6 THEN c.[NonWorkingTime] ELSE NULL END) AS [t6],
		SUM(CASE [Weekday] WHEN 7 THEN c.[NonWorkingTime] ELSE NULL END) AS [t7],
		
		SUM(CASE [Weekday] WHEN 1 THEN [Duration]/60.0 ELSE NULL END) AS [1],
		SUM(CASE [Weekday] WHEN 2 THEN [Duration]/60.0 ELSE NULL END) AS [2],
		SUM(CASE [Weekday] WHEN 3 THEN [Duration]/60.0 ELSE NULL END) AS [3],
		SUM(CASE [Weekday] WHEN 4 THEN [Duration]/60.0 ELSE NULL END) AS [4],
		SUM(CASE [Weekday] WHEN 5 THEN [Duration]/60.0 ELSE NULL END) AS [5],
		SUM(CASE [Weekday] WHEN 6 THEN [Duration]/60.0 ELSE NULL END) AS [6],
		SUM(CASE [Weekday] WHEN 7 THEN [Duration]/60.0 ELSE NULL END) AS [7],

		SUM(CASE WHEN ([Weekday] = 1)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b1],
		SUM(CASE WHEN ([Weekday] = 2)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b2],
		SUM(CASE WHEN ([Weekday] = 3)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b3],
		SUM(CASE WHEN ([Weekday] = 4)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b4],
		SUM(CASE WHEN ([Weekday] = 5)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b5],
		SUM(CASE WHEN ([Weekday] = 6)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b6],
		SUM(CASE WHEN ([Weekday] = 7)AND([Duration] IS NOT NULL) THEN [Billable]/60.0 ELSE NULL END) AS [b7],
		
		SUM(CASE WHEN ([Weekday] = 1)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p1],
		SUM(CASE WHEN ([Weekday] = 2)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p2],
		SUM(CASE WHEN ([Weekday] = 3)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p3],
		SUM(CASE WHEN ([Weekday] = 4)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p4],
		SUM(CASE WHEN ([Weekday] = 5)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p5],
		SUM(CASE WHEN ([Weekday] = 6)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p6],
		SUM(CASE WHEN ([Weekday] = 7)AND([Duration] IS NOT NULL) THEN [Time]/60.0 ELSE NULL END) AS [p7]				
	FROM
		@calendar c			
	GROUP BY
		c.[Year],
		c.[Week]

RETURN 0
GO

GRANT EXEC ON ReportERCL TO PUBLIC

GO
