IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportEmployee')
	BEGIN
		PRINT 'Dropping Procedure ReportEmployee'
		DROP  Procedure  ReportEmployee
	END

GO

PRINT 'Creating Procedure ReportEmployee'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/
CREATE Procedure ReportEmployee
	@ID TIdentifier,
	@DateSelector int,
	@YearStart datetime
	
AS
	SET NOCOUNT ON
	
	DECLARE 
		@Date			datetime,
		@DayCount		int,
	    @MonthCount		int,
	    @PeriodStart	datetime,
	    @PeriodEnd		datetime
	    
	
	SELECT 
		@Date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS DATETIME),
		@DayCount = DATEPART(day,@Date),
		@MonthCount = DATEPART(month,@Date)		
	
	SET @PeriodStart = CASE	@DateSelector
		WHEN 0 THEN DATEADD(month, 1 - @MonthCount, DATEADD(day, 1 - @DayCount, @Date))
		WHEN 1 THEN DATEADD(year, -1, DATEADD(month, 1 - @MonthCount, DATEADD(day, 1 - @DayCount, @Date)))
		ELSE CAST(CONVERT(varchar(8), @YearStart, 112) AS datetime)
	END
	SET @PeriodEnd = DATEADD(year, 1, @PeriodStart)
	
	SELECT 
		@ID = [ID]
	FROM 
		[Employee] 
	WHERE 
		([Employee].[ID] = @ID) OR
		(@ID IS NULL AND [UserID] = dbo.adfGetCurrentUserID())
	
	DECLARE
		@UserID			TIdentifier,
		@UserName		TName,
		@tmp1			decimal(18,2),
		@tmp2			decimal(18,2),
		@tmp3			decimal(18,2),
		@tmp4			decimal(18,2),
		@tmp5			decimal(18,2),
		@tmp6			decimal(18,2),
		@tmp7			decimal(18,2)
		
	SET @UserID = dbo.adfGetCurrentUserID()
		
	SELECT
		@UserName = [Trustee].[DisplayName]
	FROM
		[Trustee]
	WHERE
		[Trustee].[ID] = @UserID
	
	SELECT
		[Employee].[FileAs] AS [EmployeeName],
		[Employee].[State],
		[Employee].[HireDate],
		[Employee].[FireDate],
		[Employee].[Comments],
		[Employee].[LastName] + ISNULL(' ' + [Employee].[FirstName], '') + ISNULL(' ' + [Employee].[MiddleName], '') AS [FullName],
		[Company].[FileAs] AS [CompanyName],
		[Qualification].[FileAs] AS [Qualification],
		[Position].[FileAs] AS [Position],
		dbo.fnPjm5GetContactList([Employee].[ID], [Employee].[Class]) AS [Contacts],
		dbo.fnPjm5GetCategoryList([Employee].[ID], [Employee].[Class]) AS [Categories],
		dbo.fnPjm5GetAttachmentList([Employee].[ID], [Employee].[Class]) AS [AttachmentList],
		@UserName AS [UserName],
		@PeriodStart AS [PeriodStart],
		DATEADD(day, -1, @PeriodEnd) AS [PeriodEnd]		
	FROM
		[Employee]
			INNER JOIN [Company]
				ON [Employee].[CompanyID] = [Company].[ID]
			LEFT OUTER JOIN [Position]
				ON [Employee].[PositionID] = [Position].[ID]	
			LEFT OUTER JOIN [Qualification]	
				ON [Employee].[QualificationID] = [Qualification].[ID]
	WHERE
		[Employee].[ID] = @ID
		
	SELECT
		@PeriodStart = CASE 
			WHEN [Employee].[HireDate] IS NOT NULL AND [Employee].[HireDate] > @PeriodStart 
			THEN [Employee].[HireDate]
			ELSE @PeriodStart
		END,
		@PeriodEnd = CASE 
			WHEN [Employee].[FireDate] IS NOT NULL AND [Employee].[FireDate] < @PeriodEnd 
			THEN [Employee].[FireDate]
			ELSE @PeriodEnd
		END	
	FROM 
		[Employee]
	WHERE
		[Employee].[ID] = @ID		
		
	DECLARE @crazytable TABLE (
		[TotalProject]				int,
		[NowProject]				int,
		[ProjectType1]				decimal(18,2),
		[ProjectType2]				decimal(18,2),
		[ProjectType3]				decimal(18,2),
		[ProjectType4]				decimal(18,2),
		[ProjectType5]				decimal(18,2),
		[ProjectType6]				decimal(18,2),
		[EndedTask]					int,
		[ActivityTask]				int,
		[TotalTimeEntry]			int,
		[TimeHours]					decimal(18,2),
		[ApprovedTimeHours]			decimal(18,2),
		[CalculationTimeHours]		decimal(18,2),
		[CanceledTimeHours]			decimal(18,2),
		[ApprovedTimePercent]		decimal(18,2),
		[CalculationTimePercent]	decimal(18,2),
		[AverageTime]				decimal(18,2),
		[AverageTimeADay]			decimal(18,2),
		[Project1]					varchar(256),
		[Project2]					varchar(256),
		[Project3]					varchar(256),
		[Project1Hours]				decimal(18,2),
		[Project2Hours]				decimal(18,2),
		[Project3Hours]				decimal(18,2),
		[Project1Percent]			decimal(18,2),
		[Project2Percent]			decimal(18,2),
		[Project3Percent]			decimal(18,2),
		[ActivityType1]				varchar(256),
		[ActivityType2]				varchar(256),
		[ActivityType3]				varchar(256),
		[ActivityType1Hours]		decimal(18,2),
		[ActivityType2Hours]		decimal(18,2),
		[ActivityType3Hours]		decimal(18,2),
		[ActivityType1Percent]		decimal(18,2),
		[ActivityType2Percent]		decimal(18,2),
		[ActivityType3Percent]		decimal(18,2),
		[NoTimeInWorkDay]			decimal(18,2),
		[TimeInNotWorkDay]			decimal(18,2),
		[Overtime]					decimal(18,2)
		)
	INSERT INTO @crazytable
		([NowProject])	
	SELECT	
		COUNT([Project].[ID])
	FROM 
		[vwAssignment]			
		INNER JOIN [Project]
			ON [vwAssignment].[ProjectID] = [Project].[ID]	
	WHERE
		[vwAssignment].[EmployeeID] = @ID AND 
		[Project].[State] = 2 AND
		[vwAssignment].[State] = 1
	
		
	SELECT
		@tmp1 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 1 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp2 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 2 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp3 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 3 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp4 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 4 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp5 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 5 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp6 = SUM(CASE [Project].[ProjectTypeID]
						WHEN 6 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END)				
								
	FROM 
		[vwTimeEntry]			
		INNER JOIN [Project]
			ON [vwTimeEntry].[ProjectID] = [Project].[ID]		
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND 
		[vwTimeEntry].[State] IN (2,3,6) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd
		
	SELECT	
		@tmp7 = COUNT(DISTINCT[Project].[ID])
	FROM 
		[vwTimeEntry]			
		INNER JOIN [Project]
			ON [vwTimeEntry].[ProjectID] = [Project].[ID]		
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND 
		[Project].[State] IN(2,3,4) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd	
		
	UPDATE
		@crazytable
	SET
		[ProjectType1] = @tmp1,
		[ProjectType2] = @tmp2,
		[ProjectType3] = @tmp3,
		[ProjectType4] = @tmp4,
		[ProjectType5] = @tmp5,
		[ProjectType6] = @tmp6,
		[TotalProject] = @tmp7 			
		
	SELECT
		@tmp1 = SUM(CASE WHEN [Task].[State] = 3 AND [Task].[EndDate] >= @PeriodStart AND [Task].[EndDate] < @PeriodEnd
						THEN 1
						ELSE 0 
					END),
		@tmp2 = SUM(CASE 
						WHEN [Task].[State] = 2	THEN 1
						ELSE 0 
					END)
	FROM 
		[vwAssignment]			
		INNER JOIN [TaskAssignment]
			ON [vwAssignment].[ID] = [TaskAssignment].[AssignmentID]
		INNER JOIN [Task]
			ON [TaskAssignment].[TaskID] = [Task].[ID]		
	WHERE
		[vwAssignment].[EmployeeID] = @ID
		
	UPDATE
		@crazytable
	SET
		[EndedTask] = @tmp1,
		[ActivityTask]= @tmp2
		
	SELECT
		@tmp1 = SUM(CASE 
						WHEN [vwTimeEntry].[State] IN (2,3,6) THEN 1
						ELSE 0 
					END),
		@tmp2 = SUM(CASE 
						WHEN [vwTimeEntry].[State] IN (2,3,6) THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp3 = SUM(CASE 
						WHEN [vwTimeEntry].[State] IN (3,6) THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END),
		@tmp4 = SUM(CASE [vwTimeEntry].[State]
						WHEN 4 THEN [vwTimeEntry].[RoundedDuration] / 60.0
						ELSE 0 
					END)		
	FROM 
		[vwTimeEntry]				
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd	
		
	UPDATE
		@crazytable
	SET
		[TotalTimeEntry] = @tmp1,
		[TimeHours] = @tmp2,
		[ApprovedTimeHours]	= @tmp3,
		[CanceledTimeHours]	= @tmp4,
		[ApprovedTimePercent] = CASE 
									WHEN @tmp2 IS NULL OR @tmp2 = 0 THEN 0
									ELSE @tmp3 * 100 / @tmp2
								END
		
	SELECT
		@tmp1 = ISNULL(SUM([vwTimeEntry].[BillableHour]), 0)
	FROM 
		[vwTimeEntry]			
		INNER JOIN [InvoiceLine]
			ON [vwTimeEntry].[ID] = [InvoiceLine].[DocumentID] AND
				[vwTimeEntry].[Class] = [InvoiceLine].[DocumentClass]
		INNER JOIN [Invoice]
			ON 	[InvoiceLine].[InvoiceID] = [Invoice].[ID]					
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND
		[Invoice].[State] IN (2,5)  AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd	
		
	SELECT
		@tmp1 = @tmp1 + ISNULL(SUM([vwTimeEntry].[BillableHour]), 0)
	FROM 
		[vwTimeEntry]
		INNER JOIN [CalculationLine]
			ON [vwTimeEntry].[ID] = [CalculationLine].[DocumentID] AND
				[vwTimeEntry].[Class] = [CalculationLine].[DocumentClass]
		INNER JOIN [Calculation]
			ON 	[CalculationLine].[CalculationID] = [Calculation].[ID]						
		INNER JOIN [InvoiceLine]
			ON [Calculation].[ID] = [InvoiceLine].[DocumentID] AND
				[Calculation].[Class] = [InvoiceLine].[DocumentClass]
		INNER JOIN [Invoice]
			ON 	[InvoiceLine].[InvoiceID] = [Invoice].[ID]					
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND
		[Invoice].[State] IN (2,5) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd
	
	UPDATE
		@crazytable
	SET
		[CalculationTimeHours] = @tmp1,
		[CalculationTimePercent] = CASE 
									WHEN @tmp2 IS NULL OR @tmp2 = 0 THEN 0
									ELSE @tmp1 * 100 / @tmp2
								END
	
	DECLARE
		@value			TVariant,
		@WorkTime		decimal(18,2)	
								
	SET DATEFIRST 7
	EXEC adfParameterGet 'WorkTime', 'Pjm5', @value OUTPUT  
	SET @WorkTime = CAST(@value AS decimal(18,2))
	SET @WorkTime = @WorkTime / 60.00								
								
	SELECT
		@tmp1 = SUM(CASE
					WHEN dbo.fnPjm5IsWorkingDay([vwTimeEntry].[StartTime]) = 1
					THEN [vwTimeEntry].[RoundedDuration] / 60.00
					ELSE 0
				END), --     
		@tmp2 = SUM(CASE
					WHEN dbo.fnPjm5IsWorkingDay([vwTimeEntry].[StartTime]) = 0
					THEN [vwTimeEntry].[RoundedDuration] / 60.00
					ELSE 0
				END),--     
		@tmp3 = SUM(CASE
					WHEN dbo.fnPjm5IsWorkingDay([vwTimeEntry].[StartTime]) = 1
					THEN 1
					ELSE 0
				END),--     
		@tmp4 = SUM([vwTimeEntry].[RoundedDuration]/ 60.0), --  	
		@tmp5 = CASE
					WHEN GETDATE() < @PeriodEnd
					THEN dbo.fnPjm5GetWorkingDays(@PeriodStart, GETDATE())					
					ELSE dbo.fnPjm5GetWorkingDays(@PeriodStart, DATEADD(day, -1, @PeriodEnd))
				END, --  
		@tmp6 = COUNT([vwTimeEntry].[ID]),--   					
		@tmp7 = SUM(CASE
					WHEN [vwTimeEntry].[OverTime] = 1
					THEN [vwTimeEntry].[RoundedDuration] / 60.00
					ELSE 0
				END)						
	FROM
		[Employee]
			INNER JOIN [vwTimeEntry]
				ON [Employee].[ID] = [vwTimeEntry].[EmployeeID] 
			INNER JOIN [Project]
				ON [vwTimeEntry].[ProjectID] = [Project].[ID]		
	WHERE
		[Employee].[ID] = @ID AND 
		[vwTimeEntry].[State] IN (2,3,6) AND
		[Project].[ProjectTypeID] IN (1,2,3)  AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd
	GROUP BY
		[Employee].[HireDate],
		[Employee].[FireDate]														
	
	UPDATE
		@crazytable
	SET
		[AverageTime] = CASE
							WHEN @tmp6 IS NOT NULL AND @tmp6 != 0
							THEN @tmp4 / @tmp6
							ELSE NULL
							END,--     					
		[AverageTimeADay]= CASE
								WHEN @tmp5 IS NOT NULL AND @tmp5 != 0
								THEN @tmp3 / @tmp5
								ELSE NULL
							END, --     			
		[NoTimeInWorkDay] = @tmp5 * @WorkTime - @tmp1, --     				
		[TimeInNotWorkDay] = @tmp2,			
		[Overtime] = @tmp7
		
	SELECT 
		@tmp1 = SUM([vwTimeEntry].[RoundedDuration]/ 60.0)
	FROM
		[vwTimeEntry]			
		INNER JOIN [Project]
			ON [vwTimeEntry].[ProjectID] = [Project].[ID]
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND 
		[vwTimeEntry].[State] IN (2,3,6) AND	
		[Project].[ProjectTypeID] IN (1,2) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd				
		
	DECLARE @table TABLE ([ID] int identity, [Name] varchar(256), [Hours] decimal(18,2))
	INSERT INTO @table ([Name], [Hours])
	SELECT TOP 3
		[Project].[FileAs],
		SUM([vwTimeEntry].[RoundedDuration]/ 60.0)		
	FROM
		[vwTimeEntry]			
		INNER JOIN [Project]
			ON [vwTimeEntry].[ProjectID] = [Project].[ID]	
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND 
		[vwTimeEntry].[State] IN (2,3,6) AND	
		[Project].[ProjectTypeID] IN (1,2) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd
	GROUP BY 
		[Project].[FileAs]
	ORDER BY 
		SUM([vwTimeEntry].[RoundedDuration]/ 60.0) DESC
		
	UPDATE
		@crazytable
	SET
		[Project1] = t.[Name],		
		[Project1Hours]	= t.[Hours],
		[Project1Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table AS t	
	WHERE
		t.[ID] = 1	
		
	UPDATE
		@crazytable
	SET
		[Project2] = t.[Name],		
		[Project2Hours]	= t.[Hours],
		[Project2Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table AS t	
	WHERE
		t.[ID] = 2	
		
	UPDATE
		@crazytable
	SET
		[Project3] = t.[Name],		
		[Project3Hours]	= t.[Hours],
		[Project3Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table AS t	
	WHERE
		t.[ID] = 3
		
	DECLARE @table1 TABLE ([ID] int identity, [Name] varchar(256), [Hours] decimal(18,2))
	INSERT INTO @table1 ([Name], [Hours])
	SELECT TOP 3
		[ActivityType].[FileAs],
		SUM([vwTimeEntry].[RoundedDuration]/ 60.0)		
	FROM
		[vwTimeEntry]			
		INNER JOIN [ActivityType]
			ON [vwTimeEntry].[ActivityTypeID] = [ActivityType].[ID]	
	WHERE
		[vwTimeEntry].[EmployeeID] = @ID AND 
		[vwTimeEntry].[State] IN (2,3,6) AND
		[vwTimeEntry].[StartTime] >= @PeriodStart AND [vwTimeEntry].[StartTime] < @PeriodEnd
	GROUP BY 
		[ActivityType].[FileAs]
	ORDER BY 
		SUM([vwTimeEntry].[RoundedDuration]/ 60.0) DESC
		
	UPDATE
		@crazytable
	SET
		[ActivityType1] = t.[Name],		
		[ActivityType1Hours]	= t.[Hours],
		[ActivityType1Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table1 AS t	
	WHERE
		t.[ID] = 1	
		
	UPDATE
		@crazytable
	SET
		[ActivityType2] = t.[Name],		
		[ActivityType2Hours]	= t.[Hours],
		[ActivityType2Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table1 AS t	
	WHERE
		t.[ID] = 2	
		
	UPDATE
		@crazytable
	SET
		[ActivityType3] = t.[Name],		
		[ActivityType3Hours]	= t.[Hours],
		[ActivityType3Percent] = CASE
								WHEN @tmp1 IS NOT NULL AND @tmp1 != 0
								THEN t.[Hours] * 100 / @tmp1
								ELSE NULL
							END				
		
	FROM
		@table1 AS t	
	WHERE
		t.[ID] = 3	
		
	SELECT * FROM @crazytable					
		
RETURN 0
GO

GRANT EXEC ON ReportEmployee TO PUBLIC

GO
