IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportLoadOfEmployees')
	BEGIN
		PRINT 'Dropping Procedure ReportLoadOfEmployees'
		DROP  Procedure  ReportLoadOfEmployees
	END

GO

PRINT 'Creating Procedure ReportLoadOfEmployees'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/
CREATE Procedure ReportLoadOfEmployees
	@DateSelector	int,
	@WeekStart		Tdatetime,
	@CompanyID		int,
	@ManagerID		int,
	@DecimalSeparator varchar(1),
	@DivisionSelector int,
	@DivisionID		TIdentifier
AS
	SET NOCOUNT ON
	IF @CompanyID IS NULL AND @DivisionSelector<>2
	BEGIN
		RAISERROR('#CompanyIsNotSelected', 16, -1)
		RETURN 1
	END
	IF @DivisionID IS NULL AND @DivisionSelector=2 
	BEGIN
		RAISERROR('#DivisionIsNotSelected', 16, -1)
		RETURN 2
	END
	
	
	SET DATEFIRST 1
	DECLARE 
		@Date		TDate
		
	SET @Date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime)
	SET @WeekStart = CASE @DateSelector
		WHEN 0 THEN DATEADD(day, 1 - DATEPART(weekday, @Date), @Date)
		WHEN 1 THEN DATEADD(week, 1, DATEADD(day, 1 - DATEPART(weekday, @Date), @Date))
		ELSE CAST(CONVERT(varchar(8), @WeekStart, 112) AS datetime)
	END
	DECLARE @employee TABLE([ID] int, [FileAs] varchar(256))

	IF @ManagerID IS NULL
		SET @ManagerID = dbo.adfGetCurrentEmployeeID()

	INSERT INTO @employee
	SELECT DISTINCT [Employee].[ID], [Employee].[FileAs]
	FROM 
		[Employee]
		LEFT OUTER JOIN [Division]
			ON [Employee].[DivisionID] = [Division].[ID]
		INNER JOIN [vwAssignment]
			ON [vwAssignment].[EmployeeID] = [Employee].[ID]
		INNER JOIN [Project]
			ON [Project].[ID] = [vwAssignment].[ProjectID]
		
	WHERE 
		[Employee].[State] = 1 AND
		(
			@DivisionSelector = 0 OR 
			(@DivisionSelector = 1 AND [Employee].[DivisionID] IS NULL) OR
			(@DivisionSelector = 2 AND ([Employee].[DivisionID] = @DivisionID OR (@DivisionID IS NULL AND [Employee].[DivisionID] IS NOT NULL)))
		)AND 
		[Project].[SupplierCompanyID] = @CompanyID AND
		[Project].[ProjectManagerEmployeeID] = @ManagerID
		AND
		[Project].[State] IN (1, 2, 3)

	EXECUTE ReportLoadOfProjectsHeader @WeekStart, @CompanyID, @ManagerID, @DivisionID, @DivisionSelector
	SELECT
		[Employee].[FileAs],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], @WeekStart, @DecimalSeparator) AS [Day1],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 1, @WeekStart), @DecimalSeparator) AS [Day2],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 2, @WeekStart), @DecimalSeparator) AS [Day3],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 3, @WeekStart), @DecimalSeparator) AS [Day4],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 4, @WeekStart), @DecimalSeparator) AS [Day5],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 5, @WeekStart), @DecimalSeparator) AS [Day6],
		dbo.fnPjm5GetProjectListByEmployeeForDate([Employee].[ID], DATEADD(day, 6, @WeekStart), @DecimalSeparator) AS [Day7],
		ISNULL((SUM([EmployeeCalendar].[Planned]) / 60.00), 0) AS [Total]
	FROM
		@employee AS [Employee]
		LEFT OUTER JOIN [EmployeeCalendar]
			ON [Employee].[ID]  = [EmployeeCalendar].[EmployeeID] AND
				[EmployeeCalendar].[Date] >= @WeekStart AND [EmployeeCalendar].[Date] < DATEADD(week, 1, @WeekStart)		
	GROUP BY
		[Employee].[ID],
		[Employee].[FileAs]
	UNION ALL
	SELECT
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	0		
		
	RETURN 0
GO

GRANT EXEC ON ReportLoadOfEmployees TO PUBLIC

GO
