IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportTimesheet')
	BEGIN
		PRINT 'Dropping Procedure ReportTimesheet'
		DROP  Procedure  ReportTimesheet
	END

GO

PRINT 'Creating Procedure ReportTimesheet'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/
CREATE Procedure ReportTimesheet
	@EmployeeID		int,
	@DateSelector   int,
	@WeekStart	    Tdatetime
AS
	SET NOCOUNT ON
	SET DATEFIRST 1
	DECLARE 
		@Date		TDate,
		@NextDate	TDate
		
		SET @Date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime)
	
		SET @WeekStart = CASE @DateSelector
			WHEN 0 THEN DATEADD(day, 1 - DATEPART(weekday, @Date), @Date)
			WHEN 1 THEN DATEADD(week, -1, DATEADD(day, 1 - DATEPART(weekday, @Date), @Date))
			ELSE CAST(CONVERT(varchar(8), @WeekStart, 112) AS datetime)
		END
		
		SET @NextDate = CASE @DateSelector
			WHEN 0 THEN DATEADD(day, 1, @Date)
			ELSE DATEADD(week, 1, @WeekStart)
		END	

	SELECT 
		@EmployeeID = [ID]
	FROM 
		[Employee] 
	WHERE 
		([Employee].[ID] = @EmployeeID) OR
		(@EmployeeID IS NULL AND [UserID] = dbo.adfGetCurrentUserID())
	
	EXECUTE ReportTimesheetHeader @EmployeeID, @WeekStart, @NextDate
	
	DECLARE
		@t table(
			[ProjectID]				int,
			[TaskID]			int,
			[Day]				int,
			[Time]				decimal(18,2)			
		)

	INSERT INTO 
		@t(
			[ProjectID],
			[TaskID],
			[Day],
			[Time]
		)
		
	SELECT
	[Project].[ID],
	[Task].[ID],
	DATEPART(WEEKDAY,[TimeEntry].[StartTime]),
	SUM([TimeEntry].[RoundedDuration] / 60.0)
	
	FROM 
		[Project]
		INNER JOIN [vwTimeEntry] as [TimeEntry]
			ON [Project].[ID] = [TimeEntry].[ProjectID]
		LEFT OUTER JOIN [Task]
			ON [TimeEntry].[TaskID] = [Task].[ID]
	WHERE
		[TimeEntry].[EmployeeID] = @EmployeeID AND
		([TimeEntry].[StartTime] >= @WeekStart AND [TimeEntry].[StartTime] < @NextDate) AND
		[TimeEntry].[State] in (2,3,6)
 	GROUP BY
		[Project].[ID],
		[Task].[ID],
		DATEPART(WEEKDAY,[TimeEntry].[StartTime])


	DECLARE 
		@t2 table(
		[ProjectID]		int,
		[TaskID]	int,
		[Day]		int,
		[Approved]	decimal(18,2),	
		[Calculation]	decimal(18,2)
		)
	INSERT INTO @t2(
		[ProjectID],
		[TaskID],
		[Day],
		[Approved],	
		[Calculation]
		)

	SELECT
		[Project].[ID],
		[Task].[ID],
		DATEPART(WEEKDAY,[TimeEntry].[StartTime]),
		SUM([TimeEntry].[RoundedDuration] / 60.0),
		SUM([TimeEntry].[BillableDuration] * [TimeEntry].[Billable] / 60.0)
	FROM 
		[Project]
		INNER JOIN [vwTimeEntry] as [TimeEntry]
			ON [Project].[ID] = [TimeEntry].[ProjectID]
		LEFT OUTER JOIN [Task]
			ON [TimeEntry].[TaskID] = [Task].[ID]
	WHERE
		[TimeEntry].[EmployeeID] = @EmployeeID AND
		([TimeEntry].[StartTime] >= @WeekStart AND [TimeEntry].[StartTime] < @NextDate) AND
		[TimeEntry].[State] in (3,6)
 	GROUP BY
		[Project].[ID],
		[Task].[ID],
		DATEPART(WEEKDAY,[TimeEntry].[StartTime])
	
	SELECT
	[Project].[Number] AS [ProjectName],
	[Task].[FileAs] AS [TaskName],
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 1 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total1],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 1 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved1],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 1 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation1],
	
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 2 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total2],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 2 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved2],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 2 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation2],
			
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 3 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total3],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 3 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved3],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 3 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation3],
				
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 4 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total4],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 4 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved4],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 4 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation4],
			
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 5 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total5],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 5 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved5],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 5 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation5],
			
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 6 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total6],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 6 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved6],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 6 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation6],
			
	(SELECT [Time] 
		FROM @t WHERE [ProjectID] = [Project].[ID] AND [Day] = 7 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)) AS [Total7],
	ISNULL((SELECT [Approved] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 7 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Approved7],
	ISNULL((SELECT [Calculation] 
		FROM @t2 WHERE [ProjectID] = [Project].[ID] AND [Day] = 7 AND ISNULL([TaskID], -1) = ISNULL([Task].[ID], -1)), 0) AS [Calculation7],
		
	(SELECT Sum([Time]) 
		FROM @t WHERE [ProjectID] = [Project].[ID]) AS [ProjectSum]	
	FROM
		[Project]
		INNER JOIN @t AS t
			ON [Project].[ID] = t.[ProjectID]
		LEFT OUTER JOIN @t2 AS t2
			ON t.[ProjectID] = t2.[ProjectID]
		LEFT OUTER JOIN [Task]
			ON t.[TaskID] = [Task].[ID]	
	GROUP BY
		[Project].[ID],
		[Project].[Number],
		[Task].[ID],
		[Task].[FileAs]

RETURN 0
GO

GRANT EXEC ON ReportTimesheet TO PUBLIC

GO
