 IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trServiceRequestUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trServiceRequestUpdate'
		DROP  Trigger pjm5trServiceRequestUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trServiceRequestUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    11.02.10		SYV					      
**										  UserParameters    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trServiceRequestUpdate'
GO
CREATE Trigger pjm5trServiceRequestUpdate 
ON [ServiceRequest] FOR UPDATE
AS
BEGIN
	IF EXISTS(
		SELECT*
		FROM
			[inserted]
			INNER JOIN [deleted]
				ON [inserted].[ID] = [deleted].[ID]
		WHERE
			[inserted].[State] IN (2, 3, 4, 5, 7) AND
			[inserted].[EmployeeID] IS NULL AND
			[deleted].[EmployeeID] IS NOT NULL
		)
	BEGIN
		RAISERROR('#CannotApproveUnasignedServiceRequest', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	DECLARE @CurrentEmployeeID TIdentifier
    SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
   
    IF EXISTS (
		SELECT *
		FROM [inserted]
			INNER JOIN [deleted]
				ON [inserted].[ID] = [deleted].[ID]
		WHERE
			[deleted].[State] <> [inserted].[State] AND
			[inserted].[State] IN (2, 4) AND
			dbo.fnPjm5AllowCreateProjectEntry([inserted].[ProjectID], NULL) != 1 AND
			dbo.fnPjm5HasFinansDataAccess([inserted].[ProjectID], @CurrentEmployeeID, NULL, NULL) != 1
	)
	BEGIN
		IF EXISTS(SELECT * FROM [inserted] WHERE [EmployeeID] != @CurrentEmployeeID)
		BEGIN
			RAISERROR('#accessdenied', 18, -1)
			ROLLBACK TRANSACTION
			RETURN
		END
	END

	IF EXISTS (SELECT * FROM [inserted] WHERE ISNULL([inserted].[Number], '') = '' AND [inserted].[State] NOT IN (1, 6))
	BEGIN
		RAISERROR('#InvalidDocumentNumber', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@ServiceRequestClass TClass,
		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass
	SET @ServiceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	SET @CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET @StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET @InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	
	IF EXISTS(
		SELECT * 
		FROM 
			inserted 
			INNER JOIN deleted
				ON deleted.[ID] = inserted.[ID]
			INNER JOIN [Attachment]
				ON ([Attachment].[DocumentClass] = @ServiceRequestClass AND
					[Attachment].[DocumentID] = inserted.[ID] AND 
					[Attachment].[AttachmentTypeID] = 3 AND
					[Attachment].[LinkDocumentClass] IN (@CalculationClass, @StatementClass, @InvoiceClass))
		WHERE
			deleted.[State] IN (4, 5) AND
			inserted.[State] NOT IN (4, 5)
		)
	BEGIN
		RAISERROR('#cannotchangeentrystate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	DECLARE @ToState TState,
			@FromState TState,
			@value TVariant,
			@Approve bit,
			@UserID int
	
	SET @value = NULL
	SET @UserID = dbo.adfGetCurrentUserID()		
	
	SELECT
		@FromState = [State]
	FROM
		[deleted]
	
	SELECT
		@ToState = [State]
	FROM
		[inserted]
	
	SELECT @value = [Value] FROM [UserParameter] WHERE [Name] = 'TimeEntryApproveForDocumentInFinalState' AND [UserID] = @UserID
	SET @Approve = ISNULL(CAST(@value as bit), 0)
	
	IF @ToState = 4 AND @FromState IN (5, 3) AND @Approve = 1
	BEGIN
		DECLARE @DocumentClass TClass,
				@DocumentID TIdentifier
		SELECT
				@DocumentClass = [Class],
				@DocumentID = [ID]
		FROM
			[inserted]		
		EXEC pjm6TimeEntryAutoApproveForDocumentInFinalState @DocumentClass, @DocumentID
	END
	
END

GO
 