IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trStatementUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trStatementUpdate'
		DROP  Trigger pjm5trStatementUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trStatementUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trStatementUpdate'
GO
CREATE Trigger pjm5trStatementUpdate 
ON [Statement] FOR UPDATE
AS
BEGIN
	IF EXISTS (SELECT * FROM [inserted] WHERE ([CurrencyRate] IS NULL OR [CurrencyRate] <= 0) AND [State] IN (2, 3))
	BEGIN
		RAISERROR('#CurrencyRateCannotBeNull', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	IF EXISTS (SELECT * FROM [inserted] WHERE ([BaseCurrencyRate] IS NULL OR [BaseCurrencyRate] <= 0) AND [State] IN (2, 3))
	BEGIN
		RAISERROR('#BaseCurrencyRateCannotBeNull', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	IF EXISTS (SELECT * FROM [inserted] WHERE ISNULL([inserted].[Number], '') = '' AND [inserted].[State] != 1)
	BEGIN
		RAISERROR('#InvalidDocumentNumber', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@calculationClass TClass
	SET @calculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [StatementLine]
				ON inserted.[ID] = [StatementLine].[StatementID]
			INNER JOIN [Calculation]
				ON [StatementLine].[DocumentClass]  = @calculationClass AND 
					[Calculation].[ID] = [StatementLine].[DocumentID] 		
		WHERE
			[Calculation].[State] NOT IN (2, 3)
		)
	BEGIN
		RAISERROR('#CannotApproveInvoiceNotApprovedCalculation', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE
		@serviceRequestClass TClass,
		@serviceAssignmentClass TClass
	SET @serviceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	SET @serviceAssignmentClass = dbo.adfGetClassByName('Pjm5.ServiceAssignment')
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [StatementLine]
				ON [StatementLine].[StatementID] = inserted.[ID]
			INNER JOIN [Document]
				ON [Document].[Class] = [StatementLine].[DocumentClass] AND 
					[Document].[ID] = [StatementLine].[DocumentID]
			WHERE
				[StatementLine].[DocumentClass] != @calculationClass AND
				(
				[StatementLine].[DocumentClass] = @serviceRequestClass AND [Document].[State] NOT IN (4, 5) OR
				[StatementLine].[DocumentClass] NOT IN (@serviceRequestClass, @serviceAssignmentClass) AND [Document].[State] NOT IN (3, 6)
				)
			)
	BEGIN
		RAISERROR('#CannotApproveInvoiceNotApprovedRecords', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	IF EXISTS(
		SELECT * 
		FROM
			deleted
			INNER JOIN inserted
				ON deleted.[ID] = inserted.[ID] AND 
					deleted.[State] = 1 AND 
					inserted.[State] = 2
			INNER JOIN [StatementLine]
				ON [StatementLine].[StatementID] = inserted.[ID]
			LEFT OUTER JOIN [TimeEntry]
				ON [TimeEntry].[Class] = [StatementLine].[DocumentClass] AND 
					[TimeEntry].[ID] = [StatementLine].[DocumentID]
			LEFT OUTER JOIN [Expense]
				ON [Expense].[Class] = [StatementLine].[DocumentClass] AND 
					[Expense].[ID] = [StatementLine].[DocumentID]
			LEFT OUTER JOIN [ServiceRequest]
				ON [ServiceRequest].[Class] = [StatementLine].[DocumentClass] AND 
					[ServiceRequest].[ID] = [StatementLine].[DocumentID]
		WHERE
			(ISNULL([TimeEntry].[ID], 0) + 
				ISNULL([Expense].[ID], 0) +
				ISNULL([ServiceRequest].[ID], 0)) != 0
			AND		
			(ISNULL(CAST([TimeEntry].[Billable] AS int), 0) + 
				ISNULL(CAST([Expense].[Billable] AS int), 0) +
				ISNULL(CAST([ServiceRequest].[Billable] AS int), 0)) = 0
		)
	BEGIN
		RAISERROR('#CannotApproveNotBillableRecords', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	-- REJect
	UPDATE
		[Calculation]
	SET
		[Calculation].[State] = 2,
		[Calculation].[StatementApproved] = 0,
		[Calculation].[Modified] = DEFAULT,
		[Calculation].[ModifiedBy] = DEFAULT
	FROM
		[Calculation]
		INNER JOIN [StatementLine]
			ON [Calculation].[Class] = [StatementLine].[DocumentClass] AND 
				[Calculation].[ID] = [StatementLine].[DocumentID]
		INNER JOIN inserted
			ON [StatementLine].[StatementID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 4 AND inserted.[State] = 4
	WHERE
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [Calculation].[Class] AND 
				[DocumentID] = [Calculation].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [InvoiceLine] 
				INNER JOIN [Invoice]
					ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
			WHERE 
				[Invoice].[State] != 3 AND
				[DocumentClass] = [Calculation].[Class] AND 
				[DocumentID] = [Calculation].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitStatementCalculation', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE
		[TimeEntry]
	SET
		[TimeEntry].[State] = 3,
		[TimeEntry].[Modified] = DEFAULT,
		[TimeEntry].[ModifiedBy] = DEFAULT
	FROM
		[TimeEntry]
		INNER JOIN [StatementLine]
			ON [TimeEntry].[Class] = [StatementLine].[DocumentClass] AND 
				[TimeEntry].[ID] = [StatementLine].[DocumentID]
		INNER JOIN inserted
			ON [StatementLine].[StatementID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 4 AND inserted.[State] = 4
	WHERE
		[TimeEntry].[State] = 6 AND
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [TimeEntry].[Class] AND 
				[DocumentID] = [TimeEntry].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [InvoiceLine] 
				INNER JOIN [Invoice]
					ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
			WHERE 
				[Invoice].[State] IN (2, 4, 5) AND
				[DocumentClass] = [TimeEntry].[Class] AND 
				[DocumentID] = [TimeEntry].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceTimeEntry', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	UPDATE
		[Expense]
	SET
		[Expense].[State] = 3,
		[Expense].[Modified] = DEFAULT,
		[Expense].[ModifiedBy] = DEFAULT
	FROM
		[Expense]
		INNER JOIN [StatementLine]
			ON [Expense].[Class] = [StatementLine].[DocumentClass] AND 
				[Expense].[ID] = [StatementLine].[DocumentID]
		INNER JOIN inserted
			ON [StatementLine].[StatementID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 4 AND inserted.[State] = 4
	WHERE
		[Expense].[State] = 6 AND
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [Expense].[Class] AND 
				[DocumentID] = [Expense].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [InvoiceLine] 
				INNER JOIN [Invoice]
					ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
			WHERE 
				[Invoice].[State] IN (2, 4, 5) AND
				[DocumentClass] = [Expense].[Class] AND 
				[DocumentID] = [Expense].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceExpense', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE
		[ServiceRequest]
	SET
		[ServiceRequest].[State] = 4,
		[ServiceRequest].[Modified] = DEFAULT,
		[ServiceRequest].[ModifiedBy] = DEFAULT
	FROM
		[ServiceRequest]
		INNER JOIN [StatementLine]
			ON [ServiceRequest].[Class] = [StatementLine].[DocumentClass] AND 
				[ServiceRequest].[ID] = [StatementLine].[DocumentID]
		INNER JOIN inserted
			ON [StatementLine].[StatementID] = inserted.[ID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 4 AND inserted.[State] = 4
	WHERE
		[ServiceRequest].[State] = 5 AND
		NOT EXISTS(
			SELECT * 
			FROM [FactureLine] 
				INNER JOIN [Facture]
					ON [Facture].[ID] = [FactureLine].[FactureID]
			WHERE 
				[Facture].[State] IN (1, 2) AND
				[DocumentClass] = [ServiceRequest].[Class] AND 
				[DocumentID] = [ServiceRequest].[ID]
			) AND
		NOT EXISTS(
			SELECT * 
			FROM [InvoiceLine] 
				INNER JOIN [Invoice]
					ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
			WHERE 
				[Invoice].[State] IN (2, 4, 5) AND
				[DocumentClass] = [ServiceRequest].[Class] AND 
				[DocumentID] = [ServiceRequest].[ID]
			)
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRecommitInvoiceServiceRequest', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DECLARE @timeClass TClass
	SET @timeClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	UPDATE
		[EntryAmount]
	SET
		[Amount] = ROUND([TimeEntry].[RoundedDuration] / 60.0, 2),
		[Price] = [TimeEntry].[Rate]
	FROM 
		[EntryAmount]
		INNER JOIN [TimeEntry]
			ON [TimeEntry].[ID] = [EntryAmount].[ID]
		INNER JOIN [StatementLine]
			ON [StatementLine].[DocumentClass] = @timeClass AND
				[StatementLine].[DocumentID] = [EntryAmount].[ID]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [StatementLine].[StatementID]
		INNER JOIN deleted
			ON deleted.[ID] = inserted.[ID] AND 
				deleted.[State] != 3 AND inserted.[State] = 3
		LEFT OUTER JOIN [InvoiceLine]
			ON [InvoiceLine].[DocumentClass] = @timeClass AND
				[InvoiceLine].[DocumentID] = [EntryAmount].[ID]
	WHERE
		[InvoiceLine].[ID] IS NULL
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END			

	--- REJECT 
	DELETE [Attachment]
	FROM
		[Attachment]
		INNER JOIN [inserted]
			ON ([inserted].[Class] = [Attachment].[LinkDocumentClass] AND 
				[inserted].[ID] = [Attachment].[LinkDocumentID])
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 4 AND
		[deleted].[State] != 4 AND
		[Attachment].[AttachmentTypeID] = 3
	IF @@error != 0
	BEGIN
		RAISERROR('#CannotRejectStatement', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DELETE [StatementAdvanceInvoice]
	FROM
		[StatementAdvanceInvoice]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [StatementAdvanceInvoice].[StatementID]
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
	WHERE
		[inserted].[State] = 4 AND
		[deleted].[State] != 4 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	
	--     
	UPDATE
		[UserReminder]
	SET
		[State] = 0
	FROM
		[Reminder]
		INNER JOIN [inserted]
			ON ([inserted].[ID] = [Reminder].[DocumentID] AND
			[inserted].[Class] = [Reminder].[DocumentClass])
		INNER JOIN [deleted]
			ON [deleted].[ID] = [inserted].[ID]
		INNER JOIN [UserReminder]
			ON [UserReminder].[ReminderID] = [Reminder].[ID]
	WHERE
		[Reminder].[Subtype] = 'MD' AND
		[inserted].[State] = 3 AND
		[deleted].[State] != 3 
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

END

GO
