IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trTimeEntryForTfaUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm6trTimeEntryForTfaUpdate'
		DROP  Trigger pjm6trTimeEntryForTfaUpdate
	END
GO

/******************************************************************************
**		File:	pjm6trTimeEntryForTfaUpdate.sql
**		Name:	pjm6trTimeEntryForTfaUpdate
**		Desc:	    
**				     
**
**		This template can be customized:
**              
**
**		Auth: pavel.svintsov
**		Date: 02.06.2009
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		02.06.09	SPV					 
**		22.10.10	SYV					  "Object Deleted?"
*******************************************************************************/

PRINT 'Creating Trigger pjm6trTimeEntryForTfaUpdate'
GO
CREATE TRIGGER pjm6trTimeEntryForTfaUpdate
ON [TimeEntry] FOR UPDATE
AS
BEGIN
    IF @@rowcount = 0
       RETURN
	SET NOCOUNT ON

	DECLARE
		@ServiceRequestClass TClass,
		@TaskRequestClass TClass,
		@IssueClass TClass,
		@ServiceAssignmentClass TClass,
		@Invalid int,
		@flag int

	SET @ServiceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	SET @TaskRequestClass = dbo.adfGetClassByName('Pjm5.TaskRequest')
	SET @IssueClass = dbo.adfGetClassByName('Pjm5.Issue')
	SET @ServiceAssignmentClass = dbo.adfGetClassByName('Pjm5.ServiceAssignment')
	SET @invalid = 0

	DECLARE @forAdd TABLE
	(
		[ID] TIdentifier,
		[DocumentClass] TClass,
		[DocumentID] TIdentifier,
		[Duration] TCount,
		[Cost] TMoney
	)

	INSERT INTO @forAdd
	([ID], [DocumentClass], [DocumentId], [Duration], [Cost])
	SELECT
		[inserted].[ID],
		[inserted].[DocumentClass],
		[inserted].[DocumentID],
		[inserted].[Duration],
		(CASE WHEN [inserted].[Billable] = 1 THEN [inserted].[Duration] * [inserted].[Rate] / 60 ELSE 0 END)
	FROM
		[inserted]
	WHERE
	(	
		[inserted].[State] in (2, 3, 6)
	) AND
	(
		[inserted].[DocumentClass] in (@TaskRequestClass, @ServiceRequestClass, @IssueClass, @ServiceAssignmentClass)
	)
	
	DECLARE @forSubtract TABLE
	(
		[ID] TIdentifier,
		[DocumentClass] TClass,
		[DocumentID] TIdentifier,
		[Duration] TCount,
		[Cost] TMoney
	)

	INSERT INTO @forSubtract
	([ID], [DocumentClass], [DocumentId], [Duration], [Cost])
	SELECT
		[deleted].[ID],
		[deleted].[DocumentClass],
		[deleted].[DocumentID],
		[deleted].[Duration],
		(CASE WHEN [deleted].[Billable] = 1 THEN [deleted].[Duration] * [deleted].[Rate] / 60 ELSE 0 END)
	FROM
		[deleted]
	WHERE
	(	
		[deleted].[State] in (2, 3, 6)
	) AND
	(
		[deleted].[DocumentClass] in (@TaskRequestClass, @ServiceRequestClass, @IssueClass, @ServiceAssignmentClass)
	)

	DECLARE @forCheck TABLE
	(
		[DocumentID] TIdentifier null,
		[DocumentClass] TClass null
	)

	INSERT INTO @forCheck
	([DocumentClass], [DocumentID])
	SELECT [deleted].[DocumentClass], [deleted].[DocumentID]
	FROM
		[deleted]
		INNER JOIN [inserted]
			ON [inserted].[id] = [deleted].[id]
		WHERE
		(
			[inserted].[Duration] <> [deleted].[Duration]
			 OR
			(
				([inserted].[Rate] <> [deleted].[Rate]) AND ([deleted].[Billable] = 1)
			) OR
			(
				[inserted].[Billable] <> [deleted].[Billable]
			) OR
			(
				[inserted].[State] in (2, 3, 6) AND [deleted].[State] not in (2, 3, 6)
			) OR
			(
				[inserted].[State] not in (2, 3, 6) AND [deleted].[State] in (2, 3, 6)
			) OR
			(
				[inserted].[ProjectID] <> [deleted].[ProjectID]
			) OR
			(
				([inserted].[DocumentClass] = [deleted].[DocumentClass]) AND ISNULL([inserted].[DocumentID], -1) <> ISNULL([deleted].[DocumentID], -1)
			) OR
			(
				ISNULL([inserted].[DocumentClass], -1) <> ISNULL([deleted].[DocumentClass], -1)
			) 
		) AND
		ISNULL([deleted].[DocumentClass], -1) in (@TaskRequestClass, @ServiceRequestClass, @IssueClass, @ServiceAssignmentClass, -1)
	

	INSERT INTO @forCheck
	([DocumentClass], [DocumentID])
	SELECT [inserted].[DocumentClass], [inserted].[DocumentID]
	FROM
		[inserted]
		INNER JOIN [deleted]
			ON [inserted].[id] = [deleted].[id] 
		WHERE
		(
			[inserted].[Duration] <> [deleted].[Duration]
			 OR
			(
				[inserted].[Rate] <> [deleted].[Rate] AND [deleted].[Billable] = 1
			)
			OR
			(
			[inserted].[Billable] <> [deleted].[Billable]
			)
			OR
			(
				[inserted].[State] in (2, 3, 6) AND [deleted].[State] not in (2, 3, 6)
			)
			OR
			(
				[inserted].[State] not in (2, 3, 6) AND [deleted].[State] in (2, 3, 6)
			)
			OR
			(
				[inserted].[ProjectID] <> [deleted].[ProjectID]
			)
			OR
			(
				[inserted].[DocumentClass] = [deleted].[DocumentClass] AND ISNULL([inserted].[DocumentID], -1) <> ISNULL([deleted].[DocumentID], -1)
			)
			OR
			(
				[inserted].[DocumentClass] <> [deleted].[DocumentClass]
			)
		) AND
		ISNULL([inserted].[DocumentClass], -1) in (@TaskRequestClass, @ServiceRequestClass, @IssueClass, @ServiceAssignmentClass, -1)

	SET @flag = 0
	select @flag = count(*) from @forCheck
	
	IF @flag > 0
	BEGIN
	
		------------------------- Task Request ----------------------
				SELECT @invalid = count(*)
		FROM
			[TaskRequest]
			INNER JOIN [inserted]
				ON [TaskRequest].[ID] = [inserted].[DocumentID] AND
					[TaskRequest].[Class] = [inserted].[DocumentClass] AND
					[TaskRequest].[State] NOT IN (1, 2)
			LEFT OUTER JOIN [deleted]
				ON [deleted].[DocumentClass] = [TaskRequest].[Class] AND [deleted].[DocumentID] = [TaskRequest].[ID]
		WHERE
			[deleted].[ID] IS NULL

		IF @invalid > 0
		BEGIN
			RAISERROR('#tfaUpdateInvalidJobState', 18, -1)
			ROLLBACK TRANSACTION
		END
		ELSE
		BEGIN
			UPDATE [TaskRequest]
			SET
				[TaskRequest].[ActualManHour] = [TaskRequest].[ActualManHour] + [Add].[Duration],
				[TaskRequest].[ActualBudget] = [TaskRequest].[ActualBudget] + [Add].[Cost]
			FROM
				[TaskRequest]
				INNER JOIN @ForAdd AS [Add]
					ON [TaskRequest].[ID] = [Add].[DocumentID] AND
						[TaskRequest].[Class] = [Add].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END

			UPDATE [TaskRequest]
			SET
				[TaskRequest].[ActualManHour] = [TaskRequest].[ActualManHour] - [Subtract].[Duration],
				[TaskRequest].[ActualBudget] = [TaskRequest].[ActualBudget] - [Subtract].[Cost]
			FROM
				[TaskRequest]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [TaskRequest].[ID] = [Subtract].[DocumentID] AND
						 [TaskRequest].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
		END

		---------------------- Service Request ------------------------------
		SELECT @invalid = count(*)
		FROM
			[ServiceRequest]
			INNER JOIN [inserted]
				ON [ServiceRequest].[ID] = [inserted].[DocumentID] AND
					[ServiceRequest].[Class] = [inserted].[DocumentClass] AND
					[ServiceRequest].[State] NOT IN (1, 2, 3)
			LEFT OUTER JOIN [deleted]
				ON [deleted].[DocumentClass] = [ServiceRequest].[Class] AND [deleted].[DocumentID] = [ServiceRequest].[ID]
		WHERE
			[deleted].[ID] IS NULL

		IF @invalid > 0
		BEGIN
			RAISERROR('#tfaUpdateInvalidJobState', 18, -1)
			ROLLBACK TRANSACTION
		END
		ELSE
		BEGIN
			UPDATE [ServiceRequest]
			SET
				[ServiceRequest].[ActualManHour] = [ServiceRequest].[ActualManHour] + [Add].[Duration],
				[ServiceRequest].[ActualBudget] = [ServiceRequest].[ActualBudget] + [Add].[Cost]
			FROM
				[ServiceRequest]
				INNER JOIN @ForAdd AS [Add]
						ON [ServiceRequest].[ID] = [Add].[DocumentID] AND
							[ServiceRequest].[Class] = [Add].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END

			UPDATE [ServiceRequest]
			SET
				[ServiceRequest].[ActualManHour] = [ServiceRequest].[ActualManHour] - [Subtract].[Duration],
				[ServiceRequest].[ActualBudget] = [ServiceRequest].[ActualBudget] - [Subtract].[Cost]
			FROM
				[ServiceRequest]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [ServiceRequest].[ID] = [Subtract].[DocumentID] AND
						 [ServiceRequest].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
		END
		------------------------- Issue ----------------------
		SELECT @invalid = count(*)
		FROM
			[Issue]
			INNER JOIN [inserted]
				ON [Issue].[ID] = [inserted].[DocumentID] AND
					[Issue].[Class] = [inserted].[DocumentClass] AND
					[Issue].[State] NOT IN (1, 2, 3, 4, 6)
			LEFT OUTER JOIN [deleted]
				ON [deleted].[DocumentClass] = [Issue].[Class] AND [deleted].[DocumentID] = [Issue].[ID]
		WHERE
			[deleted].[ID] IS NULL

		IF @invalid > 0
		BEGIN
			RAISERROR('#tfaUpdateInvalidJobState', 18, -1)
			ROLLBACK TRANSACTION
		END
		ELSE
		BEGIN
			UPDATE [Issue]
			SET
				[Issue].[ActualManHour] = ISNULL([Issue].[ActualManHour], 0) + [Add].[Duration],
				[Issue].[ActualBudget] = ISNULL([Issue].[ActualBudget], 0) + [Add].[Cost]
			FROM
				[Issue]
				INNER JOIN @ForAdd AS [Add]
					ON [Issue].[ID] = [Add].[DocumentID] AND
						[Issue].[Class] = [Add].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END

			UPDATE [Issue]
			SET
				[Issue].[ActualManHour] = [Issue].[ActualManHour] - [Subtract].[Duration],
				[Issue].[ActualBudget] = [Issue].[ActualBudget] - [Subtract].[Cost]
			FROM
				[Issue]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [Issue].[ID] = [Subtract].[DocumentID] AND
						[Issue].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
		END
		---------------------- Service Assignment ------------------------------
		SELECT @invalid = count(*)
		FROM
			[ServiceAssignment]
			INNER JOIN [inserted]
				ON [ServiceAssignment].[ID] = [inserted].[DocumentID] AND
					[ServiceAssignment].[Class] = [inserted].[DocumentClass] AND
					[ServiceAssignment].[State] NOT IN (1)
			LEFT OUTER JOIN [deleted]
				ON [deleted].[DocumentClass] = [ServiceAssignment].[Class] AND [deleted].[DocumentID] = [ServiceAssignment].[ID]
		WHERE
			[deleted].[ID] IS NULL

		IF @invalid > 0
		BEGIN
			RAISERROR('#tfaUpdateInvalidJobState', 18, -1)
			ROLLBACK TRANSACTION
		END
		ELSE
		BEGIN
			UPDATE [ServiceAssignment]
			SET
				[ServiceAssignment].[ActualManHour] = [ServiceAssignment].[ActualManHour] + [Add].[Duration],
				[ServiceAssignment].[ActualBudget] = [ServiceAssignment].[ActualBudget] + [Add].[Cost]
			FROM
				[ServiceAssignment]
				INNER JOIN @ForAdd AS [Add]
					ON [ServiceAssignment].[ID] = [Add].[DocumentID] AND
						[ServiceAssignment].[Class] = [Add].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END

			UPDATE [ServiceAssignment]
			SET
				[ServiceAssignment].[ActualManHour] = [ServiceAssignment].[ActualManHour] - [Subtract].[Duration],
				[ServiceAssignment].[ActualBudget] = [ServiceAssignment].[ActualBudget] - [Subtract].[Cost]
			FROM
				[ServiceAssignment]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [ServiceAssignment].[ID] = [Subtract].[DocumentID] AND
						[ServiceAssignment].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
		END
		
	END
	-----------------------------------------------------------------------------
END
GO
