IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm5TimeEntryEmployeeCalendarBrowse')
	BEGIN
		PRINT 'Dropping Procedure pjm5TimeEntryEmployeeCalendarBrowse'
		DROP  Procedure  pjm5TimeEntryEmployeeCalendarBrowse
	END

GO

PRINT 'Creating Procedure pjm5TimeEntryEmployeeCalendarBrowse'
GO
/******************************************************************************
**		File: 
**		Name: pjm5TimeEntryEmployeeCalendarBrowse
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure pjm5TimeEntryEmployeeCalendarBrowse
	@EmployeeID TIdentifier,
	@PeriodStart TDate,
	@PeriodEnd TDate
AS
	SET NOCOUNT ON
	DECLARE @ViewAllFinance TBool, @CurrentEmployeeID TIdentifier
	
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance'))
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	;
	WITH [AllTimeEntries] ([Class], [ID], [State], [StartTime], [EndTime], [EmployeeID], [ProjectID], [Color], [Timestamp], [Guid]) AS 
	(
		SELECT 
			[TimeEntry].[Class],
			[TimeEntry].[ID], 
			[TimeEntry].[State],
			[TimeEntry].[StartTime], 
			DATEADD(minute, [TimeEntry].[Duration], [TimeEntry].[StartTime]) as [EndTime],
			[TimeEntry].[EmployeeID],
			[TimeEntry].[ProjectID],
			[TimeEntry].[Color],
			[TimeEntry].[Timestamp],
			[Class].[Guid]
		FROM
			[TimeEntry]
			INNER JOIN [Class]
				ON ([TimeEntry].[Class] = [Class].[Class])
		WHERE
			[TimeEntry].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd AND
			[TimeEntry].[EmployeeID] = @EmployeeID
	),
	[AllMeetingAttendees] ([Class], [ID], [State], [StartTime], [EndTime],  [EmployeeID], [ProjectID], [Color], [Timestamp], [Guid]) AS 
	(
		SELECT 
			[MeetingRequest].[Class],
			[MeetingRequest].[ID], 
			[MeetingRequest].[State],
			[MeetingRequest].[StartTime], 
			DATEADD(minute, [MeetingRequest].[Duration], [MeetingRequest].[StartTime]) as [EndTime],
			@EmployeeID,
			[MeetingRequest].[ProjectID],
			[MeetingRequest].[Color],
			[MeetingRequest].[Timestamp],
			[Class].[Guid]
		FROM
			[MeetingRequest]
			INNER JOIN [Class]
				ON ([MeetingRequest].[Class] = [Class].[Class])
		WHERE
		(
			[MeetingRequest].[OriginatorEmployeeID] = @employeeID AND [MeetingRequest].[State] NOT IN (3, 4)
			OR
			(EXISTS(SELECT * FROM [Attendee] WHERE [MeetingRequestID] = [MeetingRequest].[ID] AND [EmployeeID] = @employeeID AND ([ResponseType] IS NULL OR [ResponseType] IN ('Accept', 'Tentative'))) AND [MeetingRequest].[State] = 2)
		)
		AND
			[MeetingRequest].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd
	)
	SELECT
		[AllTimeEntries].[Class],
		[AllTimeEntries].[ID],
		[AllTimeEntries].[State],
		[TimeEntry].[FileAs],
		[AllTimeEntries].[StartTime],
		[AllTimeEntries].[EndTime],
		[TimeEntry].[ActivityTypeID],
		[TimeEntry].[ProjectID],
		[Project].[Number] as [ProjectCode],
		[TimeEntry].[TaskID],
		[Task].[FileAs] as [TaskName],
		[AllTimeEntries].[EmployeeID],
		[TimeEntry].[Billable],
		[AllTimeEntries].[Color],
		[TimeEntry].[Comments],
		[TimeEntry].[Created],
		[TimeEntry].[Modified],
		[TimeEntry].[CreatedBy],
		[TimeEntry].[ModifiedBy],
		[AllTimeEntries].[Timestamp],
		[AllTimeEntries].[Guid]
	FROM
		[AllTimeEntries]
		LEFT OUTER JOIN [TimeEntry]
			ON [TimeEntry].[ID] = [AllTimeEntries].[ID] AND
			(
				@ViewAllFinance = 1 OR 
				dbo.fnPjm5AllowCreateProjectEntry([AllTimeEntries].[ProjectID], @CurrentEmployeeID) = 1
			)
		LEFT OUTER JOIN [Project]
			ON [Project].[ID] = [TimeEntry].[ProjectID]
		LEFT OUTER JOIN [Task]
			ON [Task].[ID] = [TimeEntry].[TaskID]
	UNION ALL
	SELECT
		[AllMeetingAttendees].[Class],
		[AllMeetingAttendees].[ID],
		[AllMeetingAttendees].[State],
		[MeetingRequest].[FileAs],
		[AllMeetingAttendees].[StartTime],
		[AllMeetingAttendees].[EndTime],
		[MeetingRequest].[ActivityTypeID],
		[MeetingRequest].[ProjectID],
		[Project].[Number] as [ProjectCode],
		NULL,
		[MeetingRequest].[Location],
		[AllMeetingAttendees].[EmployeeID],
		0,
		[AllMeetingAttendees].[Color],
		[MeetingRequest].[Comments],
		[MeetingRequest].[Created],
		[MeetingRequest].[Modified],
		[MeetingRequest].[CreatedBy],
		[MeetingRequest].[ModifiedBy],
		[AllMeetingAttendees].[Timestamp],
		[AllMeetingAttendees].[Guid]
	FROM
		[AllMeetingAttendees]
		LEFT OUTER JOIN [MeetingRequest]
			ON [MeetingRequest].[ID] = [AllMeetingAttendees].[ID] AND
			(
				[MeetingRequest].[OriginatorEmployeeID] = @CurrentEmployeeID OR
				[AllMeetingAttendees].[ProjectID] IS NULL OR
				@ViewAllFinance = 1 OR 
				dbo.fnPjm5AllowCreateProjectEntry([AllMeetingAttendees].[ProjectID], @CurrentEmployeeID) = 1
			)
		LEFT OUTER JOIN [Project]
			ON [Project].[ID] = [MeetingRequest].[ProjectID]
		
	RETURN 0
GO

GRANT EXEC ON pjm5TimeEntryEmployeeCalendarBrowse TO PUBLIC
GO
  